/*  lvserial.h
 * 
 *  LVSERIAL Serial Port API
 *
 *  Copyright 2004-2014 Martin Henz
 *  All rights reserved.
 *
 *	E-Mail: martin.henz@mhst.de
 *	Web: http://www.mhst.de
 *
 *
 *  Redistribution and use in source and binary forms, with or without modification, 
 *  are permitted provided that the following conditions are met:
 *  
 *  * Redistributions of source code must retain the above copyright notice, 
 *    this list of conditions and the following disclaimer.
 *  
 *  * Redistributions in binary form must reproduce the above copyright notice, 
 *    this list of conditions and the following disclaimer in the documentation 
 *    and/or other materials provided with the distribution.
 *  
 *  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS 
 *  OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF 
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
 *  SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 *  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR 
 *  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
 *  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
 *  IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY 
 *  OF SUCH DAMAGE.
 * */

#ifndef LVSERIAL_CONST_ONLY

#include <windows.h>
#include <extcode.h>

#include "lv_error.h"

#ifdef LVSERIAL_EXPORTS
#define LVSERIAL_API __declspec(dllexport)
#else
#define LVSERIAL_API __declspec(dllimport)
#endif

#ifdef LVSERIAL_COMPILE_LIBRARY
#include "plist.h"
#include "pcre.h"

//internal data structure
typedef struct {
	HANDLE hC;							//the OS handle of the opened serial port
	char *pszPort;						//serial port name
	char *pszOriginalName;				//the given serial port name
	long lTeminationMode;				//read termination mode
	char *pcTermination;				//read termination characters
	long lNumberOfTermChar;				//number of characters in pcTermination
	pcre *RegEx;						//compiled regex (for termination mode COMM_TERM_REGEX)
	pcre_extra *RegExExtra;				//regex extra data (for termination mode COMM_TERM_REGEX)
	long lWriteMode;					//write mode
	char *pcStartOfTransmission;		//start of transmission characters
	long lNumberOfStartChars;			//number of characters in pcStartOfTransmission
	char *pcEndOfTransmission;			//end of transmission characters
	long lNumberOfEndChars;				//number of characters in pcEndOfTransmission
	unsigned long ulReadIntervalTimeout;		//read interval timeout
	unsigned long ulReadTotalTimeoutMultiplier; //read total timeout multiplier
	unsigned long ulReadTotalTimeoutConstant;	//read total timeout constant
	unsigned long ulWriteTotalTimeoutMultiplier;//write total timeout multiplier
	unsigned long ulWriteTotalTimeoutConstant;	//write total timeout constant
	unsigned long ulNOpened;			//number of CommOpenPort calls for this port
	HANDLE hLock;						//lock handle for external use
	HANDLE hReadLock;					//read lock handle for internal use
	HANDLE hWriteLock;					//write lock handle for internal use
	HANDLE hInternalLock;				//lock handle for internal use
	unsigned long ulUserData;			//user specified data
	char * pcUserData;					//user data buffer
	long lUserDataSize;					//size of user data buffer
	HANDLE hEvRead;						//read event handle
	HANDLE hEvWrite;					//write event handle
	OVERLAPPED osEvent;					//overlapped structure for comm event monitoring
	int	fWaitOnEvent;					//event monitoring actually waiting for an event
	unsigned long ulEventMask;			//location where to store an event
} COMM, *PCOMM;

#endif //#ifdef LVSERIAL_COMPILE_LIBRARY


//labview array of labview string handles
#ifndef LV_LSTRARRAY
#define LV_LSTRARRAY
//#pragma pack (1)
#include "lv_prolog.h"
typedef struct {
	int32 dimSize;
	LStrHandle elt[1];
	} LStrArray;
//#pragma pack()
#include "lv_epilog.h"
typedef LStrArray **LStrArrayHandle;
#endif


//labview array of unsigned long numbers
#ifndef LV_ULONGARRAY
#define LV_ULONGARRAY
//#pragma pack (1)
#include "lv_prolog.h"
typedef struct {
	int32 dimSize;
	uInt32 elt[1];
	} ULongArray;
//#pragma pack()
#include "lv_epilog.h"
typedef ULongArray **ULongArrayHandle;
#endif

#endif //#ifndef LVSERIAL_CONST_ONLY


//common serial port baud rates
#define COMM_BR_110             110
#define COMM_BR_300             300
#define COMM_BR_600             600
#define COMM_BR_1200            1200
#define COMM_BR_2400            2400
#define COMM_BR_4800            4800
#define COMM_BR_9600            9600
#define COMM_BR_14400           14400
#define COMM_BR_19200           19200
#define COMM_BR_38400           38400
#define COMM_BR_56000           56000
#define COMM_BR_57600           57600
#define COMM_BR_115200          115200
#define COMM_BR_128000          128000
#define COMM_BR_256000          256000


//serial port parity configuration
#define COMM_PARITY_NONE					0	// no parity
#define COMM_PARITY_ODD						1	// odd parity
#define COMM_PARITY_EVEN					2	// even parity
#define COMM_PARITY_MARK					3	// mark parity
#define COMM_PARITY_SPACE					4	// space parity


//serial port stop bit configuration
#define COMM_ONESTOPBIT						0	// one stop bit
#define COMM_ONE5STOPBITS					1   // 1.5 stop bits
#define COMM_TWOSTOPBITS					2	// two stop bits


//option flags for the CommFlush function
#define COMM_FLUSH_TXABORT					0x0001  // Kill the pending/current writes to the comm port.
#define COMM_FLUSH_RXABORT					0x0002  // Kill the pending/current reads to the comm port.
#define COMM_FLUSH_TXCLEAR					0x0004  // Kill the transmit queue if there.
#define COMM_FLUSH_RXCLEAR					0x0008  // Kill the typeahead buffer if there.


//serial port error flags
#define COMM_ERRF_RXOVER					0x0001  // Receive Queue overflow
#define COMM_ERRF_OVERRUN					0x0002  // Receive Overrun Error
#define COMM_ERRF_RXPARITY					0x0004  // Receive Parity Error
#define COMM_ERRF_FRAME						0x0008  // Receive Framing pError
#define COMM_ERRF_BREAK						0x0010  // Break Detected
#define COMM_ERRF_TXFULL					0x0100  // TX Queue is full
#define COMM_ERRF_PTO						0x0200  // LPTx Timeout
#define COMM_ERRF_IOE						0x0400  // LPTx I/O Error
#define COMM_ERRF_DNS						0x0800  // LPTx Device not selected
#define COMM_ERRF_OOP						0x1000  // LPTx Out-Of-Paper
#define COMM_ERRF_MODE						0x8000  // Requested mode unsupported


//serial port status flags

//Indicates whether transmission is waiting for the CTS (clear-to-send) 
//signal to be sent. If this flag is true, transmission is waiting.
#define COMM_STAT_CTS						0x0001

//Indicates whether transmission is waiting for the DSR (data-set-ready) 
//signal to be sent. If this flag is set, transmission is waiting.
#define COMM_STAT_DSR						0x0002

//Indicates whether transmission is waiting for the RLSD (receive-line-
//signal-detect) signal to be sent. If this flag is set, transmission is 
//waiting. 
#define COMM_STAT_RLSD						0x0004

//Indicates whether transmission is waiting because the XOFF character was 
//received. If this flag is set, transmission is waiting.
#define COMM_STAT_XOFFHOLD					0x0008

//Indicates whether transmission is waiting because the XOFF character was 
//transmitted. If this flag is set, transmission is waiting. Transmission 
//halts when the XOFF character is transmitted to a system that takes the 
//next character as XON, regardless of the actual character.
#define COMM_STAT_XOFFSENT					0x0010

//Indicates whether the end-of-file (EOF) character has been received. If 
//this flag is set, the EOF character has been received. This flag is only
//valid if a single termination character is specified.
#define COMM_STAT_EOF						0x0020

//If this flag is set, there is a character queued for transmission that 
//has come to the communications device by way of the CommTransmitChar 
//function. The communications device transmits such a character ahead of 
//other characters in the device's output buffer.
#define COMM_STAT_TXIM						0x0040


//serial port modem status flags
#define COMM_MSTAT_CTS_ON					((DWORD)0x0010) // CTS signaled
#define COMM_MSTAT_DSR_ON					((DWORD)0x0020)	// DSR signaled
#define COMM_MSTAT_RING_ON					((DWORD)0x0040) // RING signaled
#define COMM_MSTAT_RLSD_ON					((DWORD)0x0080) // RLSD signaled


//serial port flow control configuration
#define COMM_FLOW_NONE						0	// no flow control
#define COMM_FLOW_XONXOFF					1	// XON/XOFF
#define COMM_FLOW_RTSCTS					2	// RTS/CTS
#define COMM_FLOW_DTRDSR					3	// DTR/DSR


//serial port advanced control flags (used for CommControl)

//Indicates whether the CTS (clear-to-send) signal is monitored for output 
//flow control. If this flag is used and CTS is turned off, output is 
//suspended until CTS is sent again.
#define COMM_CTRL_OUTXCTSFLOW				0x00000001

//Indicates whether the DSR (data-set-ready) signal is monitored for 
//output flow control. If this flag is used and DSR is turned off, 
//output is suspended until DSR is sent again.
#define COMM_CTRL_OUTXDSRFLOW				0x00000002

//Enables the DTR line when the device is opened and leaves it on.
#define COMM_CTRL_DTRENABLE					0x00000004

//Enables DTR handshaking. If handshaking is enabled, it is an error 
//for the application to adjust the line.
#define COMM_CTRL_DTRHANDSHAKE				0x00000008

//Indicates whether the communications driver is sensitive to the 
//state of the DSR signal. If this flag is set, the driver ignores any 
//bytes received, unless the DSR modem input line is high. 
#define COMM_CTRL_DSRSENSITIVITY			0x00000010

//Indicates whether transmission stops when the input buffer is full and 
//the driver has transmitted the XoffChar character. If this member is 
//TRUE, transmission continues after the input buffer has come within 
//XoffLim bytes of being full and the driver has transmitted the XoffChar 
//character to stop receiving bytes. If this flag is set, transmission 
//does not continue until the input buffer is within XonLim bytes of being 
//empty and the driver has transmitted the XonChar character to resume 
//reception. 
#define COMM_CTRL_TXCONTINUEONXOFF			0x00000020

//Indicates whether XON/XOFF flow control is used during transmission. If 
//this flag is set, transmission stops when the XoffChar character is 
//received and starts again when the XonChar character is received. 
#define COMM_CTRL_OUTX						0x00000040

//Indicates whether XON/XOFF flow control is used during reception. If 
//this flag is used, the XoffChar character is sent when the input buffer 
//comes within XoffLim bytes of being full, and the XonChar character is 
//sent when the input buffer comes within XonLim bytes of being empty. 
#define COMM_CTRL_INX						0x00000080

//Indicates whether bytes received with parity errors are replaced with 
//the character specified by the ErrorChar member. If this flag is set 
//and the fParity flag is set, replacement occurs.
#define COMM_CTRL_ERRORCHAR					0x00000100

//Indicates whether null bytes are discarded. If this flag is set, null 
//bytes are discarded when received.
#define COMM_CTRL_NULL						0x00000200

//Enables the RTS line when the device is opened and leaves it on.
#define COMM_CTRL_RTSENABLE					0x00000400

//Enables RTS handshaking. The driver raises the RTS line when the 
//"type-ahead" (input) buffer is less than one-half full and lowers the 
//RTS line when the buffer is more than three-quarters full. If handshaking 
//is enabled, it is an error for the application to adjust the line.
#define COMM_CTRL_RTSHANDSHAKE				0x00000800

//Windows NT/2000/XP: Specifies that the RTS line will be high if bytes are 
//available for transmission. After all buffered bytes have been sent, the 
//RTS line will be low.
#define COMM_CTRL_RTSTOGGLE					0x00001000

//Indicates whether read and write operations are terminated if an error 
//occurs. If this flag is set, the driver terminates all read and write 
//operations with an error status if an error occurs.
#define COMM_CTRL_ABORTONERROR				0x00002000


//serial port read termination modes
#define COMM_TERM_NONE						0	//no termination
#define COMM_TERM_CHAR						1	//one or more characters
#define COMM_TERM_STRING					2	//termination string
#define COMM_TERM_REGEX						3	//regular expression
#define COMM_TERM_STRARRAY					4	//string array (not implemented)


//serial port write mode
#define COMM_WRITE_NORMAL					0	//normal write mode
#define COMM_WRITE_STX						1	//write start string
#define COMM_WRITE_ETX						2	//write end string
#define COMM_WRITE_STXETX                   (COMM_WRITE_STX | COMM_WRITE_ETX)


//communication events
#define COMM_EV_NONE						0x0000  // no event
#define COMM_EV_RXCHAR						0x0001  // Any Character received
#define COMM_EV_RXFLAG						0x0002  // Received certain character
#define COMM_EV_TXEMPTY						0x0004  // Transmitt Queue Empty
#define COMM_EV_CTS							0x0008  // CTS changed state
#define COMM_EV_DSR							0x0010  // DSR changed state
#define COMM_EV_RLSD						0x0020  // RLSD changed state
#define COMM_EV_BREAK						0x0040  // BREAK received
#define COMM_EV_ERR							0x0080  // Line status error occurred
#define COMM_EV_RING						0x0100  // Ring signal detected
#define COMM_EV_PERR						0x0200  // Printer error occured
#define COMM_EV_RX80FULL					0x0400  // Receive buffer is 80 percent full
#define COMM_EV_EVENT1						0x0800  // Provider specific event 1
#define COMM_EV_EVENT2					    0x1000  // Provider specific event 2


//serial port error codes
#define COMM_MIN_ERROR                      -65100	// min. error code
#define COMM_MAX_ERROR                      -65060	// max. error code

//The number of bytes transferred is equal to the specified maximum.
#define COMM_WARN_NYBTES					65060

//One of the termination characters received.
#define COMM_WARN_TERMCHAR					65061

//success
#define COMM_SUCCESS						0

//The number of bytes transferred is equal to the specified maximum.
#define COMM_ERR_NYBTES						-65060

//Termination character not received.
#define COMM_ERR_TERMCHAR					-65061

//Invalid length or byte count specified.
#define COMM_ERR_INVALIDLENGTH				-65062

//The given operation is not implemented.
#define COMM_ERR_NOTIMPLEMENTED				-65063

//The device is valid, but you cannot currently access it.
#define COMM_ERR_ACCESS						-65064

//An input buffer overflow has occurred. There is either no room in the 
//input buffer, or a character was received after the end-of-file (EOF) 
//character.
#define COMM_ERR_OVERFLOW					-65065

//Framing error occurred during transfer.
#define COMM_ERR_FRAMING					-65066

//Parity error occurred during transfer.
#define COMM_ERR_PARITY						-65067

//Timeout expired.
#define COMM_ERR_TIMEOUT					-65068

//The given device reference is invalid.
#define COMM_ERR_HANDLE						-65069

//Unknown system error.
#define COMM_ERR_SYSTEM						-65070

//Device not found.
#define COMM_ERR_RESOURCE					-65071

//Argument error.
#define COMM_ERR_ARGUMENT					-65072

//Out of memory.
#define COMM_ERR_OUTOFMEMORY				-65073

//Operation aborted.
#define COMM_ERR_ABORTED					-65074

//Unable to create the semaphore.
#define COMM_ERR_RESERVED1					-65075

//Can't lock the serial port (timeout).
#define COMM_ERR_LOCKTIMEOUT				-65076

//The serial port is not locked.
#define COMM_ERR_NOTLOCKED					-65077

//The system was unable to write the specified number of bytes.
#define COMM_ERR_WRITE						-65078

//A character-buffer overrun has occurred. The next character is lost.
#define COMM_ERR_OVERRUN					-65079

//The application tried to transmit a character, but the output buffer was full.
#define COMM_ERR_TXBUFFERFULL               -65080

//The received character is not equal to the transmitted character.
#define COMM_ERR_WRONGECHO					-65081

//The hardware detected a break condition.
#define COMM_ERR_BREAK						-65082

//A parallel device is not selected.
#define COMM_ERR_DNS						-65083

//An I/O error occurred during communications with the device.
#define COMM_ERR_IOE						-65084

//The requested mode is not supported, or the handle is invalid.
#define COMM_ERR_MODE						-65085

//A parallel device signaled that it is out of paper.
#define COMM_ERR_OOP						-65086

//A time-out occurred on a parallel device.
#define COMM_ERR_PTO						-65087

//The event handler is disabled.
#define COMM_ERR_NOEVENT        -65088


#ifndef LVSERIAL_CONST_ONLY

#if defined(__cplusplus) || defined(__cplusplus__)
   extern "C" {
#endif


//function prototypes

long LVSERIAL_API CommOpenPort(PHANDLE phComm, char * pszPort, int fDuplicate);
long LVSERIAL_API CommOpenPort2(PHANDLE phComm, char * pszPort, int fDuplicate, int *pfCreatedNew);
long LVSERIAL_API CommClosePort(HANDLE hComm);
long LVSERIAL_API CommClosePort2(HANDLE hComm, int fCloseAll);
long LVSERIAL_API CommClosePort3(HANDLE hComm, int fCloseAll, unsigned long ulTimeout);
long LVSERIAL_API CommCloseAll(void);
long LVSERIAL_API CommWrite(
	HANDLE hComm, 
	char * pcBuffer, 
	unsigned long ulBytesToWrite, 
	unsigned long * pulBytesWritten);
long LVSERIAL_API CommWriteAndReceiveEcho(
	HANDLE hComm, 
	char * pcWriteBuffer,
	unsigned long ulBytesToWrite, 
	unsigned long * pulBytesWritten,
	char * pcReadBuffer,
	unsigned long * pulBytesRead,
	long fStopIfNotEqual);
long LVSERIAL_API CommRead(
	HANDLE hComm, 
	char *pcBuffer, 
	unsigned long ulBytesToRead, 
	unsigned long * pulBytesRead);
long LVSERIAL_API CommBreak(HANDLE hComm, unsigned long ulMilliseconds);
long LVSERIAL_API CommBuffer(HANDLE hComm, unsigned long ulInQueueSize, unsigned long ulOutQueueSize);
long LVSERIAL_API CommFlush(HANDLE hComm, unsigned long ulFlags);
long LVSERIAL_API CommStatus(
	HANDLE hComm, 
	unsigned long * pulErrorFlags, 
	unsigned long * pulStatusFlags,
	unsigned long * pulInQueue,
	unsigned long * pulOutQueue);
long LVSERIAL_API CommBytesAtPort( HANDLE hComm, unsigned long * pulBytesAtPort);
long LVSERIAL_API CommModemStatus(HANDLE hComm, unsigned long * pulModemStatus);
long LVSERIAL_API CommGetCTS(HANDLE hComm, int *pfCTS);
long LVSERIAL_API CommGetDSR(HANDLE hComm, int *pfDSR);
long LVSERIAL_API CommGetRING(HANDLE hComm, int *pfRING);
long LVSERIAL_API CommGetRLSD(HANDLE hComm, int *pfRLSD);
long LVSERIAL_API CommSetXOFF(HANDLE hComm);
long LVSERIAL_API CommSetXON(HANDLE hComm);
long LVSERIAL_API CommSetRTS(HANDLE hComm);
long LVSERIAL_API CommClrRTS(HANDLE hComm);
long LVSERIAL_API CommSetDTR(HANDLE hComm);
long LVSERIAL_API CommClrDTR(HANDLE hComm);
long LVSERIAL_API CommSetBreak(HANDLE hComm);
long LVSERIAL_API CommClearBreak(HANDLE hComm);
long LVSERIAL_API CommConfig(
	HANDLE hComm, 
	unsigned long ulBaudRate,
	unsigned short usDataBits,
	unsigned short usParity,
	unsigned short usStopBits);
long LVSERIAL_API CommFlowControl(
	HANDLE hComm,
	unsigned short usFlowControl,
	unsigned char ucXOFFCharacter,
	unsigned char ucXONCharacter);
long LVSERIAL_API CommControl(
	HANDLE hComm,
	unsigned long lFlags,
	unsigned short usXonLim,
	unsigned short usXoffLim,
	unsigned char ucXonChar,
	unsigned char ucXoffChar,
	unsigned char ucErrorChar);
long LVSERIAL_API CommTermination(
	HANDLE hComm, 
	int fTermCharEnable,
	char *pcTermCharacter,
	long lNumberOfTermChar);
long LVSERIAL_API CommTermination2(
	HANDLE hComm, 
	long lTerminationMode,
	char *pcTermination,
	long lNumberOfTermChar);
long LVSERIAL_API CommWriteMode(
	HANDLE hComm, 
	long lWriteMode,
	char *pcStartOfTransmission,
	long lNumberofStartChars,
	char *pcEndOfTransmission,
	long lNumberOfEndChars);
long LVSERIAL_API CommTimeout(HANDLE hComm, unsigned long ulTimeout);
long LVSERIAL_API CommTimeout2(
	HANDLE hComm, 
	unsigned long ulReadIntervalTimeout,
	unsigned long ulReadTotalTimeoutMultiplier,
	unsigned long ulReadTotalTimeoutConstant,
	unsigned long ulWriteTotalTimeoutMultiplier,
	unsigned long ulWriteTotalTimeoutConstant);
long LVSERIAL_API CommOptions(HANDLE hComm, char *pszOptions);
long LVSERIAL_API CommLock(HANDLE hComm, unsigned long ulTimeout);
long LVSERIAL_API CommLock2(HANDLE hComm, unsigned long ulTimeout, int *pfLocked);
long LVSERIAL_API CommUnLock(HANDLE hComm);
long LVSERIAL_API CommUnLock2(HANDLE hComm, int fLocked);
long LVSERIAL_API CommSetUserData(HANDLE hComm, unsigned long ulUserData);
long LVSERIAL_API CommGetUserData(HANDLE hComm, unsigned long *pulUserData);
long LVSERIAL_API CommSetUserDataBuffer(HANDLE hComm, char *pcUserData, long lUserDataSize);
long LVSERIAL_API CommGetUserDataBuffer(HANDLE hComm, char *pcUserData, long *plUserDataSize);
long LVSERIAL_API CommListPorts(char ***ports);
void LVSERIAL_API CommFree(LPVOID ptr);
void LVSERIAL_API CommFreePortList(char **ports);
long LVSERIAL_API CommGetHandles(HANDLE **ppHandles, int *piSize);
long LVSERIAL_API CommGetPortName(HANDLE hComm, char **ppszName, char **ppszOriginal);
long LVSERIAL_API CommEnableEvent(HANDLE hComm, unsigned long ulEventMask);
long LVSERIAL_API CommWaitForEvent(HANDLE hComm, unsigned long ulTimeout, unsigned long *pulEventMask);
long LVSERIAL_API CommDisableEvent(HANDLE hComm);
long LVSERIAL_API CommSetEventChar(HANDLE hComm, unsigned char ucEventChar);
long LVSERIAL_API CommTransmitChar(HANDLE hComm, unsigned char ucChar);
long LVSERIAL_API CommSetDefaultConfig(char *pszPort, char *pszOptions);

/* -----------------------------------------------------------------------------------*/

void LVSERIAL_API lvCommOpenPort(
	PHANDLE phComm, 
	LStrHandle shPort, 
	LVBoolean *plfDuplicate, 
	pErrC pError);
void LVSERIAL_API lvCommOpenPort2(
	PHANDLE phComm, 
	LStrHandle shPort, 
	LVBoolean *plfDuplicate, 
	LVBoolean *plfCreatedNew, 
	pErrC pError);
void LVSERIAL_API lvCommClosePort(HANDLE hComm, pErrC pError);
void LVSERIAL_API lvCommClosePort2(HANDLE hComm, LVBoolean *plfCloseAll, pErrC pError);
void LVSERIAL_API lvCommClosePort3(HANDLE hComm, LVBoolean *plfCloseAll, unsigned long ulTimeout, pErrC pError);
void LVSERIAL_API lvCommCloseAll(pErrC pError);
void LVSERIAL_API lvCommWrite(
	HANDLE hComm, 
	LStrHandle shBuffer,
	unsigned long * pulBytesWritten, 
	pErrC pError);
void LVSERIAL_API lvCommWriteAndReceiveEcho(
	HANDLE hComm, 
	LStrHandle shWriteBuffer,
	unsigned long * pulBytesWritten,
	LStrHandle shReadBuffer,
	unsigned long * pulBytesRead,
	LVBoolean *plfStopIfNotEqual,
	pErrC pError);
void LVSERIAL_API lvCommRead(
	HANDLE hComm, 
	LStrHandle shBuffer, 
	unsigned long ulBytesToRead, 
	unsigned long * pulBytesRead, 
	pErrC pError);
void LVSERIAL_API lvCommBreak(HANDLE hComm, unsigned long ulMilliseconds, pErrC pError);
void LVSERIAL_API lvCommBuffer(HANDLE hComm, unsigned long ulInQueueSize, unsigned long ulOutQueueSize, pErrC pError);
void LVSERIAL_API lvCommFlush(HANDLE hComm, unsigned long ulFlags, pErrC pError);
void LVSERIAL_API lvCommStatus(
	HANDLE hComm, 
	unsigned long * pulErrorFlags, 
	unsigned long * pulStatusFlags,
	unsigned long * pulInQueue,
	unsigned long * pulOutQueue, 
	pErrC pError);
void LVSERIAL_API lvCommBytesAtPort( HANDLE hComm, unsigned long * pulBytesAtPort, pErrC pError);
void LVSERIAL_API lvCommModemStatus(HANDLE hComm, unsigned long * pulModemStatus, pErrC pError);
void LVSERIAL_API lvCommGetCTS(HANDLE hComm, LVBoolean *plfCTS, pErrC pError);
void LVSERIAL_API lvCommGetDSR(HANDLE hComm, LVBoolean *plfDSR, pErrC pError);
void LVSERIAL_API lvCommGetRING(HANDLE hComm, LVBoolean *plfRING, pErrC pError);
void LVSERIAL_API lvCommGetRLSD(HANDLE hComm, LVBoolean *plfRLSD, pErrC pError);
void LVSERIAL_API lvCommSetXOFF(HANDLE hComm, pErrC pError);
void LVSERIAL_API lvCommSetXON(HANDLE hComm, pErrC pError);
void LVSERIAL_API lvCommSetRTS(HANDLE hComm, pErrC pError);
void LVSERIAL_API lvCommClrRTS(HANDLE hComm, pErrC pError);
void LVSERIAL_API lvCommSetDTR(HANDLE hComm, pErrC pError);
void LVSERIAL_API lvCommClrDTR(HANDLE hComm, pErrC pError);
void LVSERIAL_API lvCommSetBreak(HANDLE hComm, pErrC pError);
void LVSERIAL_API lvCommClearBreak(HANDLE hComm, pErrC pError);
void LVSERIAL_API lvCommConfig(
	HANDLE hComm, 
	unsigned long ulBaudRate,
	unsigned short usDataBits,
	unsigned short usParity,
	unsigned short usStopBits, 
	pErrC pError);
void LVSERIAL_API lvCommFlowControl(
	HANDLE hComm,
	unsigned short usFlowControl,
	unsigned char ucXOFFCharacter,
	unsigned char ucXONCharacter, 
	pErrC pError);
void LVSERIAL_API lvCommControl(
	HANDLE hComm,
	unsigned long lFlags,
	unsigned short usXonLim,
	unsigned short usXoffLim,
	unsigned char ucXonChar,
	unsigned char ucXoffChar,
	unsigned char ucErrorChar,
	pErrC pError);
void LVSERIAL_API lvCommTermination(
	HANDLE hComm, 
	LVBoolean *plfTermCharEnable,
	LStrHandle shTermCharacter,
	pErrC pError);
void LVSERIAL_API lvCommTermination2(
	HANDLE hComm, 
	long lTerminationMode,
	LStrHandle shTermination,
	pErrC pError);
void LVSERIAL_API lvCommWriteMode(
	HANDLE hComm, 
	long lWriteMode,
	LStrHandle shStartOfTransmission,
	LStrHandle shEndOfTransmission,
	pErrC pError);
void LVSERIAL_API lvCommTimeout(
	HANDLE hComm, 
	unsigned long ulTimeout, 
	pErrC pError);
void LVSERIAL_API lvCommTimeout2(
	HANDLE hComm, 
	unsigned long ulReadIntervalTimeout,
	unsigned long ulReadTotalTimeoutMultiplier,
	unsigned long ulReadTotalTimeoutConstant,
	unsigned long ulWriteTotalTimeoutMultiplier,
	unsigned long ulWriteTotalTimeoutConstant,
	pErrC pError);
void LVSERIAL_API lvCommOptions(HANDLE hComm, LStrHandle shOptions, pErrC pError);
void LVSERIAL_API lvCommLock(HANDLE hComm, unsigned long ulTimeout, pErrC pError);
void LVSERIAL_API lvCommLock2(HANDLE hComm, unsigned long ulTimeout, LVBoolean *pfLocked, pErrC pError);
void LVSERIAL_API lvCommUnLock(HANDLE hComm, pErrC pError);
void LVSERIAL_API lvCommUnLock2(HANDLE hComm, LVBoolean *pfLocked, pErrC pError);
void LVSERIAL_API lvCommSetUserData(HANDLE hComm, unsigned long ulUserData, pErrC pError);
void LVSERIAL_API lvCommGetUserData(HANDLE hComm, unsigned long *pulUserData, pErrC pError);
void LVSERIAL_API lvCommSetUserDataBuffer(HANDLE hComm, LStrHandle shUserData, pErrC pError);
void LVSERIAL_API lvCommGetUserDataBuffer(HANDLE hComm, LStrHandle shUserData, pErrC pError);
void LVSERIAL_API lvCommListPorts(LStrArrayHandle *phalsPorts, pErrC pError);
void LVSERIAL_API lvCommGetHandles(ULongArrayHandle *phaulHandles, pErrC pError);
void LVSERIAL_API lvCommGetPortName(HANDLE hComm, LStrHandle shName, LStrHandle shOriginal, pErrC pError);
void LVSERIAL_API lvCommEnableEvent(HANDLE hComm, unsigned long ulEventMask, pErrC pError);
void LVSERIAL_API lvCommWaitOnEvent(HANDLE hComm, unsigned long *pulEventMask, pErrC pError);
void LVSERIAL_API lvCommEnableEvent(HANDLE hComm, unsigned long ulEventMask, pErrC pError);
void LVSERIAL_API lvCommWaitForEvent(HANDLE hComm, unsigned long ulTimeout, unsigned long *pulEventMask, pErrC pError);
void LVSERIAL_API lvCommDisableEvent(HANDLE hComm, pErrC pError);
void LVSERIAL_API lvCommSetEventChar(HANDLE hComm, unsigned char ucEventChar, pErrC pError);
void LVSERIAL_API lvCommTransmitChar(HANDLE hComm, unsigned char ucChar, pErrC pError);
void LVSERIAL_API lvCommSetDefaultConfig(LStrHandle shPort, LStrHandle shOptions, pErrC pError);

/* -----------------------------------------------------------------------------------*/

void LVSERIAL_API ogCommOpenPort(PHANDLE phComm, LStrHandle shPort, LVBoolean *plfDuplicate, pErrC pError);
void LVSERIAL_API ogCommClosePort(PHANDLE phComm, pErrC pError);
void LVSERIAL_API ogCommClosePort2(PHANDLE phComm, LVBoolean *plfCloseAll, pErrC pError);
void LVSERIAL_API ogCommClosePort3(PHANDLE phComm, LVBoolean *plfCloseAll, unsigned long ulTimeout, pErrC pError);
void LVSERIAL_API ogCommCloseAll(pErrC pError);
void LVSERIAL_API ogCommWrite(
	PHANDLE phComm, 
	LStrHandle shBuffer,
	unsigned long * pulBytesWritten, 
	pErrC pError);
void LVSERIAL_API ogCommWriteAndReceiveEcho(
	PHANDLE phComm, 
	LStrHandle shWriteBuffer,
	unsigned long * pulBytesWritten,
	LStrHandle shReadBuffer,
	unsigned long * pulBytesRead,
	LVBoolean *plfStopIfNotEqual,
	pErrC pError);
void LVSERIAL_API ogCommRead(
	PHANDLE phComm, 
	LStrHandle shBuffer, 
	unsigned long ulBytesToRead, 
	unsigned long * pulBytesRead, 
	pErrC pError);
void LVSERIAL_API ogCommBreak(PHANDLE phComm, unsigned long ulMilliseconds, pErrC pError);
void LVSERIAL_API ogCommBuffer(PHANDLE phComm, unsigned long ulInQueueSize, unsigned long ulOutQueueSize, pErrC pError);
void LVSERIAL_API ogCommFlush(PHANDLE phComm, unsigned long ulFlags, pErrC pError);
void LVSERIAL_API ogCommStatus(
	PHANDLE phComm, 
	unsigned long * pulErrorFlags, 
	unsigned long * pulStatusFlags,
	unsigned long * pulInQueue,
	unsigned long * pulOutQueue, 
	pErrC pError);
void LVSERIAL_API ogCommBytesAtPort(PHANDLE phComm, unsigned long * pulBytesAtPort, pErrC pError);
void LVSERIAL_API ogCommModemStatus(PHANDLE phComm, unsigned long * pulModemStatus, pErrC pError);
void LVSERIAL_API ogCommGetCTS(PHANDLE phComm, LVBoolean *plfCTS, pErrC pError);
void LVSERIAL_API ogCommGetDSR(PHANDLE phComm, LVBoolean *plfDSR, pErrC pError);
void LVSERIAL_API ogCommGetRING(PHANDLE phComm, LVBoolean *plfRING, pErrC pError);
void LVSERIAL_API ogCommGetRLSD(PHANDLE phComm, LVBoolean *plfRLSD, pErrC pError);
void LVSERIAL_API ogCommSetXOFF(PHANDLE phComm, pErrC pError);
void LVSERIAL_API ogCommSetXON(PHANDLE phComm, pErrC pError);
void LVSERIAL_API ogCommSetRTS(PHANDLE phComm, pErrC pError);
void LVSERIAL_API ogCommClrRTS(PHANDLE phComm, pErrC pError);
void LVSERIAL_API ogCommSetDTR(PHANDLE phComm, pErrC pError);
void LVSERIAL_API ogCommClrDTR(PHANDLE phComm, pErrC pError);
void LVSERIAL_API ogCommSetBreak(PHANDLE phComm, pErrC pError);
void LVSERIAL_API ogCommClearBreak(PHANDLE phComm, pErrC pError);
void LVSERIAL_API ogCommConfig(
	PHANDLE phComm,
	unsigned long ulBaudRate,
	unsigned short usDataBits,
	unsigned short usParity,
	unsigned short usStopBits, 
	pErrC pError);
void LVSERIAL_API ogCommFlowControl(
	PHANDLE phComm,
	unsigned short usFlowControl,
	unsigned char ucXOFFCharacter,
	unsigned char ucXONCharacter, 
	pErrC pError);
void LVSERIAL_API ogCommControl(
	PHANDLE phComm,
	unsigned long lFlags,
	unsigned short usXonLim,
	unsigned short usXoffLim,
	unsigned char ucXonChar,
	unsigned char ucXoffChar,
	unsigned char ucErrorChar,
	pErrC pError);
void LVSERIAL_API ogCommTermination(
	PHANDLE phComm, 
	LVBoolean *plfTermCharEnable,
	LStrHandle shTermCharacter,
	pErrC pError);
void LVSERIAL_API ogCommTermination2(
	PHANDLE phComm, 
	long lTerminationMode,
	LStrHandle shTermination,
	pErrC pError);
void LVSERIAL_API ogCommWriteMode(
	PHANDLE phComm, 
	long lWriteMode,
	LStrHandle shStartOfTransmission,
	LStrHandle shEndOfTransmission,
	pErrC pError);
void LVSERIAL_API ogCommTimeout(
	PHANDLE phComm, 
	unsigned long ulTimeout, 
	pErrC pError);
void LVSERIAL_API ogCommTimeout2(
	PHANDLE phComm, 
	unsigned long ulReadIntervalTimeout,
	unsigned long ulReadTotalTimeoutMultiplier,
	unsigned long ulReadTotalTimeoutConstant,
	unsigned long ulWriteTotalTimeoutMultiplier,
	unsigned long ulWriteTotalTimeoutConstant,
	pErrC pError);
void LVSERIAL_API ogCommOptions(PHANDLE phComm, LStrHandle shOptions, pErrC pError);
void LVSERIAL_API ogCommLock(PHANDLE phComm, unsigned long ulTimeout, pErrC pError);
void LVSERIAL_API ogCommUnLock(PHANDLE phComm, pErrC pError);
void LVSERIAL_API ogCommSetUserData(PHANDLE phComm, unsigned long ulUserData, pErrC pError);
void LVSERIAL_API ogCommGetUserData(PHANDLE phComm, unsigned long *pulUserData, pErrC pError);
void LVSERIAL_API ogCommSetUserDataBuffer(PHANDLE phComm, LStrHandle shUserData, pErrC pError);
void LVSERIAL_API ogCommGetUserDataBuffer(PHANDLE phComm, LStrHandle shUserData, pErrC pError);
void LVSERIAL_API ogCommListPorts(LStrArrayHandle *phalsPorts, pErrC pError);
void LVSERIAL_API ogCommGetHandles(ULongArrayHandle hapulHandles, pErrC pError);
void LVSERIAL_API ogCommGetPortName(PHANDLE phComm, LStrHandle shName, LStrHandle shOriginal, pErrC pError);
void LVSERIAL_API ogCommEnableEvent(PHANDLE phComm, unsigned long ulEventMask, pErrC pError);
void LVSERIAL_API ogCommWaitOnEvent(PHANDLE phComm, unsigned long *pulEventMask, pErrC pError);
void LVSERIAL_API ogCommEnableEvent(PHANDLE phComm, unsigned long ulEventMask, pErrC pError);
void LVSERIAL_API ogCommWaitForEvent(PHANDLE phComm, unsigned long ulTimeout, unsigned long *pulEventMask, pErrC pError);
void LVSERIAL_API ogCommDisableEvent(PHANDLE phComm, pErrC pError);
void LVSERIAL_API ogCommSetEventChar(PHANDLE phComm, unsigned char ucEventChar, pErrC pError);
void LVSERIAL_API ogCommTransmitChar(PHANDLE phComm, unsigned char ucChar, pErrC pError);
void LVSERIAL_API ogCommSetDefaultConfig(LStrHandle shPort, LStrHandle shOptions, pErrC pError);

#if defined(__cplusplus) || defined(__cplusplus__)
   }
#endif

#endif // #ifndef LVSERIAL_CONST_ONLY
