/*  lvserial.c
 * 
 *  LVSERIAL Serial Port API
 *
 *  Copyright 2004-2014 Martin Henz
 *  All rights reserved.
 *
 *	E-Mail: martin.henz@mhst.de
 *	Web: http://www.mhst.de
 *
 *
 *  Redistribution and use in source and binary forms, with or without modification, 
 *  are permitted provided that the following conditions are met:
 *  
 *  * Redistributions of source code must retain the above copyright notice, 
 *    this list of conditions and the following disclaimer.
 *  
 *  * Redistributions in binary form must reproduce the above copyright notice, 
 *    this list of conditions and the following disclaimer in the documentation 
 *    and/or other materials provided with the distribution.
 *  
 *  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS 
 *  OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF 
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
 *  SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 *  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR 
 *  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
 *  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
 *  IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY 
 *  OF SUCH DAMAGE.
 * */

#define _WIN32_WINNT 0x0500

#include <windows.h>
#include <stdio.h>
#include <objbase.h>
#include <crtdbg.h>
#include <time.h>
#include <limits.h>

#define LVSERIAL_COMPILE_LIBRARY
#include "lvserial.h"
#include "lv_error.h"
#include "pcre.h"

//list of all open serial ports
PLIST gpCommList = NULL;
HANDLE ghLock = NULL;

//default buffer size (these are the system defaults)
#define LVCOMM_DEFAULT_RX_BUFFER	4096
#define LVCOMM_DEFAULT_TX_BUFFER	4096

//function prototypes
__inline long __CommErrorFlagToCode(unsigned long ulErrFlags);
__inline long __CommClearError(PCOMM pComm, long ulDefaultError);

//helper function for the serial port read and write operations. Before these
//functions can be used the parameters should be validated, the serial port
//should be locked, so nobody else can access the port.
__inline long __CommRead(
	PCOMM pComm, 
	char * pcBuffer, 
	unsigned long ulBytesToRead, 
	unsigned long * pulBytesRead,
	OVERLAPPED osReader,
	unsigned long ulLocalTimeout);

__inline long __CommWrite(
	PCOMM pComm, 
	char * pcBuffer, 
	unsigned long ulBytesToWrite, 
	unsigned long *pulBytesWritten,
	OVERLAPPED osWrite);

__inline long __CommWriteAndReceiveEcho(
	PCOMM pComm,
	char *pcWriteBuffer,
	unsigned long ulByteToWrite,
	char *pcReadBuffer,
	int fStopIfNotEqual,
	unsigned long *pulTxCount,
	unsigned long *pulRxCount,
	OVERLAPPED osWrite,
	OVERLAPPED osReader);

//close serial port base function
long __CommClosePort(HANDLE hComm, int fCloseAll, int fLock, unsigned long ulTimeout);

//helper functions for the internal locking mechanism.
__inline HANDLE __CommCreateLock(void);
__inline long __CommLock(HANDLE hLock);
__inline long __CommLockT(HANDLE hLock, unsigned long ulTimeout);
__inline long __CommLockMT(HANDLE *lphLock, unsigned long ulCount, unsigned long ulTimeout);
__inline long __CommUnLock(HANDLE hLock);

//this function splits the pszPort (with optional serial port configuration 
//options into the serial port name and the configuration string. This function
//also changes any ASRL port name to the OS default port name (COMx)
__inline long __CommSplitPortAndOptions(char *pszPort, char **ppszRealPort, char **ppszOptions);

//serial port compare function. Uses to detect an already opened serial port.
int ElmCmpPort( char *pszPort, PCOMM pComm );

//helper function to change a lv string array size to zero
void __lvDisposeStringArrayElements(LStrArrayHandle *phalsArray, pErrC pError);

/*  ElmCmpPort
 *
 *  Compares the serial port string with the serial port name inside a COMM structure
 *
 *  parameter
 *		pszPort				pointer to the null terminated port string
 *		pComm				pointer to the COMM structure
 *		
 *  return
 *		0 if the port name matches
 */
int ElmCmpPort( char *pszPort, PCOMM pComm )
{
	//pointer tests
	_ASSERT(pszPort);
	_ASSERT(pComm->pszPort);
	_ASSERT(pComm);

	//compare the serial port names
	return _stricmp( pszPort, pComm->pszPort );
}

/*  DllMain
 *
 *  DLL main function
 *
 *  return
 *      1 if there is no error and 
 *		0 if an error occurred
 */
int __stdcall DllMain (HINSTANCE hinstDLL, DWORD fdwReason, LPVOID lpvReserved)
{
	switch (fdwReason)
	{
		case DLL_PROCESS_ATTACH:
				  
			//create the comm device list
			if ( (gpCommList = ListCreate()) == 0)
				return 0;     // out of memory

			//create the global lock
			if ( (ghLock = __CommCreateLock()) == NULL)
				return 0;

			break;
		
		case DLL_PROCESS_DETACH:
			//close all open serial ports and dispose the comm device list
			CommCloseAll();

			__CommLock(ghLock);
			ListDispose (gpCommList);
			gpCommList = NULL;
			__CommUnLock(ghLock);

			//delete the global lock
			CloseHandle(ghLock);
			ghLock = 0L;

			break;
		case DLL_THREAD_ATTACH:
			break;

		case DLL_THREAD_DETACH:
			break;
	}
	
	return 1;
}

/*  __CommCleateLock
 *
 *  This function creates a internal lock object. 
 *
 *	The function creates a unnamed semaphore with an max. and initial count
 *	of one. A critical secion or mutex object is not a working solution, because
 *	for this object the same thread is allowed to get the object multiple times.
 *
 *  return
 *		lock handle or NULL
 */
__inline HANDLE __CommCreateLock(void)
{
#ifdef _DEBUG
	HANDLE hLock;

	//create the sempaphore for the internal locking
	if ((hLock = CreateSemaphore(NULL, 1, 1, NULL)) == NULL)
		//in case of an error generate a debug report
		_ASSERT(0);
	return hLock;
#else
	return CreateSemaphore(NULL, 1, 1, NULL);
#endif
}

/*  __CommLock
 *
 *  This function locks the handle. This function wait infinite time.
 *
 *	Because the situations, when this function is used, the handle is
 *	locked only for a very short time. To prevent the system to hang,
 *	this function uses a timeout of 60s. This is subject of change
 *	in future releases.
 *
 *  parameter
 *		hLock						lock handle
 *		
 *  return
 *		error code
 */
__inline long __CommLock(HANDLE hLock)
{
	//get the semaphore. Normally, we can wait infinite, because the
	//lock should last only a view milliseconds. But for now, we set 
	//a timeout of 60s
	switch (WaitForSingleObject(hLock, 60000))
	{
		case WAIT_OBJECT_0:
			//Ownership of the object is granted.
		case WAIT_ABANDONED:
			//object that was not released by the thread that owned the mutex object 
			//before the owning thread terminated. Ownership of the object is granted.
			break;

		case WAIT_TIMEOUT:
			//wait operation timed out
			//because this is not a normal situation, we generate a debug report
			_ASSERT(0);
			return COMM_ERR_LOCKTIMEOUT;

		case WAIT_FAILED:
			//function failed, call GetLastError() to get more information
			_ASSERT(0);
			return COMM_ERR_SYSTEM;

		default:
			//huh ... something wrong
			_ASSERT(0);
			return COMM_ERR_SYSTEM;
	}

	return COMM_SUCCESS;
}

/*  __CommLockT
 *
 *  This function locks the handle. This function returns if the
 *	specified object is in signaled state or the timeout interval elapsed.
 *
 *  parameter
 *		hLock						lock handle
 *		ulTimeout					millisecond timeout value
 *		
 *  return
 *		error code
 */
__inline long __CommLockT(HANDLE hLock, unsigned long ulTimeout)
{
	//get the semaphore.
	switch (WaitForSingleObject(hLock, ulTimeout))
	{
		case WAIT_OBJECT_0:
			//Ownership of the object is granted.
		case WAIT_ABANDONED:
			//object that was not released by the thread that owned the mutex object 
			//before the owning thread terminated. Ownership of the object is granted.
			break;

		case WAIT_TIMEOUT:
			//wait operation timed out
			//because this is not a normal situation, we generate a debug report
			_ASSERT(0);
			return COMM_ERR_LOCKTIMEOUT;

		case WAIT_FAILED:
			//function failed, call GetLastError() to get more information
			_ASSERT(0);
			return COMM_ERR_SYSTEM;

		default:
			//huh ... something wrong
			_ASSERT(0);
			return COMM_ERR_SYSTEM;
	}

	return COMM_SUCCESS;
}

/*  __CommLockMT
 *
 *  This function locks multiple handles. This function returns if all
 *	specified objects are in signaled state or the timeout interval elapsed.
 *
 *  parameter
 *		lphLock						array of lock handle
 *		ulCount						number of lock handles in lphLock
 *		ulTimeout					millisecond timeout value
 *		
 *  return
 *		error code
 */
__inline long __CommLockMT(HANDLE *lphLock, unsigned long ulCount, unsigned long ulTimeout)
{
	//get the semaphores.
	switch (WaitForMultipleObjects(ulCount, lphLock, TRUE, ulTimeout))
	{
		case WAIT_OBJECT_0:
			//Ownership of the object is granted.
		case WAIT_ABANDONED:
			//object that was not released by the thread that owned the mutex object 
			//before the owning thread terminated. Ownership of the object is granted.
			break;

		case WAIT_TIMEOUT:
			//wait operation timed out
			//because this is not a normal situation, we generate a debug report
			_ASSERT(0);
			return COMM_ERR_LOCKTIMEOUT;

		case WAIT_FAILED:
			//function failed, call GetLastError() to get more information
			_ASSERT(0);
			return COMM_ERR_SYSTEM;

		default:
			//huh ... something wrong
			_ASSERT(0);
			return COMM_ERR_SYSTEM;
	}

	return COMM_SUCCESS;
}

/*  __CommUnLock
 *
 *  This function unlocks the handle
 *
 *  parameter
 *		hLock						lock handle
 *		
 *  return
 *		error code
 */
__inline long __CommUnLock(HANDLE hLock)
{
	//release the semaphore
	if (!ReleaseSemaphore(hLock, 1, NULL))
	{
		if (GetLastError() == ERROR_TOO_MANY_POSTS)
		{
			_ASSERT(0);
			return COMM_ERR_NOTLOCKED;
		}
		else
		{
			_ASSERT(0);
			return COMM_ERR_SYSTEM;
		}
	}

	return COMM_SUCCESS;
}

/*  __CommDeleteLock
 *
 *  This function deletes the handle
 *
 *  parameter
 *		hLock						lock handle
 *		
 *  return
 *		error code
 */
__inline long __CommDeleteLock(HANDLE hLock)
{
	//close the handle
	if (!CloseHandle(hLock))
	{
		//error occurred, so we generate a debug report
		_ASSERT(0);
		return COMM_ERR_SYSTEM;
	}

	return COMM_SUCCESS;
}

/*  __CommErrorFlagToCode
 *
 *  This function translates the error flags (got from ClearCommError) into a 
 *	valid COMM_ERR_ error code.
 *
 *  parameter
 *		ulErrFlags			error flags
 *		
 *  return
 *		error code
 */
__inline long __CommErrorFlagToCode(unsigned long ulErrFlags)
{
	long lResult;

	//We speed up things, for normal situations, where no error occurred:
	//If there are no flags set, we do not test each flag to find out which one
	//is set.
	if (ulErrFlags != 0)
	{
		//one or more flags are set. We are searching for only one
		//flag. The most important must be tested at first.
		if (ulErrFlags & COMM_ERRF_FRAME)
			lResult = COMM_ERR_FRAMING;
		else if (ulErrFlags & COMM_ERRF_RXPARITY)
			lResult = COMM_ERR_PARITY;
		else if (ulErrFlags & COMM_ERRF_OVERRUN)
			lResult = COMM_ERR_OVERRUN;
		else if (ulErrFlags & COMM_ERRF_RXOVER)
			lResult = COMM_ERR_OVERFLOW;
		else if (ulErrFlags & COMM_ERRF_BREAK)
			lResult = COMM_ERR_BREAK;
		else if (ulErrFlags & COMM_ERR_TXBUFFERFULL)
			lResult = COMM_ERR_TXBUFFERFULL;
		else if (ulErrFlags & COMM_ERRF_PTO)
			lResult = COMM_ERR_PTO;
		else if (ulErrFlags & COMM_ERRF_DNS)
			lResult = COMM_ERR_DNS;
		else if (ulErrFlags & COMM_ERRF_OOP)
			lResult = COMM_ERR_OOP;
		else if (ulErrFlags & COMM_ERRF_MODE)
			lResult = COMM_ERR_MODE;
		else if (ulErrFlags & COMM_ERRF_IOE)
			lResult = COMM_ERR_IOE;
		else
			lResult = COMM_ERR_IOE;
	}
	else
		//none of the flags is set
		lResult = COMM_SUCCESS;

	return lResult;
}

/*  __CommClearError
 *
 *  This function calls ClearCommError and if there was an error it returns 
 *	the appropriate error code. If there is no error the function returns the
 *	default error code.
 *
 *  parameter
 *		pComm				pointer to the serial port data structure
 *		ulDefaultError		default error code
 *		
 *  return
 *		error code
 */
__inline long __CommClearError(PCOMM pComm, long lDefaultError)
{
	unsigned long ulErrFlags;
	long lResult;
	long lErr;

	//remember the last error
	lErr = GetLastError();

	//clear the error and get the error flags
	if (!ClearCommError(pComm->hC, &ulErrFlags,NULL))
	{
		//normally, this operation does not fail, so we generate a debug report
		//Note, that there are a view situations, where this can occur, e.g. if
		//the serial port is closed while we execute this function.
		_ASSERT(0);
		return COMM_ERR_SYSTEM;
	}
	else if (ulErrFlags != 0)
	{
		//one or more flags are set:
		//translate the error flags into a valid error code
		lResult = __CommErrorFlagToCode(ulErrFlags);
	}
	else
	{
		//no error flag is set: return the default error and
		//retore the last error code
		lResult = lDefaultError;
		SetLastError(lErr);
	}

	return lResult;
}

/*  __CommSplitPortAndOptions
 *
 *  Splits the serial port name and the specified options for the serial port.
 *
 *  parameter
 *		pszPort				complete serial port name including the options
 *		ppszRealPort		the real serial port name
 *		ppszOptions			the options provided in the pszPort string
 *		
 *  return
 *		error code
 */
__inline long __CommSplitPortAndOptions(
	char *pszPort, 
	char **ppszRealPort, 
	char **ppszOptions)
{
	size_t uiPortLen;					//length of the given port name (including the options
	size_t uiPortEnd;					//position of the end of the serial port name (where the options are beginning)
	char *pszRealPort;					//pointer to the real serial port name (temporary data)
	int iPortNumber;					//number of the port (used for ASRL name conversion)

	_ASSERT(ppszRealPort);
	_ASSERT(ppszOptions);

	//if the serial port name starts with the correct name like "\\.\COM1",
	//we just ignore the beginning "\\.\" 
	if (strncmp("\\\\.\\", pszPort, 4) == 0)
		pszPort +=4;

	//get the total length of the given port name (including the options)
	uiPortLen = strlen(pszPort);

	//the serial port name ends at the first whitespace character.
	uiPortEnd = strcspn(pszPort, " \t;,");

	//allocate the buffer for the real port name and copy the given
	//port name (without the options) into this buffer.
	if ((pszRealPort = (char *)malloc(uiPortEnd+1)) == NULL)
	{
		*ppszRealPort = *ppszOptions = NULL;
		return COMM_ERR_OUTOFMEMORY;
	}

	strncpy(pszRealPort, pszPort, uiPortEnd);
	*(pszRealPort+uiPortEnd) = '\0';

	//if there are any additional options, the uiPortEnd is not equal 
	//to uiPortLength.
	if (uiPortEnd != uiPortLen)
	{
		//additional options are available: We allocate the buffer for the
		//options string and copy the options into this buffer.
		if ((*ppszOptions = (char *)malloc(strlen(pszPort) - uiPortEnd +1)) == NULL)
		{
			*ppszRealPort = NULL;
			return COMM_ERR_OUTOFMEMORY;
		}
		strcpy(*ppszOptions, pszPort+uiPortEnd);
	}
	else
		//no additional options are specified: There is nothing
		//to copy to the options string. We set it to NULL.
		*ppszOptions = NULL;

	/* Instead of using the normal serial port name of the operating
	 * system, the user can also use the VISA like ASRL name. If so,
	 * we have to change it to the operating systems name.
	 * Note, that the real operating systems serial port name is 
	 * something like "\\.\COM1" and we have to change any COM name
	 * to this syntax. This does not change anything for ports below
	 * COM10, but COM10 and higher ports will only work if we use the
	 * complete serial port name. This is the reason, why we allways
	 * use the complete and correct name.
	 */
	if ( _strnicmp(pszRealPort, "ASRL", 4) == 0)
	{
		//serial port name starts with "ASRL"

		//Change the VISA like ASRL name to the real serial port name.

		//30 bytes are more than enough for a serial port name "COMx"
		if ( (*ppszRealPort = (char *)malloc(30)) == NULL)
		{
			if (*ppszOptions != NULL)
				free(*ppszOptions);
			return COMM_ERR_OUTOFMEMORY;
		}

		iPortNumber = atoi(pszRealPort+4);
		strcpy(*ppszRealPort,"\\\\.\\COM");
		_itoa(iPortNumber, *ppszRealPort+7, 10);
	}
	else
	{
		//normal serial port name given.
		if ( (*ppszRealPort = (char *)malloc(5 + strlen(pszRealPort))) == NULL)
		{
			if (*ppszOptions != NULL)
				free(*ppszOptions);
			return COMM_ERR_OUTOFMEMORY;
		}

		strcpy(*ppszRealPort ,"\\\\.\\");
		strcat(*ppszRealPort+4, pszRealPort);
	}

	free(pszRealPort);

	return COMM_SUCCESS;
}

/*  CommOpenPort
 *
 *  Opens the serial port
 *
 *  parameter
 *		phComm				pointer where the function stores the serial port handle
 *		pszPort				name of the serial port to open
 *		fDuplicate			if this value is true, the function returns a duplicate 
 *							handle if the port is already opened.
 *		
 *  return
 *		error code
 */
long LVSERIAL_API CommOpenPort(PHANDLE phComm, char * pszPort, int fDuplicate )
{
	int fCreatedNew;

	return CommOpenPort2(phComm, pszPort, fDuplicate, &fCreatedNew );
}

/*  CommOpenPort2
 *
 *  Opens the serial port
 *
 *  parameter
 *		phComm				pointer where the function stores the serial port handle
 *		pszPort				name of the serial port to open
 *		fDuplicate			if this value is true, the function returns a duplicate 
 *						handle if the port is already opened.
 *		pfCreatedNew			will be set to true if this function has opened a new handle
 *		
 *  return
 *		error code
 */
long LVSERIAL_API CommOpenPort2(PHANDLE phComm, char * pszPort, int fDuplicate, int *pfCreatedNew )
{
	long lResult;						//function return code
	PCOMM pComm;						//pointer to the serial port COMM structure
	char *pszPortName;					//the real serial port name
	char *pszOptions;					//the options (configuration) string
	DCB dcb;							//DCB structure

	_ASSERT(gpCommList);
	_ASSERT(pfCreatedNew);

	//set the pfCreatedNew to a valid value
	*pfCreatedNew = FALSE;

	//test the pointers
	if (phComm == NULL || pszPort == NULL )
		return COMM_ERR_ARGUMENT;

	// at the beginning, we allocate the memory for the COMM structure
	if ( (pComm = (PCOMM)malloc(sizeof(COMM))) == NULL)
		return COMM_ERR_OUTOFMEMORY;

	/* the final serial port name will be stored at pszPortName. The port 
	 * name pszPort given by the caller can be either a visa like serial port
	 * resource name or a normal port name. Note, that the normal serial port 
	 * name on windows operating systems is "\.\COMx", while "x" is the serial
	 * port number. But it is also possible, that the serial port name is not
	 * "COMx".
	 */

	if ((lResult = __CommSplitPortAndOptions(pszPort, &pszPortName, &pszOptions)) != COMM_SUCCESS)
	{
		if (pszOptions != NULL)
			free(pszOptions);
		if (pszPortName != NULL)
			free(pszPortName);
		return lResult;
	}

	_ASSERT(pszPortName);

	/* Try to open the serial port. If the port is already open and fDuplicate
	 * is true, then we try to return an existing COMM handle.
	 */
	pComm->hC = CreateFile( 
		pszPortName,  
		GENERIC_READ | GENERIC_WRITE, 
		0, 
		NULL, 
		OPEN_EXISTING,
		FILE_FLAG_OVERLAPPED,
		NULL);

	/* if we where unable to open the serial port and the open error 
	 * was ERROR_ACCESS_DENIED and the fDuplicate flag is true, we
	 * are looking for this port in our internal list of open ports. If
	 * we can find this port, we return the handle of this port.
	 */
	if (pComm->hC == INVALID_HANDLE_VALUE)
	{
		*phComm = 0;
		lResult = GetLastError();
		switch (lResult)
		{
			case ERROR_ACCESS_DENIED:
				//the serial port is already open
				if (fDuplicate)
				{
					__CommLock(ghLock);
					//search for the serial port in out own list of open ports
					*phComm = ListFindElmHandle(gpCommList, pszPortName, (int(*)( const void *, const void *))ElmCmpPort );
					if (*phComm != NULL)
					{
						//the open port was found and we can return the existing
						//COMM handle.
						pComm = (PCOMM)ListGetElement(gpCommList, *phComm);
						pComm->ulNOpened += 1;
						__CommUnLock(ghLock);
						if (pszOptions != NULL)
							free(pszOptions);
						free(pszPortName);
						return COMM_SUCCESS;
					}
					__CommUnLock(ghLock);
				}
				
				//the fDuplicate flag is false, so we have to return 
				//with an error.
				if (pszOptions != NULL)
					free(pszOptions);
				free(pszPortName);
				return COMM_ERR_ACCESS;

			case ERROR_FILE_NOT_FOUND:
				//the serial port does not exist.
				if (pszOptions != NULL)
					free(pszOptions);
				free(pszPortName);
				free(pComm);
				return COMM_ERR_RESOURCE;

			default:
				//another (unknown) error occurred
				if (pszOptions != NULL)
					free(pszOptions);
				free(pszPortName);
				free(pComm);
				SetLastError(lResult);
				return COMM_ERR_SYSTEM;
		}
	}

	/* If we opened the serial port normally, we fill our data structure and 
	 * configure the serial port with our defaults.
	 */
	*pfCreatedNew = TRUE;

	//receive operation termination defaults to none. Some of the pointers have
	//to be set to NULL to prevent the CommClose and other functions to call free
	//for this pointer.
	pComm->lTeminationMode = COMM_TERM_NONE;
	pComm->lNumberOfTermChar = 0;
	pComm->pcTermination = NULL;
	pComm->RegEx = NULL;
	pComm->RegExExtra = NULL;
	pComm->lWriteMode = COMM_WRITE_NORMAL;
	pComm->pcEndOfTransmission = NULL;
	pComm->lNumberOfStartChars = 0;
	pComm->pcStartOfTransmission = 0;
	pComm->lNumberOfEndChars = 0;
	pComm->pcUserData = NULL;
	pComm->lUserDataSize = 0;
	pComm->osEvent.Internal = 0;
	pComm->osEvent.InternalHigh = 0;
	pComm->osEvent.Offset = 0;
	pComm->osEvent.OffsetHigh = 0;
	pComm->osEvent.Pointer = 0;
	pComm->osEvent.hEvent = 0;
	pComm->fWaitOnEvent = FALSE;

	//this serial port is opened only once
	pComm->ulNOpened = 1;
	
	//create the sempaphore for the external locking
	if ((pComm->hLock = __CommCreateLock()) == NULL)
	{			
		lResult = GetLastError();
		CloseHandle (pComm->hC);
		if (pszOptions != NULL)
			free(pszOptions);
		free(pszPortName);
		free(pComm);
		SetLastError(lResult);
		return COMM_ERR_SYSTEM;
	}

	//create the sempaphore for the internal read locking
	if ((pComm->hReadLock = __CommCreateLock()) == NULL)
	{
		lResult = GetLastError();
		CloseHandle (pComm->hLock);
		CloseHandle (pComm->hC);
		if (pszOptions != NULL)
			free(pszOptions);
		free(pszPortName);
		free(pComm);
		SetLastError(lResult);
		return COMM_ERR_SYSTEM;
	}

	//create the sempaphore for the internal write locking
	if ((pComm->hWriteLock = __CommCreateLock()) == NULL)
	{
		lResult = GetLastError();
		CloseHandle (pComm->hReadLock);
		CloseHandle (pComm->hLock);
		CloseHandle (pComm->hC);
		if (pszOptions != NULL)
			free(pszOptions);
		free(pszPortName);
		free(pComm);
		SetLastError(lResult);
		return COMM_ERR_SYSTEM;
	}

	//create the sempaphore for the internal write locking
	if ((pComm->hInternalLock = __CommCreateLock()) == NULL)
	{
		lResult = GetLastError();
		CloseHandle (pComm->hReadLock);
		CloseHandle (pComm->hWriteLock);
		CloseHandle (pComm->hLock);
		CloseHandle (pComm->hC);
		if (pszOptions != NULL)
			free(pszOptions);
		free(pszPortName);
		free(pComm);
		SetLastError(lResult);
		return COMM_ERR_SYSTEM;
	}

	//create the read event
	if ((pComm->hEvRead = CreateEvent(NULL, TRUE, FALSE, NULL)) == NULL)
	{
		lResult = GetLastError();
		__CommDeleteLock(pComm->hInternalLock);
		CloseHandle (pComm->hReadLock);
		CloseHandle (pComm->hWriteLock);
		CloseHandle (pComm->hLock);
		CloseHandle (pComm->hC);
		if (pszOptions != NULL)
			free(pszOptions);
		free(pszPortName);
		free(pComm);
		SetLastError(lResult);
		return COMM_ERR_SYSTEM;
	}

	//create the write event
	if ((pComm->hEvWrite = CreateEvent(NULL, TRUE, FALSE, NULL)) == NULL)
	{
		lResult = GetLastError();
		CloseHandle(pComm->hEvRead);
		__CommDeleteLock(pComm->hInternalLock);
		CloseHandle (pComm->hReadLock);
		CloseHandle (pComm->hWriteLock);
		CloseHandle (pComm->hLock);
		CloseHandle (pComm->hC);
		if (pszOptions != NULL)
			free(pszOptions);
		free(pszPortName);
		free(pComm);
		SetLastError(lResult);
		return COMM_ERR_SYSTEM;
	}

	//store the original serial port name
	if ((pComm->pszOriginalName = _strdup(pszPort)) == NULL)
	{
		lResult = GetLastError();
		CloseHandle(pComm->hEvWrite);
		__CommDeleteLock(pComm->hInternalLock);
		CloseHandle(pComm->hEvRead);
		CloseHandle (pComm->hReadLock);
		CloseHandle (pComm->hWriteLock);
		CloseHandle (pComm->hLock);
		CloseHandle (pComm->hC);
		if (pszOptions != NULL)
			free(pszOptions);
		free(pszPortName);
		free(pComm);
		SetLastError(lResult);
		return COMM_ERR_OUTOFMEMORY;
	}

	//store the real serial port name
	pComm->pszPort = pszPortName;

	//append the opened serial port to our internal open port list. This creates
	//the serial port handle which the user has to be used to call the functions
	//of this library.
	__CommLock(ghLock);
	*phComm = ListAppendElement (gpCommList, pComm);
	__CommUnLock(ghLock);
	if (*phComm == 0)
	{
		if (pszOptions != NULL)
			free(pszOptions);
		free(pComm->pszPort);
		CloseHandle(pComm->hEvWrite);
		CloseHandle(pComm->hEvRead);
		__CommDeleteLock(pComm->hInternalLock);
		CloseHandle (pComm->hReadLock);
		CloseHandle (pComm->hWriteLock);
		CloseHandle (pComm->hLock);
		CloseHandle (pComm->hC);
		free(pComm);
		return COMM_ERR_OUTOFMEMORY;
	}

	//now, all is done and the serial port can be used. Before we return, 
	//set some additional defaults for this port.
	if ((lResult = CommTimeout(*phComm, 10000)) != 0)
	{
		if (pszOptions != NULL)
			free(pszOptions);
		CommClosePort2(*phComm, FALSE);
		*phComm = NULL;
		return lResult;
	}

	//specify the default comm buffer size
	if ((lResult = CommBuffer(*phComm, LVCOMM_DEFAULT_RX_BUFFER, LVCOMM_DEFAULT_TX_BUFFER)) != 0)
	{
		if (pszOptions != NULL)
			free(pszOptions);
		CommClosePort2(*phComm, FALSE);
		*phComm = NULL;
		return lResult;
	}

	/* If an error occurred during a read or write operation, the
	 * serial port should abort the operation. Inside CommRead and
	 * CommWrite we call the ClearCommError function to get additional
	 * error information.
	 */
	if (!GetCommState(pComm->hC, &dcb))
	{
		//because this not a normal situation if GetCommState fails 
		//at this point, we pop up an assertation.
		_ASSERT(0); 
		CommClosePort2(*phComm, FALSE);
		*phComm = NULL;
		return COMM_ERR_SYSTEM;
	}
	dcb.fAbortOnError = 1;
	if (!SetCommState(pComm->hC, &dcb))
	{
		//because this not a normal situation if SetCommState fails 
		//at this point, we pop up an assertation.
		_ASSERT(0);
		CommClosePort2(*phComm, FALSE);
		*phComm = NULL;
		return COMM_ERR_SYSTEM;
	}

	//set the options specified in the serial port name string
	if (lResult = CommOptions(*phComm, pszOptions) != 0)
	{
		if (pszOptions != NULL)
			free(pszOptions);
		CommClosePort2(*phComm, FALSE);
		*phComm = NULL;
		return lResult;
	}

	if (pszOptions != NULL)
		free(pszOptions);

	SetLastError(0);

	return COMM_SUCCESS;
}

/*  CommClosePort
 *
 *  Closes the serial port. This function is implementd for backward
 *	compatibility and it only calls CommClosePort2 with fCloseAll set
 *	to false.
 *
 *  parameter
 *		hComm				pointer to the COMM structure
 *		
 *  return
 *		error code
 */
long LVSERIAL_API CommClosePort(HANDLE hComm)
{
	return CommClosePort2(hComm, FALSE);
}

/*  __CommClosePort
 *
 *  Closes the serial port.
 *
 *	Before this function closes the serial port it waits for any
 *	pending operations. In normal situation, the wait timeout could
 *	be infinite, but this can cause a dead lock in some situations.
 *	So a timeout value is specified, but this timeout can be very high
 *	esp. if the read and write timeouts are high. The exact calculation
 *	of this timeout may object of future modifications.
 *
 *  parameter
 *		hComm				serial port handle
 *		fCloseAll			if TRUE, this function closes all open instances 
 *							of the serial port
 *		fLock				if TRUE, thsi function locks the global lock. 
 *							Otherwise this function assumes, that the global
 *							lock is already locked.
 *		
 *  return
 *		error code
 */
long __CommClosePort(HANDLE hComm, int fCloseAll, int fLock, unsigned long ulTimeout)
{
	PCOMM pComm;						//pointer to the serial port COMM structure
	long lResult;						//this function return code
	unsigned long ulResult;				//function return code
	LPHANDLE lpLock[4];					//copy of the read and write lock
	HANDLE hLock;						//internal lock handle

	_ASSERT(gpCommList);


	if (fLock)
		__CommLock(ghLock);
	//test the given serial port handle and get the pointer to the
	//serial port data structure.
	if (ListCheckHandle(gpCommList, hComm) != 0)
	{
		if (fLock)
			__CommUnLock(ghLock);
		return COMM_ERR_HANDLE;
	}

	pComm = (PCOMM)ListGetElement(gpCommList, hComm);

	_ASSERT(pComm);

	//if the port has been opened multiple times, we do not 
	//really close the port.
	if (fCloseAll)
		pComm->ulNOpened = 0;
	else
		pComm->ulNOpened -= 1;

	if (pComm->ulNOpened != 0)
	{
		if (fLock)
			__CommUnLock(ghLock);

		return COMM_SUCCESS;
	}

	//assume, that we can finish successfully
	lResult = COMM_SUCCESS;

	//remove the handle from the list
	ListRemoveHandle (gpCommList, hComm);
	//Now other functions wich are starting now, are generating
	//an invalid handle error and we have to deal only with functions
	//which are still running and we can savely release the global lock.
	if (fLock)
		__CommUnLock(ghLock);

	//wait until any pending operations are finished. Note, that we 
	//continue to close the serial port even if there occurred an error 
	//or if we time out. Waiting for the semaphores only prevents the
	//other functions to return with a system error code if we close
	//the window handles during an operation on the handle.
	
	//copy the internal lock handle for later use (we have to close this
	//handle at the end of this function.
	hLock = pComm->hInternalLock;
	
	//wait for all the semaphores of this serial port
	lpLock[0] = pComm->hReadLock;
	lpLock[1] = pComm->hWriteLock;
	lpLock[2] = pComm->hLock;
	lpLock[3] = pComm->hInternalLock;

	//if the timeout is -1L, we have to calculate the timeout.
	if (ulTimeout == -1L)
	  ulTimeout = 2 * __max(pComm->ulWriteTotalTimeoutConstant, pComm->ulReadTotalTimeoutConstant);

	ulResult = WaitForMultipleObjects(
		4, 
		(HANDLE *)lpLock, 
		TRUE,
		ulTimeout
	);

	switch (ulResult)
	{
		case WAIT_OBJECT_0:
			//Ownership of the objects are granted.
		case WAIT_ABANDONED:
			//object that was not released by the thread that owned the mutex object 
			//before the owning thread terminated. Ownership of the object is granted.
			break;

		case WAIT_TIMEOUT:
			//wait operation timed out
			lResult = COMM_ERR_LOCKTIMEOUT;
			break;

		case WAIT_FAILED:
			//function failed, call GetLastError() to get more information
		default:
			//huh ... something wrong
			_ASSERT(0);
			lResult = GetLastError();
			break;
	}

	//hopefully we have granted all semaphores, so we can savely close
	//the serial port and release all handles and memory associated with
	//this serial port. If we have not granted all semaphores, we might
	//be running into one or more ugly situations.

	//close the serial port
	if (!CloseHandle( pComm->hC ))
	{
		_ASSERT(0);
		if (lResult == COMM_SUCCESS)
			lResult = GetLastError();
	}
#ifdef _DEBUG
		pComm->hC = NULL;
#endif

	//close the read event
	if (!CloseHandle( pComm->hEvRead ))
	{
		_ASSERT(0);
		if (lResult != COMM_SUCCESS)
			lResult = GetLastError();
	}
#ifdef _DEBUG
		pComm->hEvRead = NULL;
#endif

	//close the write event
	if (!CloseHandle( pComm->hEvWrite ))
	{
		_ASSERT(0);
		if (lResult != COMM_SUCCESS)
			lResult = GetLastError();
	}
#ifdef _DEBUG
		pComm->hEvWrite = NULL;
#endif

	//close the communication event
	if (pComm->osEvent.hEvent != 0)
	{
		if (!CloseHandle( pComm->osEvent.hEvent ))
		{
			_ASSERT(0);
		if (lResult != COMM_SUCCESS)
			lResult = GetLastError();
		}
#ifdef _DEBUG
			pComm->osEvent.hEvent = 0;
#endif

	}
	//close the external lock
	if (!CloseHandle( pComm->hLock ))
	{
		_ASSERT(0);
		if (lResult != COMM_SUCCESS)
			lResult = GetLastError();
	}
#ifdef _DEBUG
		pComm->hLock = NULL;
#endif

	//free the termination string
	if (pComm->pcTermination)
	{
		free(pComm->pcTermination);
#ifdef _DEBUG
		pComm->pcTermination = NULL;
#endif
	}

	//free the start of transmission buffer
	if (pComm->pcStartOfTransmission)
	{
		free(pComm->pcStartOfTransmission);
#ifdef _DEBUG
		pComm->pcStartOfTransmission = NULL;
#endif
	}

	//free the end of transmission buffer
	if (pComm->pcEndOfTransmission)
	{
		free(pComm->pcEndOfTransmission);
#ifdef _DEBUG
		pComm->pcEndOfTransmission = NULL;
#endif
	}

	//free the user data buffer
	if (pComm->pcUserData)
	{
		free(pComm->pcUserData);
#ifdef _DEBUG
		pComm->pcUserData = NULL;
		pComm->lUserDataSize = 0;
#endif
	}

	//free the real port name
	if (pComm->pszPort != NULL)
	{
		free(pComm->pszPort);
#ifdef _DEBUG
		pComm->pszPort = NULL;
#endif
	}

	//free the original port name
	if (pComm->pszOriginalName != NULL)
	{
		free(pComm->pszOriginalName);
#ifdef _DEBUG
		pComm->pszOriginalName = NULL;
#endif
	}

	//free the pcre data structure
	if (pComm->RegEx != NULL)
	{
		pcre_free(pComm->RegEx);
#ifdef _DEBUG
		pComm->RegEx = NULL;
#endif
	}

	//free the pcre extra data
	if (pComm->RegExExtra != NULL)
	{
		pcre_free(pComm->RegExExtra);
#ifdef _DEBUG
		pComm->RegExExtra = NULL;
#endif
	}

	//reset the internal read and write lock handle
#ifdef _DEBUG
	pComm->hReadLock = NULL;
	pComm->hWriteLock = NULL;
#endif

	//free the pComm data structure memory
	free(pComm);
#ifdef _DEBUG
	pComm = NULL;
#endif

	//now all things are done excapt, the read and write locks are still
	//exists. Now, we can safely close (and release) these objects.

	//close the internal read lock semaphore
	if (!CloseHandle(lpLock[0]))
	{
		_ASSERT(0);
		if (lResult != COMM_SUCCESS)
			lResult = GetLastError();
	}

	//close the internal write lock semaphore
	if (!CloseHandle(lpLock[1]))
	{
		_ASSERT(0);
		if (lResult != COMM_SUCCESS)
			lResult = GetLastError();
	}

	//close the internal lock
	if (!CloseHandle(hLock))
	{
		_ASSERT(0);
		if (lResult != COMM_SUCCESS);
			lResult = GetLastError();
	}

	//in case of an error during the close operations, we set the
	//last error status to the correct error code.
	if (lResult != COMM_SUCCESS)
	{
		if (lResult < COMM_MIN_ERROR || lResult > COMM_MAX_ERROR)
		{
			SetLastError(lResult);
			lResult = COMM_ERR_SYSTEM;
		}
	}

	return lResult;
}

/*  CommClosePort2
 *
 *  Closes the serial port
 *
 *  parameter
 *		hComm				serial port handle
 *		fCloseAll			if TRUE, this function closes all open instances 
 *							of the serial port
 *		
 *  return
 *		error code
 */

long LVSERIAL_API CommClosePort2(HANDLE hComm, int fCloseAll)
{
	return __CommClosePort(hComm, fCloseAll, TRUE, -1L);
}

/*  CommClosePort3
 *
 *  Closes the serial port
 *
 *  parameter
 *		hComm				serial port handle
 *		fCloseAll			if TRUE, this function closes all open instances 
 *							of the serial port
 *      ulTimeout			Timeout wating for pending read and write operations
 *		
 *  return
 *		error code
 */

long LVSERIAL_API CommClosePort3(HANDLE hComm, int fCloseAll, unsigned long ulTimeout)
{
	return __CommClosePort(hComm, fCloseAll, TRUE, ulTimeout);
}


/*  CommCloseAll
 *
 *  Closes all serial ports
 *
 *  return
 *		error code
 */
long LVSERIAL_API CommCloseAll(void)
{
	HANDLE hComm;						//handle of the serial port (used when closing all open ports)
	long lRes;
	long lErr;

	//close all open serial ports
	lRes = COMM_SUCCESS;

	//no other function should modify the list of open serial ports while
	//we are going through the complete list.
	__CommLock(ghLock);

	//close one port after each other while the is one or more serial
	//ports in the list. Note that we only report the first error.
	while( ListGetLength(gpCommList) > 0)
	{
		hComm = ListGetFirstHandle(gpCommList);

		_ASSERT(hComm);

		//we return the first error, but we continue to close the ports
		if (lRes == COMM_SUCCESS)
		{
			if ((lRes = __CommClosePort(hComm, TRUE, FALSE, -1L)) != COMM_SUCCESS)
				lErr = GetLastError();
		}
		else
			__CommClosePort(hComm, TRUE, FALSE, -1L);
	}
	//there are no more open serial ports and we can release the global lock and
	//other functions are allowed access the list execute.
	__CommUnLock(ghLock);

	//restore the first error
	if (lRes != COMM_SUCCESS)
		SetLastError(lErr);

	return lRes;
}

/*  __CommWrite
 *
 *  Writes bytes to the serial port from the specified buffer. This function
 *	is used by the CommWrite and CommWriteAndReceiveEcho function() and needs 
 *	an valid overlapped data structure containing an valid event handle. The 
 *	local timeout value is the remaining timeout for this write operation.
 *
 *  parameter
 *		pComm				pointer to the serial port data structure
 *		pcBuffer			pointer to a buffer from where this function reads
 *							the bytes
 *		ulBytesToWrite		number of bytes to write to the serial port
 *		pulBytesWritten		pointer where this function stores the written 
 *							number of bytes. Must be a valid pointer.
 *		osWrite				overlapped data structure containing a valid event handle
 *		
 *  return
 *		error code
 */
__inline long __CommWrite(
	PCOMM pComm, 
	char * pcBuffer, 
	unsigned long ulBytesToWrite, 
	unsigned long *pulBytesWritten,
	OVERLAPPED osWrite)
{
	unsigned long lResult;				//function return code
	long lFnkRes = COMM_SUCCESS;		//return code of this function

	/* this function is for internal use only, so we mustn't test
	 * the incoming function parameters. This has to be done by
	 * the CommRead function.
	 */

	//issue write
	if (!WriteFile(pComm->hC, pcBuffer, ulBytesToWrite, pulBytesWritten, &osWrite)) 
	{
		//WriteFile operation is not finished / any pending error
		//this may be a normal situation for overlapped write operations
		lResult = GetLastError();
		if (lResult != ERROR_IO_PENDING) 
		{ 
			// WriteFile failed, but isn't delayed. Report pError and abort.
			_ASSERT(0);
			lFnkRes = __CommClearError(pComm, COMM_ERR_SYSTEM);
		}
		else 
		{
			//Write is pending, so wait until the write operation 
			//is finished.
			lResult = WaitForSingleObject(
				osWrite.hEvent,
				pComm->ulWriteTotalTimeoutMultiplier*ulBytesToWrite +
					pComm->ulWriteTotalTimeoutConstant
			);

			switch(lResult)
			{
				// OVERLAPPED structure's event has been signaled. 
				case WAIT_OBJECT_0:
					if (!GetOverlappedResult(pComm->hC, &osWrite, pulBytesWritten, FALSE))
					{
						switch (GetLastError())
						{
							case ERROR_OPERATION_ABORTED:
								_ASSERT(0);
								lFnkRes = COMM_ERR_ABORTED;
								break;

							default:
								_ASSERT(0);
								lFnkRes = COMM_ERR_SYSTEM;
								break;
						}
					}
					else if (*pulBytesWritten != ulBytesToWrite)
					{
						_ASSERT(0);
						lFnkRes = COMM_ERR_WRITE;
					}
					break;

				case WAIT_TIMEOUT:
					if (!GetOverlappedResult(pComm->hC, &osWrite, pulBytesWritten, FALSE))
					{
						switch (GetLastError())
						{
							case ERROR_OPERATION_ABORTED:
								_ASSERT(0);
								lFnkRes = COMM_ERR_ABORTED;
								break;

							default:
								_ASSERT(0);
								lFnkRes = COMM_ERR_SYSTEM;
								break;
						}
					}
					lFnkRes = COMM_ERR_TIMEOUT;
					break;
           
				case WAIT_FAILED:
					//function failed, see GetLastError() for details
					//This can be a normal situation, e.g. if the serial 
					//port was closed while we are waiting
					_ASSERT(0);
					lFnkRes = COMM_ERR_SYSTEM;
					break;

				default:				
					// An error has occurred in WaitForSingleObject.
					// This usually indicates a problem with the
					// OVERLAPPED structure's event handle.
					_ASSERT(0);
					lFnkRes = COMM_ERR_SYSTEM;
					break;
			}
		}
	}

	return lFnkRes;
}

/*  CommWrite
 *
 *  Writes data to the serial port.
 *	
 *	The function waits until any pending writes are finished using the
 *	specified timeout value. The write operation itself can also last
 *	up to the specified timeout value. This means, that the complete
 *	execution of this function can last up to twice the specified timeout.
 *
 *  parameter
 *		hComm				serial port handle
 *		pcBuffer			buffer containing the bytes to write
 *		ulBytesToWrite		number of bytes to write
 *		pulBytesWritten		pointer where this function stores the number of 
 *							bytes written to the serial port
 *		
 *  return
 *		error code
 */
long LVSERIAL_API CommWrite(
	HANDLE hComm, 
	char * pcBuffer, 
	unsigned long ulBytesToWrite, 
	unsigned long * pulBytesWritten)
{
	PCOMM pComm;						//pointer to the serial port COMM structure
	OVERLAPPED osWrite = {0};			//overlapped structure
	unsigned long ulBytesWritten = 0;	//number of bytes written
	unsigned long ulStxBytesWritten = 0;//number of STX bytes written
	unsigned long ulEtxBytesWritten = 0;//number of ETX bytes written
	long lFnkRes = COMM_SUCCESS;		//result of this function
	long lResult;						//function return code

	_ASSERT(gpCommList);

	if (pcBuffer == NULL)
		return COMM_ERR_ARGUMENT;

	//test the given serial port handle and get the pointer to the
	//serial port data structure.
	__CommLock(ghLock);
	if (ListCheckHandle(gpCommList, hComm) != 0)
	{
		__CommUnLock(ghLock);
		return COMM_ERR_HANDLE;
	}

	pComm = (PCOMM)ListGetElement(gpCommList, hComm);
	__CommUnLock(ghLock);

	_ASSERT(pComm);

	//reset the numnber of bytes written, so in case of an error, 
	//this value is zero.
	if (pulBytesWritten != NULL)
		*pulBytesWritten = 0;

	//copy the write event to the overlapped structure
	osWrite.hEvent = pComm->hEvWrite;

	//wait until any pending write operation has finished
	lResult = __CommLockT(
		pComm->hWriteLock, 
		pComm->ulWriteTotalTimeoutConstant + 
			pComm->ulWriteTotalTimeoutMultiplier*ulBytesToWrite
	);

	if (lResult != COMM_SUCCESS)
		return lResult;

	//write the start of transmission string if enabled
	if (pComm->lWriteMode & COMM_WRITE_STX)
		lFnkRes = __CommWrite(
			pComm, 
			pComm->pcStartOfTransmission, 
			pComm->lNumberOfStartChars, 
			&ulStxBytesWritten, 
			osWrite);

	//transmit the specified string
	if (lFnkRes == COMM_SUCCESS)
	{
		lFnkRes = __CommWrite(
			pComm, 
			pcBuffer, 
			ulBytesToWrite, 
			&ulBytesWritten, 
			osWrite);

		//write the end of transmission string if enabled
		if (lFnkRes == COMM_SUCCESS && (pComm->lWriteMode & COMM_WRITE_ETX))
		{
			lFnkRes = __CommWrite(
				pComm, 
				pComm->pcEndOfTransmission, 
				pComm->lNumberOfEndChars,
				&ulEtxBytesWritten, 
				osWrite);
		}
	}

	//in case of an error we call the clear error function and 
	//hopefully we get some additional inforamtion with which we
	//can set any of our own error codes.
	if	(lFnkRes == COMM_ERR_SYSTEM)
		lFnkRes = __CommClearError(pComm, COMM_ERR_SYSTEM);
    
	//unlock the serial device
	if ((lResult = __CommUnLock(pComm->hWriteLock)) != COMM_SUCCESS)
	{
		_ASSERT(0);
		if (lFnkRes != COMM_SUCCESS)
			lFnkRes = lResult;
	}

	if (pulBytesWritten != NULL)
		*pulBytesWritten = ulStxBytesWritten + ulBytesWritten + ulEtxBytesWritten;

	return lFnkRes;
}

/*  __CommWriteAndReceiveEcho
 *
 *  Writes data to the serial port and received the echo. Note, that this
 *	function does not handle the specified termination, start and end of 
 *	transmission conditions.
 *
 *  parameter
 *		hComm				serial port handle
 *		pcWriteBuffer		buffer containing the bytes to write
 *		ulBytesToWrite		number of bytes to write
 *		pcReceiveBuffer		This is the location where this function stores
 *							the received characters
 *		fStopIfNotEqual		If this value is true, the transmission is stopped,
 *							if the received character is not equal to the
 *							transmitted character.
 *		pulTxCount			location where this function stores the number of
 *							transmitted bytes
 *		pulRxCount			location where this function stores the number of
 *							received bytes
 *		
 *  return
 *		error code
 */
__inline long __CommWriteAndReceiveEcho(
	PCOMM pComm,
	char *pcWriteBuffer,
	unsigned long ulBytesToWrite,		
	char *pcReadBuffer,
	int fStopIfNotEqual,
	unsigned long *pulTxCount,
	unsigned long *pulRxCount,
	OVERLAPPED osWrite,
	OVERLAPPED osReader)
{
	long lResult;						//function return code
	unsigned long ulCounter;			//tracks the number of written bytes
	unsigned long ulTimeout;			//calculated total timeout
	unsigned long ulNumberOfBytes;		//number of bytes written for a singe write operation
	char *pcTX = pcWriteBuffer;			//pointer to the byte to write
	char *pcRX;							//pointer where to store the received byte
	char cRX;							//received byte if the pointer to the receive buffer is NULL

	//if the pointer to the read buffer is null, we use cRX instead 
	//of the receive buffer
	if (pcReadBuffer == NULL)
		pcRX = &cRX;
	else
		pcRX = pcReadBuffer;

	//calculate the read timeout value
	ulTimeout = pComm->ulReadTotalTimeoutMultiplier*ulBytesToWrite + pComm->ulReadTotalTimeoutConstant;

	//now, we are ready to start to transmit and receive the echo
	//this for loop write and receives one character after each other
	//and ends if all bytes are written and received or if an error
	//occurred.
	for (
		ulCounter = 0, lResult = COMM_SUCCESS;
		ulCounter < ulBytesToWrite && lResult == COMM_SUCCESS; 
		ulCounter++, pcTX++)
	{
		//write the next character of the transmit buffer
		lResult = __CommWrite(pComm, pcTX, 1L, &ulNumberOfBytes, osWrite);
		
		//if the write operation was successful, we should receive the echo
		if (lResult == COMM_SUCCESS)
		{
			//increment the number of bytes sucessfully written
			//but only if pulTxCount is not NULL
			if (pulTxCount != NULL)
				++*pulTxCount;
			
			//receive the echo
			lResult = __CommRead(pComm, pcRX, 1L, &ulNumberOfBytes, osReader, ulTimeout);

			//if there war no error there are some little things to do
			if (lResult == COMM_SUCCESS)
			{
				//if the receive buffer pointer is not NULL, we increment
				//that pointer, so the next character will be stored into
				//the next byte.
				if (pulRxCount != NULL)
					++*pulRxCount;

				//if the written character should be equal to the echo
				//we test this condition here
				if (fStopIfNotEqual)
				{
					if (*pcRX != *pcTX)
						lResult = COMM_ERR_WRONGECHO;
				}

				//if the pointer to the receive buffer is NULL, we
				//store the received characters in cRX and we are
				//not allowed to increment the pcRX pointer.
				if (pcReadBuffer != NULL)
					pcRX++;
			}
		}
	}

	return lResult;
}

/*  CommWriteAndReceiveEcho
 *
 *  Writes data to the serial port and received the echo. Note, that this
 *	function does not handle the specified termination conditions.
 *
 *  parameter
 *		hComm				serial port handle
 *		pcWriteBuffer		buffer containing the bytes to write
 *		ulBytesToWrite		number of bytes to write
 *		pulBytesWritten		pointer where this function stores the number of 
 *							bytes written to the serial port
 *		pcReceiveBuffer		This is the location where this function stores
 *							the received characters
 *		fStopIfNotEqual		If this value is true, the transmission is stopped,
 *							if the received character is not equal to the
 *							transmitted character.
 *		
 *  return
 *		error code
 */
long LVSERIAL_API CommWriteAndReceiveEcho(
	HANDLE hComm, 
	char * pcWriteBuffer,
	unsigned long ulBytesToWrite, 
	unsigned long * pulBytesWritten,
	char * pcReadBuffer,
	unsigned long * pulBytesRead,
	long fStopIfNotEqual)
{
	PCOMM pComm;						//pointer to the serial port COMM structure
	OVERLAPPED osWrite = {0};			//overlapped structure for the write operation
	OVERLAPPED osReader = {0};			//overlapped structure for the read operation
	unsigned long ulTxCount = 0;		//number of bytes written
	unsigned long ulRxCount = 0;		//number of bytes read
	HANDLE ahLock[2];					//read and write lock handles
	long lFnkRes = COMM_SUCCESS;		//this function return code
	long lResult;						//function return code

	_ASSERT(gpCommList);

	//test the write buffer pointer
	if (pcWriteBuffer == NULL)
		return COMM_ERR_ARGUMENT;

	//finish with success if there is nothing to transmit
	if (ulBytesToWrite == 0)
		return COMM_SUCCESS;

	//In case of an error, we should return with no bytes read and written.
	if (pulBytesRead != NULL)
		*pulBytesRead = 0;

	if (pulBytesWritten != NULL)
		*pulBytesWritten = 0;

	//test the given serial port handle and get the pointer to the
	//serial port data structure.
	__CommLock(ghLock);
	if (ListCheckHandle(gpCommList, hComm) != 0)
	{
		__CommUnLock(ghLock);
		return COMM_ERR_HANDLE;
	}

	pComm = (PCOMM)ListGetElement(gpCommList, hComm);
	__CommUnLock(ghLock);

	_ASSERT(pComm);

	//copy the read and write event to the overlapped structure
	osWrite.hEvent = pComm->hEvWrite;
	osReader.hEvent = pComm->hEvRead;

	//wait until any pending read and write operation are finished
	ahLock[0] = pComm->hReadLock;
	ahLock[1] = pComm->hWriteLock;
	lResult = __CommLockMT( 
		ahLock, 
		2, 
		__max(
			pComm->ulWriteTotalTimeoutConstant + pComm->ulWriteTotalTimeoutMultiplier*ulBytesToWrite, 
			pComm->ulReadTotalTimeoutConstant + pComm->ulReadTotalTimeoutMultiplier*ulBytesToWrite
			)
	);

	if (lResult != COMM_SUCCESS)
		return lResult;

	//flush the receive buffer
	lResult = CommFlush(hComm, COMM_FLUSH_RXABORT | COMM_FLUSH_RXCLEAR);
	if (lResult == COMM_SUCCESS)
	{
		//write the start of transmission string if enabled
		if (pComm->lWriteMode & COMM_WRITE_STX)
			lFnkRes = __CommWriteAndReceiveEcho(
				pComm,
				pComm->pcStartOfTransmission,
				pComm->lNumberOfStartChars,
				NULL,
				fStopIfNotEqual,
				NULL,
				NULL,
				osWrite,
				osReader);

		//if there was no error we continue with the transmission 
		//of the write buffer
		if (lFnkRes == COMM_SUCCESS)
		{
			lFnkRes = __CommWriteAndReceiveEcho(
				pComm,
				pcWriteBuffer,
				ulBytesToWrite,
				pcReadBuffer,
				fStopIfNotEqual,
				&ulTxCount,
				&ulRxCount,
				osWrite,
				osReader);

			//if there was no error we continue with the transmission 
			//of the termination string
			if (lFnkRes == COMM_SUCCESS && (pComm->lWriteMode & COMM_WRITE_ETX))
			{
				lFnkRes = __CommWriteAndReceiveEcho(
					pComm,
					pComm->pcEndOfTransmission,
					pComm->lNumberOfEndChars,
					NULL,
					fStopIfNotEqual,
					NULL,
					NULL,
					osWrite,
					osReader);
			}
		}
	}

	//in case of an error we call the clear error function and 
	//hopefully we get some additional inforamtion with which we
	//can set any of our own error codes.
	if	(lFnkRes == COMM_ERR_SYSTEM)
		lFnkRes = __CommClearError(pComm, COMM_ERR_SYSTEM);
    
	//unlock the serial device
	if ((lResult = __CommUnLock(pComm->hReadLock)) != COMM_SUCCESS)
	{
		_ASSERT(0);
		if (lFnkRes == COMM_SUCCESS)
			lFnkRes = lResult;
	}

	if ((lResult = __CommUnLock(pComm->hWriteLock)) != COMM_SUCCESS)
	{
		_ASSERT(0);
		if (lFnkRes == COMM_SUCCESS)
			lFnkRes = lResult;
	}

	if (pulBytesWritten != NULL)
		*pulBytesWritten = ulTxCount;

	if (pulBytesRead != NULL)
		*pulBytesRead = ulRxCount;

	return lFnkRes;
}

/*  __CommRead
 *
 *  Reads bytes from the serial port and stores the data into the specified
 *  buffer. This function is used by the CommRead function and needs an valid
 *  overlapped data structure containing an valid event handle. The local timeout
 *  value is the remaining timeout for this file read operation.
 *
 *  parameter
 *		pComm				pointer to the serial port data structure
 *		pcBuffer			pointer to a buffer where this function stores the 
 *							read bytes
 *		ulBytesToRead		number of bytes to read from the serial port
 *		pulBytesRead		pointer where this function stores the read number 
 *							of bytes. Must be a valid pointer.
 *		osReader			overlapped data structure containing a valid event handle
 *		ulLocalTimeout		timeout in milliseconds for the WaitForSingleObject
 *							function.
 *		
 *  return
 *		error code
 */
__inline long __CommRead(
	PCOMM pComm, 
	char * pcBuffer, 
	unsigned long ulBytesToRead, 
	unsigned long * pulBytesRead,
	OVERLAPPED osReader,
	unsigned long ulLocalTimeout)
{
	unsigned long ulResult;				//function return code
	long lFnkRes = COMM_SUCCESS;		//return code of this function

	/* this function is for internal use only, so we mustn't test
	 * the incoming function parameters. This has to be done by
	 * the CommRead function.
	 */

	//issue read operation.
	if (!ReadFile(pComm->hC, pcBuffer, ulBytesToRead, pulBytesRead, &osReader)) 
	{
		//read not delayed?
		if ((ulResult=GetLastError()) != ERROR_IO_PENDING)
		{
			//error in communications
			_ASSERT(0);
			return COMM_ERR_SYSTEM;
		}
		else
		{
			//the read operation is not finished yet, so we should wait 
			//until the read operation has finished by waiting for the 
			//read event. Here we go.

			//wait until the read operation is completed
			ulResult = WaitForSingleObject(osReader.hEvent, ulLocalTimeout);
			switch(ulResult)
			{
				//read completed
				case WAIT_OBJECT_0:
					//store the received bytes into the receive buffer
					if (!GetOverlappedResult(pComm->hC, &osReader, pulBytesRead, FALSE))
					{
						//error in communications
						//set the return code depending on the windows error code
						switch (GetLastError())
						{
							case ERROR_OPERATION_ABORTED:
								_ASSERT(0);
								lFnkRes = COMM_ERR_ABORTED;
								break;

							default:
								_ASSERT(0);
								lFnkRes = COMM_ERR_SYSTEM;
								break;
						}
						return lFnkRes;
					}
					else			 
					{
						//read completed. If we got less bytes than requested, 
						//then the operation has been timed out
						if (*pulBytesRead < ulBytesToRead)
							return COMM_ERR_TIMEOUT;

						return COMM_SUCCESS;
					}
					break;

				case WAIT_TIMEOUT:
					//read operation timed out
					if (!GetOverlappedResult(pComm->hC, &osReader, pulBytesRead, FALSE))
					{
						//error in communications
						//set the return code depending on the windows error code
						switch (GetLastError())
						{
							case ERROR_OPERATION_ABORTED:
								//The I/O operation has been aborted because of 
								//either a thread exit or an application request.
								_ASSERT(0);
								lFnkRes = COMM_ERR_ABORTED;
								break;

							case ERROR_IO_INCOMPLETE:
								//Overlapped I/O event is not in a signaled state.
								//We get this situation if a termination mode is enabled
								//and we received some characters but the termination
								//condition never matches.
								lFnkRes = COMM_ERR_TIMEOUT;
								break;

							default:
								_ASSERT(0);
								lFnkRes = COMM_ERR_SYSTEM;
								break;
						}
						return lFnkRes;
					}

					return COMM_ERR_TIMEOUT;
					break;

				case WAIT_FAILED:
					//the function has failed and we return the
					//system error code
					//This can be a normal situation, if, for example,
					//the serial port was closed while we are waiting
					_ASSERT(0);
					return COMM_ERR_SYSTEM;
					break;

				default:
					//error in the WaitForSingleObject. This indicates 
					//a problem with the OVERLAPPED structure's event handle.
					//Oh yes, we do not have a special error code for this
					//situation (may be changed in future releases) and we
					//allways return COMM_ERR_SYSTEM.
					if (ulResult = GetLastError() != 0)
					{
						_ASSERT(0);
						return COMM_ERR_SYSTEM;
					}
					else
					{
						_ASSERT(0);
						return COMM_ERR_SYSTEM;
					}
					break;
			}
		}
	}
	else 
	{    
		//read completed immediately
		return COMM_SUCCESS;
	}

	return COMM_SUCCESS;
}

/*  CommRead
 *
 *  Reads bytes from the serial port and stores the data into the specified
 *  buffer.
 *
 *	The function waits until any pending reads are finished using the
 *	specified timeout value. The read operation itself can also last
 *	up to the specified timeout value. This means, that the complete
 *	execution of this function can last up to twice the specified timeout.
 *
 *  parameter
 *		hComm				serial port handle
 *		pcBuffer			pointer to a buffer where this function stores the 
 *							read bytes
 *		ulBytesToRead		number of bytes to read from the serial port
 *		pulBytesRead		pointer where this function stores the read number 
 *							of bytes.
 *		
 *  return
 *		error code
 */
long LVSERIAL_API CommRead(
	HANDLE hComm, 
	char * pcBuffer, 
	unsigned long ulBytesToRead, 
	unsigned long * pulBytesRead)
{
	PCOMM pComm;						//pointer to the serial port COMM structure
	long lResult;						//function return code
	long lFnkRes;						//this function return code
	OVERLAPPED osReader = {0};			//overlapped structure
	unsigned long ulBytesRead;			//number of bytes read
	unsigned long ulTotalBytesRead;		//total number of bytes read
	int fTermReceived;					//this flag is true if the termination condition matches
	clock_t Finish;						//clock ticks, when the read operation should time out
	unsigned long ulRemainingTime;		//remaining time until the read operation times out
	int aiOffsets[3];					//offsets for the regex match operation

	//test the global list of opened ports
	_ASSERT(gpCommList);

	//test the pointer to the receive buffer
	if (pcBuffer == NULL)
		return COMM_ERR_ARGUMENT;

	//test the given serial port handle and get the pointer to the
	//serial port data structure.
	__CommLock(ghLock);
	if (ListCheckHandle(gpCommList, hComm) != 0)
	{
		__CommUnLock(ghLock);
		return COMM_ERR_HANDLE;
	}

	//get the pComm structure
	pComm = (PCOMM)ListGetElement(gpCommList, hComm);
	__CommUnLock(ghLock);

	//validate the pComm pointer
	_ASSERT(pComm);

	//initialize the number of received bytes to 0 if the pointer is not NULL
	if (pulBytesRead != NULL)
		*pulBytesRead = 0;

	//if there is nothing to receive, we return with success
	if (ulBytesToRead == 0)
		return COMM_SUCCESS;

	//get the overlapped event
	osReader.hEvent = pComm->hEvRead;

	//initialize the total number of received bytes
	ulTotalBytesRead = 0;

	//wait until any pending read operation has finished
	lResult = __CommLockT(
		pComm->hReadLock, 
		pComm->ulReadTotalTimeoutConstant + 
			pComm->ulReadTotalTimeoutMultiplier*ulBytesToRead
	);

	if (lResult != COMM_SUCCESS)
	{
		_ASSERT(0);
		return lResult;
	}

	//if any of the termination modes are enabled, we should take care
	//of that. Otherwise, we can issue a single read operation (see below)
	if (pComm->lTeminationMode != COMM_TERM_NONE)
	{
		//Read one character after each other and test for termination.
		//So for each of these read operation we have to recalculate the
		//remaining total timeout.
		
		Finish = clock() +
			pComm->ulReadTotalTimeoutMultiplier*ulBytesToRead +
			pComm->ulReadTotalTimeoutConstant;

		//nothing received: initialize fTermReceived flag to false
		fTermReceived = FALSE;

		//read one byte after each other and test the termination
		//condition. This continues until the termination condition
		//matches, the maximum number bytes are received or an if 
		//error occurred.
		do {
			//only for this iteration: no bytes received.
			ulBytesRead = 0;

			//calculate the remaining time out
			ulRemainingTime = Finish - clock();

			//read one byte from the serial port
			lFnkRes = __CommRead(
				pComm, 
				pcBuffer+ulTotalBytesRead, 
				1, 
				&ulBytesRead, 
				osReader,
				ulRemainingTime);
			
			//if we received a byte, we shold update the total number of 
			//received bytes and test the termination condition.
			if (ulBytesRead > 0)
			{
				//update the total number of received bytes
				ulTotalBytesRead += ulBytesRead;

				//test the termination condition
				switch (pComm->lTeminationMode)
				{
					case COMM_TERM_CHAR: //one or more termination characters
						//search the received character in the buffer of the
						//termination characters.
						fTermReceived = 
							memchr(					
								pComm->pcTermination, 
								*(pcBuffer+ulTotalBytesRead-1), 
								pComm->lNumberOfTermChar) != NULL;
						break;

					case COMM_TERM_STRING: //termination string
						//there must be at least the number of bytes of 
						//the termination string.
						if (ulTotalBytesRead >= (unsigned long)pComm->lNumberOfTermChar)
						{
							//we only test the last bytes of the receive buffer
							fTermReceived = memcmp(
								pcBuffer + ulTotalBytesRead - pComm->lNumberOfTermChar,
								pComm->pcTermination, 
								pComm->lNumberOfTermChar) == 0;
						}
						break;

					case COMM_TERM_REGEX: //regular expression
						//execute the precompiled regular expression
						fTermReceived = pcre_exec(
							pComm->RegEx, 
							pComm->RegExExtra,
							pcBuffer, 
							ulTotalBytesRead, 
							0,
							PCRE_NOTEMPTY,
							aiOffsets,
							3) >= 0;
						break;

					default:
						//huh ... unknown termination mode
						_ASSERT(0);
						fTermReceived = 1;
				}
			}
					  
			//Repeat this until 
			// - an error occurred or
			// - the termination condition is true or 
			// - we timed out
		} while (!lFnkRes && 
				!fTermReceived &&
				ulTotalBytesRead < ulBytesToRead &&
				Finish > clock());

		//adjust the result code according to the result of 
		//the read operation.
		if (lFnkRes == COMM_SUCCESS)
		{
			if (!fTermReceived)
			{
				//termination condition not matched, so we test, if the max
				//number of bytes are received.
				if (ulTotalBytesRead == ulBytesToRead)
					lFnkRes = COMM_WARN_NYBTES;
				else
					lFnkRes = COMM_ERR_TERMCHAR;
			}			
			else
				//termination condition matched
				lFnkRes = COMM_WARN_TERMCHAR;
		}
	}
	else
	{
		//The termination is not activated. So we can read all
		//requested bytes in a single step.
		lFnkRes = __CommRead(
			pComm, 
			pcBuffer, 
			ulBytesToRead, 
			&ulTotalBytesRead, 
			osReader,
			pComm->ulReadTotalTimeoutMultiplier*ulBytesToRead +
				pComm->ulReadTotalTimeoutConstant
		);
	}

	//in case of an error we call the clear error function and 
	//hopefully we get some additional inforamtion with which we
	//can set any of our own error codes.
	if	(lFnkRes == COMM_ERR_SYSTEM)
		lFnkRes = __CommClearError(pComm, COMM_ERR_SYSTEM);

	//release the internal semaphore, so others can continue 
	//with their read operation.
	if ((lResult = __CommUnLock(pComm->hReadLock)) != COMM_SUCCESS)
	{
		_ASSERT(0);
		if (lFnkRes >= COMM_SUCCESS)
			lFnkRes = lResult;
	}

	//copy the number of bytes read
	if (pulBytesRead != NULL)
		*pulBytesRead = ulTotalBytesRead;

	return lFnkRes;
}

/*  CommBreak
 *
 *  Sets the break state for the specified number of milliseconds
 *
 *  parameter
 *		hComm				serial port handle
 *		ulMilliseconds		break state duration
 *		
 *  return
 *		error code
 */
long LVSERIAL_API CommBreak(HANDLE hComm, unsigned long ulMilliseconds)
{
	PCOMM pComm;						//pointer to the serial port COMM structure

	_ASSERT(gpCommList);

	//test the given serial port handle and get the pointer to the
	//serial port data structure.
	__CommLock(ghLock);
	if (ListCheckHandle(gpCommList, hComm) != 0)
	{
		__CommUnLock(ghLock);
		return COMM_ERR_HANDLE;
	}

	pComm = (PCOMM)ListGetElement(gpCommList, hComm);
	__CommUnLock(ghLock);

	_ASSERT(pComm);

	//set the break state, delay for the specified milliseconds 
	//and clear the break condition.
	if (!SetCommBreak(pComm->hC))
		return __CommClearError(pComm, COMM_ERR_SYSTEM);
	
	Sleep(ulMilliseconds);

	if (!ClearCommBreak(pComm->hC))
		return __CommClearError(pComm, COMM_ERR_SYSTEM);

	return COMM_SUCCESS;
}

/*  CommBuffer
 *
 *  Sets the input and output queue size. If you specify a buffer size below
 *	the minimum of the windows operating system, the OS limits the queue sizes
 *	these limits. IMHO this limit is 4096.
 *
 *  parameter
 *		hComm				serial port handle
 *		ulInQueue			input queue size
 *		ulOutQueue			output queue size
 *		
 *  return
 *		error code
 */
long LVSERIAL_API CommBuffer(HANDLE hComm, unsigned long ulInQueue, unsigned long ulOutQueue)
{
	DCB dcb;							//DCB structure
	PCOMM pComm;						//pointer to the serial port COMM structure

	_ASSERT(gpCommList);

	//test the given serial port handle and get the pointer to the
	//serial port data structure.
	__CommLock(ghLock);
	if (ListCheckHandle(gpCommList, hComm) != 0)
	{
		__CommUnLock(ghLock);
		return COMM_ERR_HANDLE;
	}

	pComm = (PCOMM)ListGetElement(gpCommList, hComm);
	__CommUnLock(ghLock);

	_ASSERT(pComm);

	if (ulInQueue > _UI16_MAX || ulOutQueue > _UI16_MAX)
		return COMM_ERR_ARGUMENT;
	
	//when changing the queue sizes, we also have to adjust the
	//size of the Xon and Xoff limits. This is necesseray if one
	//of the handshake modes are enables.
	if (!SetupComm(pComm->hC, ulInQueue, ulOutQueue))
		return __CommClearError(pComm, COMM_ERR_ARGUMENT);

	//modify the dcb structure. At first, we lock the device, because no
	//other function should modify the DCB during this operation.
	__CommLock(pComm->hInternalLock);
	if (!GetCommState(pComm->hC, &dcb))
	{
		__CommUnLock(pComm->hInternalLock);
		return __CommClearError(pComm, COMM_ERR_SYSTEM);
	}

	dcb.XonLim = (WORD)((ulInQueue * 2) / 3);
	dcb.XoffLim = (WORD)ulInQueue / 3;

	if (!SetCommState(pComm->hC, &dcb))
	{
		__CommUnLock(pComm->hInternalLock);
		return __CommClearError(pComm, COMM_ERR_ARGUMENT);
	}
	__CommUnLock(pComm->hInternalLock);

	return COMM_SUCCESS;
}

/*  CommFlush
 *
 *  The CommFlush function discards all characters from the output or input 
 *  buffer of a specified serial port. It can also terminate pending read 
 *  or write operations.
 *
 *  parameter
 *		hComm				serial port handle
 *		ulFlags				Specifies the action to take.
 *			COMM_FLUSH_TXABORT		Kill the pending/current writes to the comm port.
 *			COMM_FLUSH_RXABORT		Kill the pending/current reads to the comm port.
 *			COMM_FLUSH_TXCLEAR		Kill the transmit queue if there.
 *			COMM_FLUSH_RXCLEAR		Kill the typeahead buffer if there.
 *		
 *  return
 *		error code
 */
long LVSERIAL_API CommFlush(HANDLE hComm, unsigned long ulFlags)
{
	PCOMM pComm;						//pointer to the serial port COMM structure

	_ASSERT(gpCommList);

	//test the given serial port handle and get the pointer to the
	//serial port data structure.
	__CommLock(ghLock);
	if (ListCheckHandle(gpCommList, hComm) != 0)
	{
		__CommUnLock(ghLock);
		return COMM_ERR_HANDLE;
	}

	pComm = (PCOMM)ListGetElement(gpCommList, hComm);
	__CommUnLock(ghLock);

	_ASSERT(pComm);

	//flush the serial port buffer
	if (!PurgeComm(pComm->hC, ulFlags))
		return __CommClearError(pComm, COMM_ERR_SYSTEM);

	return COMM_SUCCESS;
}

/*  CommStatus
 *
 *  The CommStatus function retrieves information about a communications pError 
 *  and reports the current status of a communications device. The function is 
 *  called when a communications pError occurs, and it clears the device's pError 
 *  flag to enable additional input and output (I/O) operations.
 *
 *  parameter
 *		hComm				serial port handle
 *		pulErrorFlags		pError flags (see COMM_ERRF_ )
 *		pulFlags			status flags (see COMM_STAT_ )
 *		pulInQueue			Number of bytes received by the serial provider but not 
 *							yet read by a CommRead operation.
 *		pulOutQueue			Number of bytes of user data remaining to be transmitted 
 *							for all write operations. 
 *		
 *  return
 *		error code
 */
long LVSERIAL_API CommStatus(
	HANDLE hComm, 
	unsigned long * pulErrorFlags, 
	unsigned long * pulFlags,
	unsigned long * pulInQueue,
	unsigned long * pulOutQueue)
{
	COMSTAT stat;						//COMSTAT structure
	PCOMM pComm;						//pointer to the serial port COMM structure

	_ASSERT(gpCommList);

	//test the given serial port handle and get the pointer to the
	//serial port data structure.
	__CommLock(ghLock);
	if (ListCheckHandle(gpCommList, hComm) != 0)
	{
		__CommUnLock(ghLock);
		if (pulErrorFlags != NULL)
			*pulErrorFlags = 0;
		if (pulFlags != NULL)
			*pulFlags = 0;
		if (pulInQueue != NULL)
			*pulInQueue = 0;
		if (pulOutQueue != NULL)
			*pulOutQueue = 0;
		return COMM_ERR_HANDLE;
	}

	pComm = (PCOMM)ListGetElement(gpCommList, hComm);
	__CommUnLock(ghLock);

	_ASSERT(pComm);

	//get the serial port status by calling the 
	//ClearCommError function
	if (!ClearCommError(pComm->hC, pulErrorFlags, &stat))
	{
		_ASSERT(0);
		if (pulErrorFlags != NULL)
			*pulErrorFlags = 0;
		if (pulFlags != NULL)
			*pulFlags = 0;
		if (pulInQueue != NULL)
			*pulInQueue = 0;
		if (pulOutQueue != NULL)
			*pulOutQueue = 0;
		return COMM_ERR_SYSTEM;
	}

	if (pulInQueue != NULL)
		*pulInQueue = stat.cbInQue;

	if (pulOutQueue != NULL)
		*pulOutQueue = stat.cbOutQue;

	if (pulFlags != 0)
	{
		*pulFlags = 
			stat.fCtsHold +
			(stat.fDsrHold << 1) +
			(stat.fRlsdHold << 2) +
			(stat.fXoffHold << 3) +
			(stat.fXoffSent << 4) +
			(stat.fEof << 5) +
			(stat.fTxim << 6);
	}

	return __CommErrorFlagToCode(*pulErrorFlags);
}

/*  CommBytesAtPort
 *
 *  The function retrieves the number of bytes at the serial ports input queue.
 *
 *  parameter
 *		hComm				serial port handle
 *		pulBytesAtPort		Number of bytes received by the serial provider but not 
 *							yet read by a CommRead operation.
 *		
 *  return
 *		error code
 */
long LVSERIAL_API CommBytesAtPort( HANDLE hComm, unsigned long * pulBytesAtPort)
{
	COMSTAT stat;						//COMMSTAT structure
	unsigned long ulErrorFlags;			//serial port error flags
	PCOMM pComm;						//pointer to the serial port COMM structure

	_ASSERT(gpCommList);

	//test the given serial port handle and get the pointer to the
	//serial port data structure.
	__CommLock(ghLock);
	if (ListCheckHandle(gpCommList, hComm) != 0)
	{
		__CommUnLock(ghLock);
		if (pulBytesAtPort != NULL)
			*pulBytesAtPort = 0;
		return COMM_ERR_HANDLE;
	}

	pComm = (PCOMM)ListGetElement(gpCommList, hComm);
	__CommUnLock(ghLock);

	_ASSERT(pComm);

	//get the number of bytes in the serial port queue
	if (!ClearCommError(pComm->hC, &ulErrorFlags, &stat))
	{
		_ASSERT(0);
		if ((void *)*pulBytesAtPort != NULL)
			*pulBytesAtPort = 0;
		return COMM_ERR_SYSTEM;
	}

	if (pulBytesAtPort != NULL)
		*pulBytesAtPort = stat.cbInQue;

	return __CommErrorFlagToCode(ulErrorFlags);
}

/*  CommModemStatus
 *
 *  The CommModemStatus function retrieves modem control-register values.
 *
 *  parameter
 *		hComm				serial port handle
 *		pulModemStatus		Pointer to a variable that specifies the current state 
 *							of the modem control-register values. (see COMM_MSTAT_ )
 *
 *  return
 *		error code
 */
long LVSERIAL_API CommModemStatus(HANDLE hComm, unsigned long * pulModemStatus)
{
	PCOMM pComm;						//pointer to the serial port COMM structure

	_ASSERT(gpCommList);

	//test the given serial port handle and get the pointer to the
	//serial port data structure.
	__CommLock(ghLock);
	if (ListCheckHandle(gpCommList, hComm) != 0)
	{
		__CommUnLock(ghLock);
		if (pulModemStatus != NULL)
			*pulModemStatus = 0;
		return COMM_ERR_HANDLE;
	}

	pComm = (PCOMM)ListGetElement(gpCommList, hComm);
	__CommUnLock(ghLock);

	_ASSERT(pComm);

	//retrieve the modem status
	if (!GetCommModemStatus(pComm->hC, pulModemStatus))
	{
		if (pulModemStatus != NULL)
			*pulModemStatus = 0;
		return __CommClearError(pComm,COMM_ERR_SYSTEM);
	}

	return COMM_SUCCESS;
}

/*  CommGetCTS
 *
 *  The CommGetCTS function retrieves modem control-register value.
 *
 *  parameter
 *		hComm				serial port handle
 *		pfCTS				Pointer to a variable that specifies the current state 
 *							of the CTS signal.
 *		
 *  return
 *		error code
 */
long LVSERIAL_API CommGetCTS(HANDLE hComm, int * pfCTS)
{
	unsigned long ulModemStatus;		//modem status flags
	PCOMM pComm;						//pointer to the serial port COMM structure

	_ASSERT(gpCommList);

	//test the given serial port handle and get the pointer to the
	//serial port data structure.
	__CommLock(ghLock);
	if (ListCheckHandle(gpCommList, hComm) != 0)
	{
		__CommUnLock(ghLock);
		if (pfCTS != NULL)
			pfCTS = FALSE;
		return COMM_ERR_HANDLE;
	}

	pComm = (PCOMM)ListGetElement(gpCommList, hComm);
	__CommUnLock(ghLock);

	_ASSERT(pComm);

	//get the modem status
	if (!GetCommModemStatus(pComm->hC, &ulModemStatus))
	{
		//reading the modem status has faild, return false for the line,
		//clear any pending communication error and return extended 
		//error information if it is available.
		if (pfCTS != NULL)
			pfCTS = FALSE;
		return __CommClearError(pComm,COMM_ERR_SYSTEM);
	}

	//copy the line state to the target variable
	if (pfCTS != NULL)
		*pfCTS = (ulModemStatus & MS_CTS_ON) ? TRUE : FALSE;

	return COMM_SUCCESS;
}

/*  CommGetDSR
 *
 *  The CommGetDSR function retrieves modem control-register value.
 *
 *  parameter
 *		hComm				serial port handle
 *		pfDSR				Pointer to a variable that specifies the current state 
 *							of the DSR signal.
 *		
 *  return
 *		error code
 */
long LVSERIAL_API CommGetDSR(HANDLE hComm, int *pfDSR)
{
	unsigned long ulModemStatus;		//modem status flags
	PCOMM pComm;						//pointer to the serial port COMM structure

	_ASSERT(gpCommList);

	//test the given serial port handle and get the pointer to the
	//serial port data structure.
	__CommLock(ghLock);
	if (ListCheckHandle(gpCommList, hComm) != 0)
	{
		__CommUnLock(ghLock);
		if (pfDSR != NULL)
			*pfDSR = FALSE;
		return COMM_ERR_HANDLE;
	}

	pComm = (PCOMM)ListGetElement(gpCommList, hComm);
	__CommUnLock(ghLock);

	_ASSERT(pComm);

	//get the modem status
	if (!GetCommModemStatus(pComm->hC, &ulModemStatus))
	{
		//reading the modem status has faild, return false for the line,
		//clear any pending communication error and return extended 
		//error information if it is available.
		if (pfDSR != NULL)
			*pfDSR = FALSE;
		return __CommClearError(pComm,COMM_ERR_SYSTEM);
	}

	//copy the line status to the output variable
	if (pfDSR != NULL)
		*pfDSR = (ulModemStatus & MS_DSR_ON) ? TRUE : FALSE;

	return COMM_SUCCESS;
}

/*  CommGetRING
 *
 *  The CommGetRING function retrieves modem control-register value.
 *
 *  parameter
 *		hComm				serial port handle
 *		pfRING				Pointer to a variable that specifies the current state 
 *							of the RING signal.
 *		
 *  return
 *		error code
 */
long LVSERIAL_API CommGetRING(HANDLE hComm, int *pfRING)
{
	unsigned long ulModemStatus;		//modem status flags
	PCOMM pComm;						//pointer to the serial port COMM structure

	_ASSERT(gpCommList);

	//test the given serial port handle and get the pointer to the
	//serial port data structure.
	__CommLock(ghLock);
	if (ListCheckHandle(gpCommList, hComm) != 0)
	{
		__CommUnLock(ghLock);
		if (pfRING != NULL)
			*pfRING = FALSE;
		return COMM_ERR_HANDLE;
	}

	pComm = (PCOMM)ListGetElement(gpCommList, hComm);
	__CommUnLock(ghLock);

	_ASSERT(pComm);

	//get the modem status
	if (!GetCommModemStatus(pComm->hC, &ulModemStatus))
	{
		//reading the modem status has faild, return false for the line,
		//clear any pending communication error and return extended 
		//error information if it is available.
		if (pfRING != NULL)
			*pfRING = FALSE;
		return __CommClearError(pComm,COMM_ERR_SYSTEM);
	}

	//copy the line status to the target variable
	if (pfRING != NULL)
		*pfRING = (ulModemStatus & MS_RING_ON) ? TRUE : FALSE;

	return COMM_SUCCESS;
}

/*  CommGetRLSD
 *
 *  The CommGetRLSD function retrieves modem control-register value.
 *
 *  parameter
 *		hComm				serial port handle
 *		pfRLSD				Pointer to a variable that specifies the current state 
 *							of the RLSD signal (receive-line-signal-detect).
 *		
 *  return
 *		error code
 */
long LVSERIAL_API CommGetRLSD(HANDLE hComm, int *pfRLSD)
{
	unsigned long ulModemStatus;		//modem status flags
	PCOMM pComm;						//pointer to the serial port COMM structure

	_ASSERT(gpCommList);

	//test the given serial port handle and get the pointer to the
	//serial port data structure.
	__CommLock(ghLock);
	if (ListCheckHandle(gpCommList, hComm) != 0)
	{
		__CommUnLock(ghLock);
		if (pfRLSD != NULL)
			*pfRLSD = FALSE;
		return COMM_ERR_HANDLE;
	}

	pComm = (PCOMM)ListGetElement(gpCommList, hComm);
	__CommUnLock(ghLock);

	_ASSERT(pComm);

	//get the modem status
	if (!GetCommModemStatus(pComm->hC, &ulModemStatus))
	{
		//reading the modem status has faild, return false for the line,
		//clear any pending communication error and return extended 
		//error information if it is available.
		if (pfRLSD != NULL)
			*pfRLSD = FALSE;
		return __CommClearError(pComm,COMM_ERR_SYSTEM);
	}

	//copy the line status to the target variable
	if (pfRLSD != NULL)
		*pfRLSD = (ulModemStatus & MS_RLSD_ON) ? TRUE : FALSE;

	return COMM_SUCCESS;
}

/*  CommSetXOFF
 *
 *  Causes transmission to act as if an XOFF character has been received.
 *
 *  parameter
 *		hComm				serial port handle
 *		
 *  return
 *		error code
 */
long LVSERIAL_API CommSetXOFF(HANDLE hComm)
{
	PCOMM pComm;						//pointer to the serial port COMM structure

	_ASSERT(gpCommList);

	//test the given serial port handle and get the pointer to the
	//serial port data structure.
	__CommLock(ghLock);
	if (ListCheckHandle(gpCommList, hComm) != 0)
	{
		__CommUnLock(ghLock);
		return COMM_ERR_HANDLE;
	}

	pComm = (PCOMM)ListGetElement(gpCommList, hComm);
	__CommUnLock(ghLock);

	_ASSERT(pComm);

	//set the XOFF status
	if (!EscapeCommFunction(pComm->hC, SETXOFF))
		return __CommClearError(pComm,COMM_ERR_SYSTEM);

	return COMM_SUCCESS;
}

/*  CommSetXON
 *
 *  Causes transmission to act as if an XON character has been received.
 *
 *  parameter
 *		hComm				serial port handle
 *		
 *  return
 *		error code
 */
long LVSERIAL_API CommSetXON(HANDLE hComm)
{
	PCOMM pComm;						//pointer to the serial port COMM structure

	_ASSERT(gpCommList);

	//test the given serial port handle and get the pointer to the
	//serial port data structure.
	__CommLock(ghLock);
	if (ListCheckHandle(gpCommList, hComm) != 0)
	{
		__CommUnLock(ghLock);
		return COMM_ERR_HANDLE;
	}

	pComm = (PCOMM)ListGetElement(gpCommList, hComm);
	__CommUnLock(ghLock);

	_ASSERT(pComm);

	//set the XON status
	if (!EscapeCommFunction(pComm->hC, SETXON))
		return __CommClearError(pComm,COMM_ERR_SYSTEM);

	return COMM_SUCCESS;
}

/*  CommSetRTS
 *
 *  Sends the RTS (request-to-send) signal.
 *
 *  parameter
 *		hComm				serial port handle
 *		
 *  return
 *		error code
 */
long LVSERIAL_API CommSetRTS(HANDLE hComm)
{
	PCOMM pComm;						//pointer to the serial port COMM structure

	_ASSERT(gpCommList);

	//test the given serial port handle and get the pointer to the
	//serial port data structure.
	__CommLock(ghLock);
	if (ListCheckHandle(gpCommList, hComm) != 0)
	{
		__CommUnLock(ghLock);
		return COMM_ERR_HANDLE;
	}

	pComm = (PCOMM)ListGetElement(gpCommList, hComm);
	__CommUnLock(ghLock);

	_ASSERT(pComm);

	//send the RTS signal
	if (!EscapeCommFunction(pComm->hC, SETRTS))
		return __CommClearError(pComm,COMM_ERR_SYSTEM);

	return COMM_SUCCESS;
}

/*  CommClrRTS
 *
 *  Clears the RTS (request-to-send) signal.
 *
 *  parameter
 *		hComm				serial port handle
 *		
 *  return
 *		error code
 */
long LVSERIAL_API CommClrRTS(HANDLE hComm)
{
	PCOMM pComm;						//pointer to the serial port COMM structure

	_ASSERT(gpCommList);

	//test the given serial port handle and get the pointer to the
	//serial port data structure.
	__CommLock(ghLock);
	if (ListCheckHandle(gpCommList, hComm) != 0)
	{
		__CommUnLock(ghLock);
		return COMM_ERR_HANDLE;
	}

	pComm = (PCOMM)ListGetElement(gpCommList, hComm);
	__CommUnLock(ghLock);

	_ASSERT(pComm);

	//clear the RTS signal
	if (!EscapeCommFunction(pComm->hC, CLRRTS))
		return __CommClearError(pComm,COMM_ERR_SYSTEM);

	return COMM_SUCCESS;
}

/*  CommSetDTR
 *
 *  Sends the DTR (data-terminal-ready) signal.
 *
 *  parameter
 *		hComm				serial port handle
 *		
 *  return
 *		error code
 */
long LVSERIAL_API CommSetDTR(HANDLE hComm)
{
	PCOMM pComm;						//pointer to the serial port COMM structure

	_ASSERT(gpCommList);

	//test the given serial port handle and get the pointer to the
	//serial port data structure.
	__CommLock(ghLock);
	if (ListCheckHandle(gpCommList, hComm) != 0)
	{
		__CommUnLock(ghLock);
		return COMM_ERR_HANDLE;
	}

	pComm = (PCOMM)ListGetElement(gpCommList, hComm);
	__CommUnLock(ghLock);

	_ASSERT(pComm);

	//send the DTR signal
	if (!EscapeCommFunction(pComm->hC, SETDTR))
		return __CommClearError(pComm,COMM_ERR_SYSTEM);

	return COMM_SUCCESS;
}

/*  CommClrDTR
 *
 *  Clears the DTR (data-terminal-ready) signal.
 *
 *  parameter
 *		hComm				serial port handle
 *		
 *  return
 *		error code
 */
long LVSERIAL_API CommClrDTR(HANDLE hComm)
{
	PCOMM pComm;						//pointer to the serial port COMM structure

	_ASSERT(gpCommList);

	//test the given serial port handle and get the pointer to the
	//serial port data structure.
	__CommLock(ghLock);
	if (ListCheckHandle(gpCommList, hComm) != 0)
	{
		__CommUnLock(ghLock);
		return COMM_ERR_HANDLE;
	}

	pComm = (PCOMM)ListGetElement(gpCommList, hComm);
	__CommUnLock(ghLock);

	_ASSERT(pComm);

	//clear the DTR signal
	if (!EscapeCommFunction(pComm->hC, CLRDTR))
		return __CommClearError(pComm,COMM_ERR_SYSTEM);

	return COMM_SUCCESS;
}

/*  CommSetBreak
 *
 *  Suspends character transmission and places the transmission line in a break state 
 *  until the CommClearBreak function is called.
 *
 *  parameter
 *		hComm				serial port handle
 *		
 *  return
 *		error code
 */
long LVSERIAL_API CommSetBreak(HANDLE hComm)
{
	PCOMM pComm;						//pointer to the serial port COMM structure

	_ASSERT(gpCommList);

	//test the given serial port handle and get the pointer to the
	//serial port data structure.
	__CommLock(ghLock);
	if (ListCheckHandle(gpCommList, hComm) != 0)
	{
		__CommUnLock(ghLock);
		return COMM_ERR_HANDLE;
	}

	pComm = (PCOMM)ListGetElement(gpCommList, hComm);
	__CommUnLock(ghLock);

	_ASSERT(pComm);

	//send the break signal
	if (!EscapeCommFunction(pComm->hC, SETBREAK))
	{
		__CommUnLock(pComm->hInternalLock);
		return __CommClearError(pComm,COMM_ERR_SYSTEM);
	}

	return COMM_SUCCESS;
}

/*  CommClearBreak
 *
 *  Restores character transmission and places the transmission line in a nonbreak 
 *  state.
 *
 *  parameter
 *		hComm				serial port handle
 *		
 *  return
 *		error code
 */
long LVSERIAL_API CommClearBreak(HANDLE hComm)
{
	PCOMM pComm;						//pointer to the serial port COMM structure

	_ASSERT(gpCommList);

	//test the given serial port handle and get the pointer to the
	//serial port data structure.
	__CommLock(ghLock);
	if (ListCheckHandle(gpCommList, hComm) != 0)
	{
		__CommUnLock(ghLock);
		return COMM_ERR_HANDLE;
	}

	pComm = (PCOMM)ListGetElement(gpCommList, hComm);
	__CommUnLock(ghLock);

	_ASSERT(pComm);

	//clear the break signal
	if (!EscapeCommFunction(pComm->hC, CLRBREAK))
	{
		__CommUnLock(pComm->hInternalLock);
		return __CommClearError(pComm,COMM_ERR_SYSTEM);
	}

	return COMM_SUCCESS;
}

/*  CommConfig
 *
 *  Sets the basic serial port communication parameters.
 *
 *  parameter
 *		hComm				serial port handle
 *		ulBaudRate			baud rate
 *		usDataBits			number of data bits
 *		usParity			parity (see COMM_PARITY_ )
 *		usStopBits			number of stop bits (COMM_ONESTOPBIT, COMM_ONE5STOPBITS,
 *							COMM_TWOSTOPBITS or one od thze values 10, 15 or 20).
 *		
 *  return
 *		error code
 */
long LVSERIAL_API CommConfig(
	HANDLE hComm, 
	unsigned long ulBaudRate,
	unsigned short usDataBits,
	unsigned short usParity,
	unsigned short usStopBits)
{
	DCB dcb;							//DCB structure
	PCOMM pComm;						//pointer to the serial port COMM structure

	_ASSERT(gpCommList);

	//test the given serial port handle and get the pointer to the
	//serial port data structure.
	__CommLock(ghLock);
	if (ListCheckHandle(gpCommList, hComm) != 0)
	{
		__CommUnLock(ghLock);
		return COMM_ERR_HANDLE;
	}

	pComm = (PCOMM)ListGetElement(gpCommList, hComm);
	__CommUnLock(ghLock);

	_ASSERT(pComm);

	//modify the dcb structure. At first, we lock the device, because no
	//other function should modify the DCB during this operation.
	__CommLock(pComm->hInternalLock);
	if (!GetCommState(pComm->hC, &dcb))
	{
		__CommUnLock(pComm->hInternalLock);
		return __CommClearError(pComm,COMM_ERR_SYSTEM);
	}

	dcb.BaudRate = ulBaudRate;
	dcb.ByteSize = (unsigned char)usDataBits;
	dcb.Parity = (unsigned char)usParity;
	dcb.fParity = (usParity == COMM_PARITY_NONE) ? FALSE : TRUE; 

	switch (usStopBits)
	{
		case 10:
			usStopBits = COMM_ONESTOPBIT;
			break;
		case 15:
			usStopBits = COMM_ONE5STOPBITS;
			break;
		case 20:
			usStopBits = COMM_TWOSTOPBITS;
			break;
	}

	dcb.StopBits = (unsigned char)usStopBits;
    
	if (!SetCommState(pComm->hC, &dcb))
	{
		__CommUnLock(pComm->hInternalLock);
		return __CommClearError(pComm,COMM_ERR_ARGUMENT);
	}

	__CommUnLock(pComm->hInternalLock);

	return COMM_SUCCESS;
}

/*  CommFlowControl
 *
 *  Configures the basic flow control settings for the serial port.
 *
 *  parameter
 *		hComm				serial port handle
 *		ulFlowControl		flow control mode (see COMM_FLOW_)
 *		ucXOFFCharacter		XOFF character
 *		ucXONCharacter		XON character
 *		
 *  return
 *		error code
 */
long LVSERIAL_API CommFlowControl(
	HANDLE hComm,
	unsigned short usFlowControl,
	unsigned char ucXOFFCharacter,
	unsigned char ucXONCharacter)
{
	DCB dcb;							//DCB structure
	PCOMM pComm;						//pointer to the serial port COMM structure

	_ASSERT(gpCommList);

	//test the given serial port handle and get the pointer to the
	//serial port data structure.
	__CommLock(ghLock);
	if (ListCheckHandle(gpCommList, hComm) != 0)
	{
		__CommUnLock(ghLock);
		return COMM_ERR_HANDLE;
	}

	pComm = (PCOMM)ListGetElement(gpCommList, hComm);
	__CommUnLock(ghLock);

	_ASSERT(pComm);

	//modify the dcb structure. At first, we lock the device, because no
	//other function should modify the DCB during this operation.
	__CommLock(pComm->hInternalLock);

	//get the DCB
	if (!GetCommState(pComm->hC, &dcb))
	{
		__CommUnLock(pComm->hInternalLock);
		return __CommClearError(pComm,COMM_ERR_SYSTEM);
	}

	switch (usFlowControl)
	{
		case COMM_FLOW_NONE:	//no flow control
			dcb.fInX = FALSE;
			dcb.fOutX = FALSE;
			dcb.fOutxCtsFlow = FALSE;
			dcb.fRtsControl = RTS_CONTROL_DISABLE;
			dcb.fOutxDsrFlow = FALSE;
			dcb.fDtrControl = DTR_CONTROL_DISABLE;
			break;

		case COMM_FLOW_XONXOFF:	//XON/XOFF flow control
			dcb.fInX = TRUE;
			dcb.fOutX = TRUE;
			dcb.fOutxCtsFlow = FALSE;
			dcb.fRtsControl = RTS_CONTROL_DISABLE;
			dcb.fOutxDsrFlow = FALSE;
			dcb.fDtrControl = DTR_CONTROL_DISABLE;
			break;

		case COMM_FLOW_RTSCTS:	//RTS/CTS flow control
			dcb.fInX = FALSE;
			dcb.fOutX = FALSE;
			dcb.fOutxCtsFlow = TRUE;
			dcb.fRtsControl = RTS_CONTROL_HANDSHAKE;
			dcb.fOutxDsrFlow = FALSE;
			dcb.fDtrControl = DTR_CONTROL_DISABLE;
			break;

		case COMM_FLOW_DTRDSR:	//DTR/DSR flow control
			dcb.fInX = FALSE;
			dcb.fOutX = FALSE;
			dcb.fOutxCtsFlow = FALSE;
			dcb.fRtsControl = RTS_CONTROL_DISABLE;
			dcb.fOutxDsrFlow = TRUE;
			dcb.fDtrControl = DTR_CONTROL_HANDSHAKE;
			break;
	}

	//store the XON and XOFF characters
	dcb.XoffChar = ucXOFFCharacter;
	dcb.XonChar = ucXONCharacter;
    
	//set the modified DCB
	if (!SetCommState(pComm->hC, &dcb))
	{
		__CommUnLock(pComm->hInternalLock);
		return __CommClearError(pComm,COMM_ERR_ARGUMENT);
	}

	//unlock
	__CommUnLock(pComm->hInternalLock);

	return COMM_SUCCESS;
}    

/*  CommControl
 *
 *  Extended flow and error handling control configuration
 *
 *  parameter
 *		hComm				serial port handle
 *		lFlags				specifies the handshake and error settings
 *							use the COMM_CTRL_ flags
 *		XonLim				Minimum number of bytes allowed in the input 
 *							buffer before flow control is activated to 
 *							inhibit the sender. Note that the sender may
 *							transmit characters after the flow control 
 *							signal has been activated, so this value should 
 *							never be zero.
 *		XoffLim				Maximum number of bytes allowed in the input 
 *							buffer before flow control is activated to 
 *							allow transmission by the sender.
 *		XonChar				Value of the XON character for both 
 *							transmission and reception.
 *		XoffChar			Value of the XOFF character for both 
 *							transmission and reception.
 *		ErrorChar			Value of the character used to replace bytes 
 *							received with a parity error. 
 *		
 *  return
 *		error code
 */
long LVSERIAL_API CommControl(
	HANDLE hComm,
	unsigned long ulFlags,
	unsigned short usXonLim,
	unsigned short usXoffLim,
	unsigned char ucXonChar,
	unsigned char ucXoffChar,
	unsigned char ucErrorChar)
{
	DCB dcb;							//DCB structure
	PCOMM pComm;						//pointer to the serial port COMM structure

	_ASSERT(gpCommList);

	//test the given serial port handle and get the pointer to the
	//serial port data structure.
	__CommLock(ghLock);
	if (ListCheckHandle(gpCommList, hComm) != 0)
	{
		__CommUnLock(ghLock);
		return COMM_ERR_HANDLE;
	}

	pComm = (PCOMM)ListGetElement(gpCommList, hComm);
	__CommUnLock(ghLock);

	_ASSERT(pComm);

	//modify the dcb structure. At first, we lock the device, because no
	//other function should modify the DCB during this operation.
	__CommLock(pComm->hInternalLock);
	if (!GetCommState(pComm->hC, &dcb))
	{
		__CommUnLock(pComm->hInternalLock);
		return __CommClearError(pComm,COMM_ERR_SYSTEM);
	}

	if (ulFlags & COMM_CTRL_OUTXCTSFLOW)
		dcb.fOutxCtsFlow = 1;
	else
		dcb.fOutxCtsFlow = 0;

	if (ulFlags & COMM_CTRL_OUTXDSRFLOW)
		dcb.fOutxDsrFlow = 1;
	else
		dcb.fOutxDsrFlow = 0;

	if (ulFlags & COMM_CTRL_DTRENABLE)
		dcb.fDtrControl = DTR_CONTROL_ENABLE;
	else if (ulFlags & COMM_CTRL_DTRHANDSHAKE)
		dcb.fDtrControl = DTR_CONTROL_HANDSHAKE;
	else
		dcb.fDtrControl = DTR_CONTROL_DISABLE;

	if (ulFlags & COMM_CTRL_DSRSENSITIVITY)
       	dcb.fDsrSensitivity = 1;
	else
		dcb.fDsrSensitivity = 0;

	if (ulFlags & COMM_CTRL_TXCONTINUEONXOFF)
		dcb.fTXContinueOnXoff = 1;
	else
		dcb.fTXContinueOnXoff = 0;

	if (ulFlags & COMM_CTRL_OUTX)
		dcb.fOutX = 1;
	else
		dcb.fOutX = 0;

	if (ulFlags & COMM_CTRL_INX)
		dcb.fInX = 1;
	else
		dcb.fInX = 0;

	if (ulFlags & COMM_CTRL_ERRORCHAR)
		dcb.fErrorChar = 1;
	else
		dcb.fErrorChar = 0;

	if (ulFlags & COMM_CTRL_NULL)
		dcb.fNull = 1;
	else
		dcb.fNull = 0;

	if (ulFlags & COMM_CTRL_RTSENABLE)
		dcb.fRtsControl = RTS_CONTROL_ENABLE;
	else if (ulFlags & COMM_CTRL_RTSHANDSHAKE)
		dcb.fRtsControl = RTS_CONTROL_HANDSHAKE;
	else if (ulFlags & COMM_CTRL_RTSTOGGLE)
		dcb.fRtsControl = RTS_CONTROL_TOGGLE;
	else
		dcb.fRtsControl = RTS_CONTROL_DISABLE;

	if (ulFlags & COMM_CTRL_ABORTONERROR)
		dcb.fAbortOnError = 1;
	else
		dcb.fAbortOnError = 0;

	dcb.XonLim = usXonLim;
	dcb.XoffLim = usXoffLim;
	dcb.XonChar = ucXonChar;
	dcb.XoffChar = ucXoffChar;
	dcb.ErrorChar = ucErrorChar;
    
	if (!SetCommState(pComm->hC, &dcb))
	{
		__CommUnLock(pComm->hInternalLock);
		return __CommClearError(pComm,COMM_ERR_ARGUMENT);
	}

	__CommUnLock(pComm->hInternalLock);

	return COMM_SUCCESS;
}

/*  CommTermination
 *
 *  Configures the termiation characters for the serial port.
 *
 *  parameter
 *		hComm				serial port handle
 *		fTermCharEnable		specifies weather the termination characters are used
 *							to terminate any read operation.
 *		pcTermCharacter		buffer containing the termination characters
 *		lNumberOfTermChar	number of termination characters in pcTermCharacter
 *		
 *  return
 *		error code
 */
long LVSERIAL_API CommTermination(
	HANDLE hComm, 
	int fTermCharEnable,
	char *pcTermCharacter,
	long lNumberOfTermChar)
{
	PCOMM pComm;						//pointer to the serial port COMM structure
	DCB dcb;							//DCB data structure

	_ASSERT(gpCommList);

	//test the given serial port handle and get the pointer to the
	//serial port data structure.
	__CommLock(ghLock);
	if (ListCheckHandle(gpCommList, hComm) != 0)
	{
		__CommUnLock(ghLock);
		return COMM_ERR_HANDLE;
	}

	pComm = (PCOMM)ListGetElement(gpCommList, hComm);
	__CommUnLock(ghLock);

	_ASSERT(pComm);

	if (fTermCharEnable)
	{
		//any of the given characters at pcTermCharacter terminates 
		//the read operation

		if (pcTermCharacter == NULL || lNumberOfTermChar == 0)
			return COMM_ERR_ARGUMENT;

		//enable termination
		pComm->lTeminationMode = COMM_TERM_CHAR;

		//if we have some bytes allocated for a previously used
		//termination string, we release this buffer and
		if (pComm->pcTermination != NULL)
			free(pComm->pcTermination);

		//allocating a new buffer for the termination string
		if ((pComm->pcTermination = (char *)malloc(lNumberOfTermChar+1)) == NULL)
		{
			pComm->lNumberOfTermChar = 0L;
			pComm->lTeminationMode = COMM_TERM_NONE;
			return COMM_ERR_OUTOFMEMORY;
		}
        //and copy the termination string into this buffer
		memcpy(pComm->pcTermination, pcTermCharacter, lNumberOfTermChar);
		*(pComm->pcTermination+lNumberOfTermChar) = '\0';
		pComm->lNumberOfTermChar = lNumberOfTermChar;
	}
	else
	{
		//No termincation mode selected

		//set the termination mode
		pComm->lTeminationMode = COMM_TERM_NONE;

		//there are also no termination characters
		pComm->lNumberOfTermChar = 0;
		//if there are still some bytes allocated for the termination
		//string, we free this buffer.
		if (pComm->pcTermination != NULL)
		{
			free(pComm->pcTermination);
			pComm->pcTermination = NULL;
		}
	}

	//if there is only one termination character, we change
	//the DCB termination character.
	if (lNumberOfTermChar == 1)
	{
		//modify the dcb structure. At first, we lock the device, because no
		//other function should modify the DCB during this operation.
		__CommLock(pComm->hInternalLock);
		if (!GetCommState(pComm->hC, &dcb))
		{
			__CommUnLock(pComm->hInternalLock);
			return __CommClearError(pComm,COMM_ERR_SYSTEM);
		}

		dcb.EofChar = *pcTermCharacter;
        if (!SetCommState(pComm->hC, &dcb))
		{
			__CommUnLock(pComm->hInternalLock);
			return __CommClearError(pComm,COMM_ERR_SYSTEM);
		}
		__CommUnLock(pComm->hInternalLock);
	}

	return COMM_SUCCESS;
}

/*  CommTermination2
 *
 *  Configures the termiation characters for the serial port.
 *
 *  parameter
 *		hComm				serial port handle
 *		lTerminationMode	specifies the termination mode, this can be one of the
 *							following value:
 *								COMM_TERM_NONE		no termination
 *								COMM_TERM_CHAR		one or more termination characters
 *								COMM_TERM_STRING	a single termination string
 *								COMM_TERM_REGEX		a regular expression
 *		pcTermination		buffer containing the termination characters or string
 *		lNumberOfTermChar	number of characters in pcTermination
 *		
 *  return
 *		error code
 */
long LVSERIAL_API CommTermination2(
	HANDLE hComm, 
	long lTerminationMode,
	char *pcTermination,
	long lNumberOfTermChar)
{
	DCB dcb;							//DCB data structure
	PCOMM pComm;						//pointer to the serial port COMM structure
	int iErrPos;						//error position in the regex when copiling
	char * pszErrMsg;					//error message of the regex functions

	_ASSERT(gpCommList);

	//test the given serial port handle and get the pointer to the
	//serial port data structure.
	__CommLock(ghLock);
	if (ListCheckHandle(gpCommList, hComm) != 0)
	{
		__CommUnLock(ghLock);
		return COMM_ERR_HANDLE;
	}

	pComm = (PCOMM)ListGetElement(gpCommList, hComm);
	__CommUnLock(ghLock);

	_ASSERT(pComm);

	//free the regex data if allocated
	if (pComm->RegEx != NULL)
	{
		pcre_free(pComm->RegEx);
		pComm->RegEx = NULL;
	}
	if (pComm->RegExExtra != NULL)
	{
		pcre_free(pComm->RegExExtra);
		pComm->RegExExtra = NULL;
	}

	if (lTerminationMode != COMM_TERM_NONE)
	{
		//read termination is enabled. At first, we test the
		//function parameters for the termination settings and
		//copy the termination characters and the number of 
		//termination characters into the COMM structure.

		if (pcTermination == NULL || lNumberOfTermChar == 0)
			return COMM_ERR_ARGUMENT;

		//store the termination mode
		pComm->lTeminationMode = lTerminationMode;

		if (pComm->pcTermination != NULL)
			free(pComm->pcTermination);

		if ((pComm->pcTermination = (char *)malloc(lNumberOfTermChar+1)) == NULL)
		{
			pComm->lNumberOfTermChar = 0L;
			pComm->lTeminationMode = COMM_TERM_NONE;
			return COMM_ERR_OUTOFMEMORY;
		}
        
		memcpy(pComm->pcTermination, pcTermination, lNumberOfTermChar);
		*(pComm->pcTermination+lNumberOfTermChar) = '\0';
		pComm->lNumberOfTermChar = lNumberOfTermChar;

		//In case of the termination mode COMM_TERM_REGEX we have to
		//compile the regular expression. We also try to get the RegEx
		//extra data to speed up the regular expression match operation.
		if (lTerminationMode == COMM_TERM_REGEX)
		{
		   	pComm->RegEx = pcre_compile2(
				pComm->pcTermination,
				0,
				NULL,
				(const char **)&pszErrMsg,
				&iErrPos,
				NULL);

			if (pComm->RegEx == NULL)
			{
				pComm->lTeminationMode = COMM_TERM_NONE;
				return COMM_ERR_ARGUMENT;
			}

			pComm->RegExExtra = pcre_study(pComm->RegEx, 0, (const char **)&pszErrMsg);

			if (pszErrMsg != NULL)
			{
				pcre_free(pComm->RegEx);
				pComm->RegEx = NULL;
				return COMM_ERR_ARGUMENT;
			}
		}
	}
	else
	{
		//no termination mode selected

		//store the termination mode
		pComm->lTeminationMode = COMM_TERM_NONE;

		//read termination is disabled and we can free the memory for the
		//termination characters if they where allocated previously.
		pComm->lNumberOfTermChar = 0;
		if (pComm->pcTermination != NULL)
		{
			free(pComm->pcTermination);
			pComm->pcTermination = NULL;
		}
	}

	//if there is only one termination character, we change
	//the DCB termination character.
	if (lNumberOfTermChar == 1 && 
		lTerminationMode != COMM_TERM_REGEX)
	{
		//modify the dcb structure. At first, we lock the device, because no
		//other function should modify the DCB during this operation.
		__CommLock(pComm->hInternalLock);
		if (!GetCommState(pComm->hC, &dcb))
		{
			__CommUnLock(pComm->hInternalLock);
			return __CommClearError(pComm,COMM_ERR_SYSTEM);
		}

		dcb.EofChar = *pcTermination;

        if (!SetCommState(pComm->hC, &dcb))
		{
			__CommUnLock(pComm->hInternalLock);
			return __CommClearError(pComm,COMM_ERR_SYSTEM);
		}
		__CommUnLock(pComm->hInternalLock);
	}

	return COMM_SUCCESS;
}

/*  CommWriteMode
 *
 *  Configures the write mode. The transmission of a start of transmission
 *	and/or end of transmission string.
 *
 *  parameter
 *		hComm					serial port handle
 *		lWriteMode				sprecifies the write mode
 *									COMM_WRITE_NORMAL
 *									COMM_WRITE_STX
 *									COMM_WRITE_ETX
 *									COMM_WRITE_STXETX
 *		pcStartOfTransmission	buffer containing the start of transmission string
 *		lNumberOfStartChars		number of characters in pcStartOfTransmission
 *		pcEndOfTransmission		buffer containing the end of transmission string
 *		lNumberOfEndChars		number of characters in pcStartOfTransmission
 *		
 *  return
 *		error code
 */
long LVSERIAL_API CommWriteMode(
	HANDLE hComm, 
	long lWriteMode,
	char *pcStartOfTransmission,
	long lNumberOfStartChars,
	char *pcEndOfTransmission,
	long lNumberOfEndChars)
{
	PCOMM pComm;						//pointer to the serial port COMM structure

	_ASSERT(gpCommList);

	//test the given serial port handle and get the pointer to the
	//serial port data structure.
	__CommLock(ghLock);
	if (ListCheckHandle(gpCommList, hComm) != 0)
	{
		__CommUnLock(ghLock);
		return COMM_ERR_HANDLE;
	}

	pComm = (PCOMM)ListGetElement(gpCommList, hComm);
	__CommUnLock(ghLock);

	_ASSERT(pComm);

	if (lWriteMode > COMM_WRITE_STXETX)
		return COMM_ERR_ARGUMENT;

	pComm->lWriteMode = lWriteMode;

	//In case of the normal write mode (no special write handling), we
	//free any previously allocated memory for the STX and ETX strings
	if (lWriteMode == COMM_WRITE_NORMAL)
	{
		if (pComm->pcStartOfTransmission != NULL)
		{
			free(pComm->pcStartOfTransmission);
			pComm->pcStartOfTransmission = NULL;
			pComm->lNumberOfStartChars = 0;
		}

		if (pComm->pcEndOfTransmission != NULL)
		{
			free(pComm->pcEndOfTransmission);
			pComm->pcEndOfTransmission = NULL;
			pComm->lNumberOfEndChars = 0;
		}

		return COMM_SUCCESS;
	}

	//one or more of the special write modes are enabled

	if (lWriteMode & COMM_WRITE_STX)
	{
		/* the pcStartOfTransmission character should be transmitted at the
		 * beginning of each write operation. We copy the STX string into the
		 * COMM structure. In case of invalid parameters we change the write
		 * mode to COMM_WRITE_NORMAL.
		 */

		if (pcStartOfTransmission == NULL || lNumberOfStartChars == 0)
			return COMM_ERR_ARGUMENT;

		if (pComm->pcStartOfTransmission != NULL)
			free(pComm->pcStartOfTransmission);

		if ((pComm->pcStartOfTransmission = (char *)malloc(lNumberOfStartChars+1)) == NULL)
		{
			pComm->lNumberOfStartChars = 0L;
			lWriteMode = COMM_WRITE_NORMAL;
			return COMM_ERR_OUTOFMEMORY;
		}
        
		memcpy(pComm->pcStartOfTransmission, pcStartOfTransmission, lNumberOfStartChars);
		*(pComm->pcStartOfTransmission+lNumberOfStartChars) = '\0';
		pComm->lNumberOfStartChars = lNumberOfStartChars;
	}

	if (lWriteMode & COMM_WRITE_ETX)
	{
		/* the pcEndOfTransmission character should be transmitted at the
		 * end of each write operation. We copy the ETX string into the
		 * COMM structure. In case of invalid parameters we change the write
		 * mode to COMM_WRITE_NORMAL.
		 */

		if (pcEndOfTransmission == NULL || lNumberOfEndChars == 0)
			return COMM_ERR_ARGUMENT;

		if (pComm->pcEndOfTransmission != NULL)
			free(pComm->pcEndOfTransmission);

		if ((pComm->pcEndOfTransmission = (char *)malloc(lNumberOfEndChars+1)) == NULL)
		{
			pComm->lNumberOfEndChars = 0L;
			lWriteMode = COMM_WRITE_NORMAL;
			return COMM_ERR_OUTOFMEMORY;
		}
        
		memcpy(pComm->pcEndOfTransmission, pcEndOfTransmission, lNumberOfEndChars);
		*(pComm->pcEndOfTransmission+lNumberOfEndChars) = '\0';
		pComm->lNumberOfEndChars = lNumberOfEndChars;
	}

	return COMM_SUCCESS;
}

/*  CommTimeout
 *
 *  Configures the timeout value for read and write operations. This function only
 *	sets the total timeour constant values. For more advaced settings the function
 *	CommTimeout2 can be used.
 *
 *  parameter
 *		hComm				serial port handle
 *		ulTimeout			timeout in milliseconds
 *		
 *  return
 *		error code
 */
long LVSERIAL_API CommTimeout(
	HANDLE hComm, 
	unsigned long ulTimeout)
{
	COMMTIMEOUTS Timeouts;				//COMMTIMEOUTS structure
	PCOMM pComm;						//pointer to the serial port COMM structure

	_ASSERT(gpCommList);

	//test the given serial port handle and get the pointer to the
	//serial port data structure.
	__CommLock(ghLock);
	if (ListCheckHandle(gpCommList, hComm) != 0)
	{
		__CommUnLock(ghLock);
		return COMM_ERR_HANDLE;
	}

	pComm = (PCOMM)ListGetElement(gpCommList, hComm);
	__CommUnLock(ghLock);

	_ASSERT(pComm);

	pComm->ulReadIntervalTimeout = 0;
	pComm->ulReadTotalTimeoutConstant = ulTimeout;
	pComm->ulReadTotalTimeoutMultiplier = 0;
	pComm->ulWriteTotalTimeoutConstant = ulTimeout;
	pComm->ulWriteTotalTimeoutMultiplier = 0;

	Timeouts.ReadIntervalTimeout = 0;
	Timeouts.ReadTotalTimeoutConstant = ulTimeout;
	Timeouts.ReadTotalTimeoutMultiplier = 0;
	Timeouts.WriteTotalTimeoutConstant = ulTimeout;
	Timeouts.WriteTotalTimeoutMultiplier = 0;
	
	if (!SetCommTimeouts(pComm->hC, &Timeouts))
		return __CommClearError(pComm,COMM_ERR_ARGUMENT);

	return COMM_SUCCESS;
}

/*  CommTimeout2
 *
 *  Configures the timeout value for read and write operations. Note that the 
 *	CommRead function does not take care about some of these settings if any
 *	of the read termination modes are enabled.
 *
 *  parameter
 *		hComm							serial port handle
 *		ulReadIntervalTimeout			timeout in milliseconds between two characters
 *		ulReadTotalTimeoutMultiplier	timeout value for each requested character
 *		ulReadTotalTimeoutConstant		timeout constant
 *		ulWriteTotalTimeoutMultiplier	timeout value for each character to write
 *		ulWriteTotalTimeoutConstant		timeout constant
 *
 *  return
 *		error code
 */
long LVSERIAL_API CommTimeout2(
	HANDLE hComm, 
	unsigned long ulReadIntervalTimeout,
	unsigned long ulReadTotalTimeoutMultiplier,
	unsigned long ulReadTotalTimeoutConstant,
	unsigned long ulWriteTotalTimeoutMultiplier,
	unsigned long ulWriteTotalTimeoutConstant)
{
	COMMTIMEOUTS Timeouts;				//COMMTIMEOUTS structure
	PCOMM pComm;						//pointer to the serial port COMM structure

	_ASSERT(gpCommList);

	//test the given serial port handle and get the pointer to the
	//serial port data structure.
	__CommLock(ghLock);
	if (ListCheckHandle(gpCommList, hComm) != 0)
	{
		__CommUnLock(ghLock);
		return COMM_ERR_HANDLE;
	}

	pComm = (PCOMM)ListGetElement(gpCommList, hComm);
	__CommUnLock(ghLock);

	_ASSERT(pComm);

	pComm->ulReadIntervalTimeout = ulReadIntervalTimeout;
	pComm->ulReadTotalTimeoutMultiplier = ulReadTotalTimeoutMultiplier;
	pComm->ulReadTotalTimeoutConstant = ulReadTotalTimeoutConstant;
	pComm->ulWriteTotalTimeoutMultiplier = ulWriteTotalTimeoutMultiplier;
	pComm->ulWriteTotalTimeoutConstant = ulWriteTotalTimeoutConstant;

	Timeouts.ReadIntervalTimeout = ulReadIntervalTimeout;
	Timeouts.ReadTotalTimeoutMultiplier = ulReadTotalTimeoutMultiplier;
	Timeouts.ReadTotalTimeoutConstant = ulReadTotalTimeoutConstant;
	Timeouts.WriteTotalTimeoutMultiplier = ulWriteTotalTimeoutMultiplier;
	Timeouts.WriteTotalTimeoutConstant = ulWriteTotalTimeoutConstant;
	
	if (!SetCommTimeouts(pComm->hC, &Timeouts))
		return __CommClearError(pComm,COMM_ERR_ARGUMENT);

	return COMM_SUCCESS;
}

/*  CommOptions
 *
 *  Sets the serial port options. This function is also called inside
 *  the CommOpen function and allows the user to specify the major 
 *	settings together with the serial port name in one string.
 *
 *  parameter
 *		hComm				serial port handle
 *		pszOptions			serial port options string
 *		
 *  return
 *		error code
 *
 *	Options specification
 *	There are two generic different formats of this string:
 *
 *	<baud>,<parity>,<data>,<stop>[,<handshake>]
 *
 *	where
 *		baud		any valid baudrate
 *		partity		N=none, O=odd, E=even, M=mark, S=space
 *		data		is the number of data bits
 *		stop		is the number of stop bits (1, 1.5 or 2)
 *					valid values are also 10, 15 and 20)
 *		handshake	either x (XON/XOFF) or p (DTR/DSR and RTS/CTS handshake)
 *
 *
 *	baudrate:<baud>; parity:<parity>; handshake:<hand>; 
 *  databits=<data>; stopbits:<stop>; timeout:<timeout>
 *
 *	where
 *		baud		any valid baudrate
 *		partity		N=none, O=odd, E=even, M=mark, S=space
 *					(only the first character is tested)
 *		handshake	N=none, X=xonxoff, R=rstcts, D=dsrdtr
 *					(only the first character is tested)
 *		data		is the number of data bits
 *		stop		is the number of stop bits (1, 1.5 or 2)
 *					valid values are also 10, 15 and 20)
 *		timeout		is the read and write timeout in milliseconds. You can
 *					also use "TO" instead of the "timeout" keyword.
 *		xonchar		specifies the XON character
 *		xoffchar	specifies the XOFF character
 *		xon			enables (on) or disables (off) the XON/XOFF handshake
 *		odsr		Indicates whether the DSR (data-set-ready) signal is 
 *					monitored for output flow control. Possible values 
 *					are on or off.
 *		octs		Indicates whether the CTS (clear-to-send) signal is 
 *					monitored for output flow control. Possible values
 *					are on or off.
 *		dtr			sets the DTR flow control mode to on, off or hs (=handshake)
 *		rts			sets the RTS flow control mode to on, off, hs (=handshake) 
 *					or ts (=toggle)
 *		idsr		this value can be on or off. If it is on, the driver
 *					is sensitive to the DSR signal. The driver only reveives
 *					bytes, if the DSR input line is high.
 *
 *	String and character compirison is not case sensitive. The second
 *	format is less strict, so you can place whitespaces between the 
 *	keywords and values. You can use "=" instead of ":" and you must't
 *	specify ";" between the values (a single whitespace character is 
 *	enough).
 *
 */
long LVSERIAL_API CommOptions(HANDLE hComm, char *pszOptions)
{
	DCB dcb;							//DCB data structure
	COMMTIMEOUTS Timeouts;				//the OS serial port timeout values structure
	PCOMM pComm;						//pointer to the COMM structure
	long lResult;						//function return code
	double dValue;						//double value of the option
										//because we use regular expressions to get
										//the parameters from the pszOptions string, 
										//we also need the following variables
	pcre *pRegEx;						//compiled regex
	char *pszError;						//regex error message
	int iErrorOffset;					//offset of an error in the regular expression
	int iOptionsLength;					//number of bytes in the subject string
	int iOffset;						//start offset for the match
	int aiOffsets[18];					//found offsets in the subject string
	int iMatches;						//number of found matches
	int boSimpleHandshake;				//simple form with handshake detected

	_ASSERT(gpCommList);

	//test the given serial port handle and get the pointer to the
	//serial port data structure.
	__CommLock(ghLock);
	if (ListCheckHandle(gpCommList, hComm) != 0)
	{
		__CommUnLock(ghLock);
		return COMM_ERR_HANDLE;
	}

	pComm = (PCOMM)ListGetElement(gpCommList, hComm);
	__CommUnLock(ghLock);

	_ASSERT(pComm);

	//if there are not options, we don't have do to anything.
	if (pszOptions == NULL)
		return COMM_SUCCESS;

	//we get the serial port settings, so we can store new settings
	//into this structure. At the end, we set these confuguration with
	//the SetCommState function.

	//modify the dcb structure. At first, we lock the device, because no
	//other function should modify the DCB during this operation.
	__CommLock(pComm->hInternalLock);
	if (!GetCommState(pComm->hC, &dcb))
	{
		__CommUnLock(pComm->hInternalLock);
		return __CommClearError(pComm,COMM_ERR_SYSTEM);
	}

	//At first, we try to detect the simple form with handshake
	//<baudrate>,<parity>,<databits>,<stopbits>,<handshake>
	//This is only used to set the handshake mode.
	pRegEx = pcre_compile(
		"(?i)(?:\\d+),(?:[neoms]),(?:\\d{1,2}),(?:[\\d\\.]{1,4}),([xp])", 
		0,
		(const char **)&pszError,
		&iErrorOffset,
		NULL);

	_ASSERT(pRegEx);
	
	iOptionsLength = (int)strlen(pszOptions);
	iOffset = 0;

	iMatches = pcre_exec(
		pRegEx, 
		NULL, 
		pszOptions, 
		iOptionsLength, 
		iOffset, 
		PCRE_NOTEMPTY, 
		aiOffsets,
		18);

	//if we detect the simple form with handshake, we set the
	//boSimpleHandshake to true and set only the handshake
	//parameters in the DCB.
	if (iMatches > 0)
	{
		//indicates, that the next step should not set the handshake, 
		//because this is already done here. If the handshake mode
		//is invalid, we set the handshake to none.
		boSimpleHandshake = TRUE;

		switch (*(pszOptions+aiOffsets[2]))
		{
		case 'x':
		case 'X':
			dcb.fInX = dcb.fOutX = TRUE;
			dcb.fOutxDsrFlow = dcb.fOutxCtsFlow = FALSE;
			dcb.fDtrControl = DTR_CONTROL_ENABLE;
			dcb.fRtsControl = RTS_CONTROL_ENABLE;
			break;
		case 'p':
		case 'P':
			dcb.fInX = dcb.fOutX = FALSE;
			dcb.fOutxDsrFlow = dcb.fOutxCtsFlow = TRUE;
			dcb.fDtrControl = DTR_CONTROL_HANDSHAKE;
			dcb.fRtsControl = RTS_CONTROL_HANDSHAKE;
			break;
		}
	}
	else
		boSimpleHandshake = FALSE;

	//free the compiled regex
	pcre_free(pRegEx);

	//Second, we try to detect the simple form (ignore the handshake)
	//<baudrate>,<parity>,<databits>,<stopbits>
	pRegEx = pcre_compile(
		"(?i)(\\d+),([neoms]),(\\d{1,2}),([\\d\\.]{1,4})", 
		0,
		(const char **)&pszError,
		&iErrorOffset,
		NULL);

	_ASSERT(pRegEx);
	
	iOptionsLength = (int)strlen(pszOptions);
	iOffset = 0;

	iMatches = pcre_exec(
		pRegEx, 
		NULL, 
		pszOptions, 
		iOptionsLength, 
		iOffset, 
		PCRE_NOTEMPTY, 
		aiOffsets,
		18);

	if (iMatches > 0)
	{
		//the options string contains the simple form.

		if (!boSimpleHandshake)
		{
			dcb.fInX = dcb.fOutX = FALSE;
			dcb.fOutxDsrFlow = dcb.fOutxCtsFlow = FALSE;
			dcb.fDtrControl = DTR_CONTROL_ENABLE;
			dcb.fRtsControl = RTS_CONTROL_ENABLE;
		}

		//store the baud rate
		dcb.BaudRate = atol(pszOptions+aiOffsets[2]);

		//store the parity
		switch (*(pszOptions+aiOffsets[4]))
		{
		case 'n':
		case 'N':
			dcb.fParity = FALSE;
			dcb.Parity = COMM_PARITY_NONE;
			break;
		case 'o':
		case 'O':
			dcb.fParity = TRUE;
			dcb.Parity = COMM_PARITY_ODD;
			break;
		case 'e':
		case 'E':
			dcb.fParity = TRUE;
			dcb.Parity = COMM_PARITY_EVEN;
			break;
		case 'm':
		case 'M':
			dcb.fParity = TRUE;
			dcb.Parity = COMM_PARITY_MARK;
			break;
		case 's':
		case 'S':
			dcb.fParity = TRUE;
			dcb.Parity = COMM_PARITY_SPACE;
			break;
		}

		//store the number of bytes
		dcb.ByteSize = (unsigned char)atoi(pszOptions+aiOffsets[6]);

		//store the number of stop bits
		dValue = atof(pszOptions+aiOffsets[8]);

		if (dValue == 10.0 || dValue == 1.0)
			dcb.StopBits = COMM_ONESTOPBIT;
		else if (dValue == 15.0 || dValue == 1.5)
			dcb.StopBits = COMM_ONE5STOPBITS;
		else if (dValue == 20.0 || dValue == 2.0)
			dcb.StopBits = COMM_TWOSTOPBITS;
		else
			lResult = COMM_ERR_ARGUMENT;

		//release the compiled regex memory
		pcre_free(pRegEx);

		//finally we set the new configuration
		if (!SetCommState(pComm->hC, &dcb))
		{
			__CommUnLock(pComm->hInternalLock);
			return __CommClearError(pComm,COMM_ERR_ARGUMENT);
		}
		__CommUnLock(pComm->hInternalLock);

		return COMM_SUCCESS;
	}

	//the options string does not match the simple form.

	//free the compiled regex
	pcre_free(pRegEx);

	//compile the regular expression for the complex form of the
	//options string. With this regex, we can find all options in
	//the string. It gives us the keyword and the value.
	pRegEx = pcre_compile(
		"\\w*\\s*(?::|=)\\s*([\\w\\.]*)", 
		0, 
		(const char **)&pszError,
		&iErrorOffset,
		NULL);

	_ASSERT(pRegEx);
	
	//We now have to search for the options starting at the 
	//beginning of the string.

	lResult = COMM_SUCCESS;
    iOffset = 0;

	do
	{
		//search for the next option
		iMatches = pcre_exec(
			pRegEx, 
			NULL, 
			pszOptions, 
			iOptionsLength, 
			iOffset, 
			PCRE_NOTEMPTY, 
			aiOffsets,
			18);

		//any option found?
		if (iMatches >= 0)
		{
			//a new option was found 

			if (_strnicmp(pszOptions+aiOffsets[0], "baud", 4) == 0)
			{
				//set baudrate
				dcb.BaudRate = atol(pszOptions+aiOffsets[2]);
			} 
			else if (_strnicmp(pszOptions+aiOffsets[0], "parity", 6) == 0)
			{
				//set the parity
				switch (*(pszOptions+aiOffsets[2]))
				{
				case 'n':
				case 'N':
					dcb.fParity = FALSE;
					dcb.Parity = COMM_PARITY_NONE;
					break;
				case 'o':
				case 'O':
					dcb.fParity = TRUE;
					dcb.Parity = COMM_PARITY_ODD;
					break;
				case 'e':
				case 'E':
					dcb.fParity = TRUE;
					dcb.Parity = COMM_PARITY_EVEN;
					break;
				case 'm':
				case 'M':
					dcb.fParity = TRUE;
					dcb.Parity = COMM_PARITY_MARK;
					break;
				case 's':
				case 'S':
					dcb.fParity = TRUE;
					dcb.Parity = COMM_PARITY_SPACE;
				default:
					lResult = COMM_ERR_ARGUMENT;
				}
			}
			else if (_strnicmp(pszOptions+aiOffsets[0], "hand", 4) == 0)
			{
				//set the handshake mode
				switch (*(pszOptions+aiOffsets[2]))
				{
				case 'n':
				case 'N':
					dcb.fInX = FALSE;
					dcb.fOutX = FALSE;
					dcb.fOutxCtsFlow = FALSE;
					dcb.fRtsControl = RTS_CONTROL_DISABLE;
					dcb.fOutxDsrFlow = FALSE;
					dcb.fDtrControl = DTR_CONTROL_DISABLE;
					break;
				case 'x':
				case 'X':
					dcb.fInX = TRUE;
					dcb.fOutX = TRUE;
					dcb.fOutxCtsFlow = FALSE;
					dcb.fRtsControl = RTS_CONTROL_ENABLE;
					dcb.fOutxDsrFlow = FALSE;
					dcb.fDtrControl = DTR_CONTROL_ENABLE;
					break;
				case 'r':
				case 'R':
					dcb.fInX = FALSE;
					dcb.fOutX = FALSE;
					dcb.fOutxCtsFlow = TRUE;
					dcb.fRtsControl = RTS_CONTROL_HANDSHAKE;
					dcb.fOutxDsrFlow = FALSE;
					dcb.fDtrControl = DTR_CONTROL_ENABLE;
					break;
				case 'd':
				case 'D':
					dcb.fInX = FALSE;
					dcb.fOutX = FALSE;
					dcb.fOutxCtsFlow = FALSE;
					dcb.fRtsControl = RTS_CONTROL_ENABLE;
					dcb.fOutxDsrFlow = TRUE;
					dcb.fDtrControl = DTR_CONTROL_HANDSHAKE;
					break;
				case 'p':
				case 'P':
					dcb.fInX = FALSE;
					dcb.fOutX = FALSE;
					dcb.fOutxCtsFlow = TRUE;
					dcb.fRtsControl = RTS_CONTROL_HANDSHAKE;
					dcb.fOutxDsrFlow = TRUE;
					dcb.fDtrControl = DTR_CONTROL_HANDSHAKE;
					break;
				default:
					lResult = COMM_ERR_ARGUMENT;
				}
			}
			else if (_strnicmp(pszOptions+aiOffsets[0], "xonc", 3) == 0)
			{
				//set the XON character
				dcb.XonChar = (char)atoi(pszOptions+aiOffsets[2]);
			}
			else if (_strnicmp(pszOptions+aiOffsets[0], "xoffc", 4) == 0)
			{
				//set the XOFF character
				dcb.XoffChar = (char)atoi(pszOptions+aiOffsets[2]);
			}
			else if (_strnicmp(pszOptions+aiOffsets[0], "xon", 3) == 0)
			{
				if (_strnicmp(pszOptions+aiOffsets[2],"on",2) == 0)
					dcb.fInX = dcb.fOutX = TRUE;
				else if (_strnicmp(pszOptions+aiOffsets[2],"off",3) == 0)
					dcb.fInX = dcb.fOutX = FALSE;
			}
			else if (_strnicmp(pszOptions+aiOffsets[0], "odsr", 3) == 0)
			{
				if (_strnicmp(pszOptions+aiOffsets[2],"on",2) == 0)
					dcb.fOutxDsrFlow = TRUE;
				else if (_strnicmp(pszOptions+aiOffsets[2],"off",3) == 0)
					dcb.fOutxDsrFlow = FALSE;
			}
			else if (_strnicmp(pszOptions+aiOffsets[0], "octs", 3) == 0)
			{
				if (_strnicmp(pszOptions+aiOffsets[2],"on",2) == 0)
					dcb.fOutxCtsFlow = TRUE;
				else if (_strnicmp(pszOptions+aiOffsets[2],"off",3) == 0)
					dcb.fOutxCtsFlow = FALSE;
			}
			else if (_strnicmp(pszOptions+aiOffsets[0], "dtr", 3) == 0)
			{
				if (_strnicmp(pszOptions+aiOffsets[2],"on",2) == 0)
					dcb.fDtrControl = DTR_CONTROL_ENABLE;
				else if (_strnicmp(pszOptions+aiOffsets[2],"off",3) == 0)
					dcb.fDtrControl = DTR_CONTROL_DISABLE;
				else if (_strnicmp(pszOptions+aiOffsets[2],"hs",2) == 0)
					dcb.fDtrControl = DTR_CONTROL_HANDSHAKE;
			}
			else if (_strnicmp(pszOptions+aiOffsets[0], "rts", 3) == 0)
			{
				if (_strnicmp(pszOptions+aiOffsets[2],"on",2) == 0)
					dcb.fRtsControl = DTR_CONTROL_ENABLE;
				else if (_strnicmp(pszOptions+aiOffsets[2],"off",3) == 0)
					dcb.fRtsControl = DTR_CONTROL_DISABLE;
				else if (_strnicmp(pszOptions+aiOffsets[2],"hs",2) == 0)
					dcb.fRtsControl = DTR_CONTROL_HANDSHAKE;
				else if (_strnicmp(pszOptions+aiOffsets[2],"tg",2) == 0)
					dcb.fRtsControl = RTS_CONTROL_TOGGLE;
			}
			else if (_strnicmp(pszOptions+aiOffsets[0], "idsr", 3) == 0)
			{
				if (_strnicmp(pszOptions+aiOffsets[2],"on",2) == 0)
					dcb.fDsrSensitivity = TRUE;
				else if (_strnicmp(pszOptions+aiOffsets[2],"off",3) == 0)
					dcb.fDsrSensitivity = FALSE;
			}
			else if (_strnicmp(pszOptions+aiOffsets[0], "data", 4) == 0)
			{
				//set the data byte size
				dcb.ByteSize = (unsigned char)atoi(pszOptions+aiOffsets[2]);
			}
			else if (_strnicmp(pszOptions+aiOffsets[0], "stop", 4) == 0)
			{
				//set the stop bit size
				dValue = atof(pszOptions+aiOffsets[2]);

				if (dValue == 10.0 || dValue == 1.0)
					dcb.StopBits = COMM_ONESTOPBIT;
				else if (dValue == 15.0 || dValue == 1.5)
					dcb.StopBits = COMM_ONE5STOPBITS;
				else if (dValue == 20.0 || dValue == 2.0)
					dcb.StopBits = COMM_TWOSTOPBITS;
				else
					lResult = COMM_ERR_ARGUMENT;
			}
			else if (_strnicmp(pszOptions+aiOffsets[0], "timeout", 7) == 0 ||
					 _strnicmp(pszOptions+aiOffsets[0], "to", 7) == 0)
			{
				//set the time out value
				pComm->ulReadTotalTimeoutConstant = 
                pComm->ulWriteTotalTimeoutConstant =
                Timeouts.ReadTotalTimeoutConstant =
                Timeouts.WriteTotalTimeoutConstant = atol(pszOptions+aiOffsets[2]);

				pComm->ulReadIntervalTimeout =
				pComm->ulReadTotalTimeoutMultiplier =
				pComm->ulWriteTotalTimeoutMultiplier =
                Timeouts.ReadIntervalTimeout =	    
				Timeouts.ReadTotalTimeoutMultiplier =
				Timeouts.WriteTotalTimeoutMultiplier = 0;

				if (!SetCommTimeouts(pComm->hC, &Timeouts))
					lResult = COMM_ERR_ARGUMENT;
			}
		}

		//preparation for the search
		iOffset = aiOffsets[1];

	} while (iMatches >= 0);

	//release the compiled regex data
	pcre_free(pRegEx);

	//set the serial port configuration
	if (!SetCommState(pComm->hC, &dcb))
	{
		__CommUnLock(pComm->hInternalLock);
		return __CommClearError(pComm,COMM_ERR_ARGUMENT);
	}
	__CommUnLock(pComm->hInternalLock);

	return lResult;
}

/*  CommLock
 *
 *  Locks the serial port
 *
 *  parameter
 *		hComm				serial port handle
 *		ulTimeout			timeout in milliseconds
 *		
 *  return
 *		error code
 */
long LVSERIAL_API CommLock(HANDLE hComm, unsigned long ulTimeout)
{
	PCOMM pComm;						//pointer to the serial port COMM structure

	_ASSERT(gpCommList);

	//test the given serial port handle and get the pointer to the
	//serial port data structure.
	__CommLock(ghLock);
	if (ListCheckHandle(gpCommList, hComm) != 0)
	{
		__CommUnLock(ghLock);
		return COMM_ERR_HANDLE;
	}

	pComm = (PCOMM)ListGetElement(gpCommList, hComm);
	__CommUnLock(ghLock);

	_ASSERT(pComm);
	
	return __CommLockT(pComm->hLock, ulTimeout);
}

/*  CommLock2
 *
 *  Locks the serial port
 *
 *  parameter
 *		hComm				serial port handle
 *		ulTimeout			timeout in milliseconds
 *		pfLocked			will be set to 1 if the port is locked
 *		
 *  return
 *		error code
 */
long LVSERIAL_API CommLock2(HANDLE hComm, unsigned long ulTimeout, int *pfLocked)
{
	PCOMM pComm;						//pointer to the serial port COMM structure
	long lLockResult;

	_ASSERT(gpCommList);

	//test the given serial port handle and get the pointer to the
	//serial port data structure.
	__CommLock(ghLock);
	if (ListCheckHandle(gpCommList, hComm) != 0)
	{
		__CommUnLock(ghLock);
		return COMM_ERR_HANDLE;
	}

	pComm = (PCOMM)ListGetElement(gpCommList, hComm);
	__CommUnLock(ghLock);

	_ASSERT(pComm);

	lLockResult = __CommLockT(pComm->hLock, ulTimeout);

	if (pfLocked != NULL)
		*pfLocked = (lLockResult == COMM_SUCCESS) ? 1 : 0;
	
	return lLockResult;
}

/*  CommUnLock
 *
 *  Unlocks the serial port
 *
 *  parameter
 *		hComm				serial port handle
 *		
 *  return
 *		error code
 */
long LVSERIAL_API CommUnLock(HANDLE hComm)
{
	PCOMM pComm;						//pointer to the serial port COMM structure

	_ASSERT(gpCommList);

	//test the given serial port handle and get the pointer to the
	//serial port data structure.
	__CommLock(ghLock);
	if (ListCheckHandle(gpCommList, hComm) != 0)
	{
		__CommUnLock(ghLock);
		return COMM_ERR_HANDLE;
	}

	pComm = (PCOMM)ListGetElement(gpCommList, hComm);
	__CommUnLock(ghLock);

	_ASSERT(pComm);

	return __CommUnLock(pComm->hLock);
}

/*  CommUnLock2
 *
 *  Unlocks the serial port
 *
 *  parameter
 *		hComm				serial port handle
 *		fLocked				port will be unlocked only if this value is 1 (TRUE)
 *		
 *  return
 *		error code
 */
long LVSERIAL_API CommUnLock2(HANDLE hComm, int fLocked)
{
	PCOMM pComm;						//pointer to the serial port COMM structure

	if (!fLocked)
		return COMM_SUCCESS;

	_ASSERT(gpCommList);

	//test the given serial port handle and get the pointer to the
	//serial port data structure.
	__CommLock(ghLock);
	if (ListCheckHandle(gpCommList, hComm) != 0)
	{
		__CommUnLock(ghLock);
		return COMM_ERR_HANDLE;
	}

	pComm = (PCOMM)ListGetElement(gpCommList, hComm);
	__CommUnLock(ghLock);

	_ASSERT(pComm);

	return __CommUnLock(pComm->hLock);
}

/*  CommSetUserData
 *
 *  Sets the user specific data for the serial port.
 *
 *  parameter
 *		hComm				serial port handle
 *		ulUserData			user data
 *		
 *  return
 *		error code
 */
long LVSERIAL_API CommSetUserData(HANDLE hComm, unsigned long ulUserData)
{
	PCOMM pComm;						//pointer to the serial port COMM structure

	_ASSERT(gpCommList);

	//test the given serial port handle and get the pointer to the
	//serial port data structure.
	__CommLock(ghLock);
	if (ListCheckHandle(gpCommList, hComm) != 0)
	{
		__CommUnLock(ghLock);
		return COMM_ERR_HANDLE;
	}

	pComm = (PCOMM)ListGetElement(gpCommList, hComm);
	__CommUnLock(ghLock);

	_ASSERT(pComm);

	pComm->ulUserData = ulUserData;

	return COMM_SUCCESS;
}

/*  CommGetUserData
 *
 *  Returns the user specific data for the serial port.
 *
 *  parameter
 *		hComm				serial port handle
 *		pulUserData			user data
 *		
 *  return
 *		error code
 */
long LVSERIAL_API CommGetUserData(HANDLE hComm, unsigned long *pulUserData)
{
	PCOMM pComm;						//pointer to the serial port COMM structure

	_ASSERT(gpCommList);

	//test the given serial port handle and get the pointer to the
	//serial port data structure.
	__CommLock(ghLock);
	if (ListCheckHandle(gpCommList, hComm) != 0)
	{
		__CommUnLock(ghLock);
		if (pulUserData != NULL)
			*pulUserData = 0;
		return COMM_ERR_HANDLE;
	}

	pComm = (PCOMM)ListGetElement(gpCommList, hComm);
	__CommUnLock(ghLock);

	_ASSERT(pComm);

	if (pulUserData != NULL)
		*pulUserData = pComm->ulUserData;

	return COMM_SUCCESS;
}

/*  CommSetUserDataBuffer
 *
 *  Stores the user specific data buffer for the serial port.
 *
 *  parameter
 *		hComm				serial port handle
 *		pcUserData			user data buffer
 *		lUserDataSize		size of user data buffer
 *		
 *  return
 *		error code
 */
long LVSERIAL_API CommSetUserDataBuffer(HANDLE hComm, char *pcUserData, long lUserDataSize)
{
	PCOMM pComm;						//pointer to the serial port COMM structure

	_ASSERT(gpCommList);

	//test the given serial port handle and get the pointer to the
	//serial port data structure.
	__CommLock(ghLock);
	if (ListCheckHandle(gpCommList, hComm) != 0)
	{
		__CommUnLock(ghLock);
		return COMM_ERR_HANDLE;
	}

	pComm = (PCOMM)ListGetElement(gpCommList, hComm);
	__CommUnLock(ghLock);

	_ASSERT(pComm);

	//modify the user data buffer. At first, we lock the device, because no
	//other function should modify the buffer during this operation.
	__CommLock(pComm->hInternalLock);
	if (pComm->pcUserData != NULL)
		free(pComm->pcUserData);

	if (lUserDataSize == 0 || pcUserData == NULL)
	{
		pComm->pcUserData = NULL;
		pComm->lUserDataSize = 0;
	}
	else
	{
		//allocate memory for the user data buffer
		if ((pComm->pcUserData = malloc(lUserDataSize)) == NULL)
		{
			pComm->lUserDataSize = 0;
			__CommUnLock(pComm->hInternalLock);
			return COMM_ERR_OUTOFMEMORY;
		}
		//copy the user data into the new buffer
		memcpy(pComm->pcUserData, pcUserData, lUserDataSize);
		pComm->lUserDataSize = lUserDataSize;
	}
	__CommUnLock(pComm->hInternalLock);

	return COMM_SUCCESS;
}

/*  CommGetUserDataBuffer
 *
 *  Returns the user specific data for the serial port. If pcUserData is NULL,
 *	the function copies the number of user data bytes at plUserDataSize.
 *
 *  parameter
 *		hComm				serial port handle
 *		pcUserData			user data buffer
 *		plUserDataSize		size of the user data buffer
 *		
 *  return
 *		error code
 */
long LVSERIAL_API CommGetUserDataBuffer(HANDLE hComm, char *pcUserData, long *plUserDataSize)
{
	PCOMM pComm;						//pointer to the serial port COMM structure
	long lBytesToCopy;					//number of bytes to copy

	_ASSERT(gpCommList);

	//test the given serial port handle and get the pointer to the
	//serial port data structure.
	__CommLock(ghLock);
	if (ListCheckHandle(gpCommList, hComm) != 0)
	{
		__CommUnLock(ghLock);
		if (plUserDataSize != NULL)
			*plUserDataSize = 0;

		return COMM_ERR_HANDLE;
	}

	pComm = (PCOMM)ListGetElement(gpCommList, hComm);
	__CommUnLock(ghLock);

	_ASSERT(pComm);

	if (pcUserData == NULL)
	{
		if (plUserDataSize == NULL)
			return COMM_ERR_ARGUMENT;

		*plUserDataSize = pComm->lUserDataSize;
		return COMM_SUCCESS;
	}

	//copy the user data buffer. At first, we lock the device, because no
	//other function should modify the buffer during this operation.
	__CommLock(pComm->hInternalLock);

	//If plUserDataSize is a NULL pointer, we cannot store the number of bytes
	//at this location and we assume, that the target buffer is large enough.
	if (plUserDataSize == NULL)
		lBytesToCopy = pComm->lUserDataSize;
	else
		//plUserDataSize is a valid pointer, so we limit the number of bytes to
		//the target buffer size.
		lBytesToCopy = *plUserDataSize = __min(*plUserDataSize, pComm->lUserDataSize);

  	//copy the user data into the target buffer
	memcpy(pcUserData, pComm->pcUserData, lBytesToCopy);

	__CommUnLock(pComm->hInternalLock);

	return COMM_SUCCESS;
}

/*  CommListPorts
 *
 *  Lists all serial ports and places them into a array of pointers to zero
 *  terminated strings. The array and the strings are allocated by this
 *  function and should. Call CommFree or CommFreePortList to release the 
 *	memory.
 *
 *  parameter
 *		pppszPorts			pointer to the array of serial port names
 *		
 *  return
 *		error code
 */
long LVSERIAL_API CommListPorts(char ***pppszPorts)
{
	long lErr;							//function error code
	HKEY hKey = NULL;					//registry key handle
	unsigned long ulIndex;				//index of registry value
	int fContinue;						//continue enumeration
	unsigned long ulValueBufSize;		//size of value buffer
	unsigned long ulPortBufSize;		//size of port name buffer
	unsigned long ulDfltValueBufSize;	//default value buffer size
	unsigned long ulDfltPortBufSize;	//default port name buffer size
	LPTSTR lpValue;						//registry value name
	char *pszPort;						//serial port name from registry
	char *pszPortName;					//serial port name to append

	//test the pointer to the location where this function should
	//store the list existing of serial port names
	_ASSERT(pppszPorts);

	if (pppszPorts == NULL)
		return COMM_ERR_ARGUMENT;

	//We allocate the memory for the serial port list. The are up 
	//to 256 serial ports in the system.
	if ( (*pppszPorts = (char **)malloc(256 * sizeof(void *))) == NULL)
		return COMM_ERR_OUTOFMEMORY;
	
	//For now, there are no elements in this list. We set the first
	//element pointer to NULL.
	**pppszPorts = NULL;

	//open the registry key "HKLM\HARDWARE\DEVICEMAP\SERIALCOMM"
	if ((lErr=RegOpenKeyEx(
			HKEY_LOCAL_MACHINE,
			"HARDWARE\\DEVICEMAP\\SERIALCOMM",
			0,KEY_READ,
			&hKey)) != ERROR_SUCCESS)
	{
		CommFreePortList(*pppszPorts);
		*pppszPorts = NULL;
		return lErr;
	}

	//allocate the buffers for the value name and port name. If the size for
	//the value name is not large enough, we allocate a larger buffer if needed.
	//The ulDflt..BufSize variables are the allocated buffer size and ul..BufSize
	//is the variable used to call RegEnumValue().

	//allocate the value name buffer
	ulDfltValueBufSize = 32 * sizeof(TCHAR);
	if ((lpValue = malloc(ulDfltValueBufSize)) == NULL)
	{
		CommFreePortList(*pppszPorts);
		*pppszPorts = NULL;
		RegCloseKey(hKey);
		return COMM_ERR_OUTOFMEMORY;
	}

	//allocate the port name buffer
	ulDfltPortBufSize = 32 * sizeof(TCHAR);
	if ((pszPort = malloc(ulDfltPortBufSize)) == NULL)
	{
		CommFreePortList(*pppszPorts);
		*pppszPorts = NULL;
		RegCloseKey(hKey);
		free(lpValue);
		return COMM_ERR_OUTOFMEMORY;
	}

	//now we are ready to start to retrieve all serial ports.
	for (ulIndex = 0, fContinue = TRUE; fContinue; ulIndex++)
	{
		ulValueBufSize = ulDfltValueBufSize;
		ulPortBufSize = ulDfltPortBufSize;

		//retrieve the serial port name
		lErr = RegEnumValue(
				hKey,
				ulIndex,
				lpValue,
				&ulValueBufSize,
				NULL,
				NULL,
				pszPort,
				&ulPortBufSize);

		if (lErr == ERROR_SUCCESS)
		{
			//operation sucessful: append the port name to array

			//duplicate the serial port name and append the pointer to
			//the serial port list.
			if ( (pszPortName = _strdup(pszPort)) == NULL)
			{
				CommFreePortList(*pppszPorts);
				*pppszPorts = NULL;
				RegCloseKey(hKey);
				free(lpValue);
				free(pszPort);
				return COMM_ERR_OUTOFMEMORY;
			}

			(*pppszPorts)[ulIndex] = pszPortName;
			//the last element of the list should allways a NULL pointer.
			(*pppszPorts)[ulIndex+1] = NULL;
		}
		else if (lErr == ERROR_MORE_DATA)
		{ 
			//there is not enough space for the value name. If the buffer size 
			//is lower than 1024 byte, we try doubling. If the actual buffer size
			//is larger than 1024 byte, we abort the operation.
			if (ulDfltValueBufSize < 1024)
			{
				ulIndex--;
				free(lpValue);
				ulDfltValueBufSize *= 2;
				if ((lpValue = malloc(ulDfltValueBufSize)) == NULL)
				{
					CommFreePortList(*pppszPorts);
					*pppszPorts = NULL;
					RegCloseKey(hKey);
					free(pszPort);
					return lErr;
				}
			}
			else
			{
				//buffer is larger than 1024 byte: Abort with error
				CommFreePortList(*pppszPorts);
				*pppszPorts = NULL;
				RegCloseKey(hKey);
				free(lpValue);
				free(pszPort);
				return lErr;
			}
		}
		else if (lErr == ERROR_NO_MORE_ITEMS)
			//there are no more items - finish
			fContinue = FALSE;
		else
		{
			//there is any other error
			CommFreePortList(*pppszPorts);
			*pppszPorts = NULL;
			RegCloseKey(hKey);
			free(lpValue);
			free(pszPort);
			return lErr;
		}
	}

	//close the key and free the buffers.
	RegCloseKey(hKey);
	free(lpValue);
	free(pszPort);

	return COMM_SUCCESS;
}

/*  CommFree
 *
 *  Releases the memory allocated by the CommListPorts function. This function 
 *  can be used to free a single string of the list and the lists base pointer.
 *
 *  parameter
 *		lpPtr				pointer to the allocated memory
 */
void LVSERIAL_API CommFree(LPVOID lpPtr)
{
	free(lpPtr);
}

/*  CommFreePortList
 *
 *  Releases the complete serial port list allocated by the CommListPorts function.
 *
 *  parameter
 *		lpPtr				pointer to the allocated memory
 */
void LVSERIAL_API CommFreePortList(char **ppszPorts)
{
	char **p;							//pointer to an element

	_ASSERT(ppszPorts);

	if (ppszPorts == NULL)
		return;

	for (p=ppszPorts; *p != NULL; p++)
		free(*p);

	free(ppszPorts);
}

/*  CommGetHandles
 *
 *	This function returns a list of all existing serial port handles.
 *	The memory of the strings must be released using the CommFree function.
 *
 *  parameter
 *		ppHandles			pointer to the location where this function
 *							stores the real port name
 *		
 *  return
 *		error code
 */
long LVSERIAL_API CommGetHandles(HANDLE **ppHandles, int *piSize)
{
	HANDLE *pH;
	HANDLE hH;
	int iSize;
	int i;

	_ASSERT(gpCommList);

	if (ppHandles == NULL)
	{
		if (piSize != NULL)
			*piSize = 0L;

		return COMM_SUCCESS;
	}

	__CommLock(ghLock);
	iSize = ListGetLength(gpCommList);

	if ((*ppHandles = (HANDLE *)malloc((iSize +1) * sizeof(HANDLE))) == NULL)
	{
		__CommUnLock(ghLock);
		*ppHandles = NULL;
		if (piSize != NULL)
			*piSize = 0L;
		return COMM_ERR_OUTOFMEMORY;
	}

	pH = *ppHandles;
	*pH = hH = ListGetFirstHandle(gpCommList);

	for (i = 1; i < iSize; i++)
	{
		*++pH = hH = ListGetNextHandle(gpCommList, hH);
	}
	__CommUnLock(ghLock);

	return COMM_SUCCESS;
}

/*  CommGetPortName
 *
 *	This function returns the real port name and or the original port name, 
 *	which was used when CommOpenPort was called.
 *	The memory of the strings must be released using the CommFree function.
 *
 *  parameter
 *		hComm				serial port handle
 *		ppszName			pointer to the location where this function
 *							stores the real port name
 *		ppszOriginal		pointer to the location where this function
 *							stores the original port name
 *		
 *  return
 *		error code
 */
long LVSERIAL_API CommGetPortName(HANDLE hComm, char **ppszName, char **ppszOriginal)
{
	PCOMM pComm;						//pointer to the serial port COMM structure

	_ASSERT(gpCommList);

	//test the given serial port handle and get the pointer to the
	//serial port data structure.
	__CommLock(ghLock);
	if (ListCheckHandle(gpCommList, hComm) != 0)
	{
		__CommUnLock(ghLock);
		if (ppszName != NULL)
			*ppszName = NULL;
		if (ppszOriginal != NULL)
			*ppszOriginal = NULL;
		return COMM_ERR_HANDLE;
	}

	pComm = (PCOMM)ListGetElement(gpCommList, hComm);
	__CommUnLock(ghLock);

	_ASSERT(pComm);

	if (ppszName != NULL)
	{
		//if the input pointer is not NULL, we copy the port name to ppszName
		if ((*ppszName = _strdup(pComm->pszPort)) == NULL)
		{
			*ppszName = NULL;
			if (ppszOriginal != NULL)
				*ppszOriginal = NULL;
			return COMM_ERR_OUTOFMEMORY;
		}
	}

	if (ppszOriginal != NULL)
	{
		//if the input pointer is not NULL, we copy the original name to ppszName
		if ((*ppszOriginal = _strdup(pComm->pszOriginalName)) == NULL)
		{
			if (ppszName != NULL)
			{
				free(*ppszName);
				*ppszName = NULL;
			}
			*ppszOriginal = NULL;
			return COMM_ERR_OUTOFMEMORY;
		}
	}

	return COMM_SUCCESS;
}

/*  CommEnableEvent
 *
 *	The CommSetEventMask function specifies a set of events to be monitored 
 *	for a communications device. 
 *
 *  parameter
 *		hComm				serial port handle
 *		ulEventMask			event mask
 *								COMM_EV_BREAK: A break was detected on input. 
 *								COMM_EV_CTS		The CTS (clear-to-send) signal 
 *												changed state. 
 *								COMM_EV_DSR		The DSR (data-set-ready) signal 
 *												changed state. 
 *								COMM_EV_ERR		A line-status error occurred. Line-
 *												status errors are CE_FRAME, 
 *												CE_OVERRUN and CE_RXPARITY. 
 *								COMM_EV_RING	A ring indicator was detected. 
 *								COMM_EV_RLSD	The RLSD (receive-line-signal-detect)
 *												signal changed state. 
 *								COMM_EV_RXCHAR	A character was received and placed 
 *												in the input buffer. 
 *								COMM_EV_RXFLAG	The event character was received and 
 *												placed in the input buffer. The event 
 *												character is specified in the device's DCB structure, which is applied to a serial port by using the SetCommState function. 
 *								COMM_EV_TXEMPTY	The last character in the output 
 *												buffer was sent. 
 *		
 *  return
 *		error code
 */
long LVSERIAL_API CommEnableEvent(HANDLE hComm, unsigned long ulEventMask)
{
	PCOMM pComm;
	unsigned long ulResult;

	_ASSERT(gpCommList);

	//test the given serial port handle and get the pointer to the
	//serial port data structure.
	__CommLock(ghLock);
	if (ListCheckHandle(gpCommList, hComm) != 0)
	{
		__CommUnLock(ghLock);
		return COMM_ERR_HANDLE;
	}

	pComm = (PCOMM)ListGetElement(gpCommList, hComm);
	__CommUnLock(ghLock);

	_ASSERT(pComm);

	//set the event mask
	if (!SetCommMask(pComm->hC, ulEventMask))
		return COMM_ERR_SYSTEM;

	//create the communication event
	if (pComm->osEvent.hEvent == NULL)
	{
		if ((pComm->osEvent.hEvent = CreateEvent(NULL, TRUE, FALSE, NULL)) == NULL)
			return COMM_ERR_SYSTEM;
	}

	if (ulEventMask == COMM_EV_NONE)
		return COMM_SUCCESS;

	//wait for an event to occur
	if (!WaitCommEvent(pComm->hC, &(pComm->ulEventMask), &(pComm->osEvent)))
	{
		//event delayed, test for errors
		if ((ulResult=GetLastError()) == ERROR_IO_PENDING)
			pComm->fWaitOnEvent = TRUE;
		else
		{
			pComm->fWaitOnEvent = FALSE;
			return COMM_ERR_SYSTEM;
		}
	}
	else
	{
		//WaitCommEvent returned immediately.
		pComm->fWaitOnEvent = FALSE;
		return COMM_SUCCESS;
	}

	return COMM_SUCCESS;
}

/*  CommWaitForEvent
 *
 *	This function waits for an event to occur for a specified communications device. 
 *	The set of events that are monitored by this function is contained in the event 
 *	mask associated with the device handle. 
 *
 *  parameter
 *		hComm				serial port handle
 *		ulTimeout			Specifies the time-out interval, in milliseconds.
 *		pulEventMask		Pointer to a variable that receives a mask indicating 
 *							the type of event that occurred. If an error occurs, 
 *							the value is zero; otherwise, it is one of the COMM_EV_
 *							values (see CommSetEventMask).
 *		
 *  return
 *		error code
 */
long LVSERIAL_API CommWaitForEvent(HANDLE hComm, unsigned long ulTimeout, unsigned long *pulEventMask)
{
	PCOMM pComm;
	unsigned long ulResult;
	unsigned long ulEvMask;
	unsigned long ulOvRes;

	_ASSERT(gpCommList);

	//test the pulEventMask pointer
	if (pulEventMask == NULL)
		return COMM_ERR_ARGUMENT;

	//test the given serial port handle and get the pointer to the
	//serial port data structure.
	__CommLock(ghLock);
	if (ListCheckHandle(gpCommList, hComm) != 0)
	{
		__CommUnLock(ghLock);
		*pulEventMask = 0;
		return COMM_ERR_HANDLE;
	}

	pComm = (PCOMM)ListGetElement(gpCommList, hComm);
	__CommUnLock(ghLock);

	_ASSERT(pComm);

	//if there is no event mask specified, we set the event mask to zero
	if (!GetCommMask(pComm->hC, &ulEvMask))
	{
		*pulEventMask = 0;
		return COMM_ERR_SYSTEM;
	}

	if (pComm->osEvent.hEvent == 0)
	{
		*pulEventMask = 0;
		return COMM_ERR_NOEVENT;
	}

    if (ulEvMask == COMM_EV_NONE)
	{
		*pulEventMask = 0;
		return COMM_SUCCESS;
	}

	//wait for an event to occur
	if (!pComm->fWaitOnEvent) 
	{
		if (!WaitCommEvent(pComm->hC, &(pComm->ulEventMask), &(pComm->osEvent)))
		{
			//event delayed, test for errors
			if ((ulResult=GetLastError()) == ERROR_IO_PENDING)
				//event not occurred, so we continue to monitor the event
				pComm->fWaitOnEvent = TRUE;
			else
				//error in WaitCommEvent function
				return COMM_ERR_SYSTEM;
		}
		else
		{
			//WaitCommEvent returned immediately.
			*pulEventMask = pComm->ulEventMask;
			return COMM_SUCCESS;
		}
	}
				
	//the operation is not finished yet, so we should wait 
	//until the operation has finished by waiting for the 
	//event. Here we go.

	//wait for the event
	ulResult = WaitForSingleObject(pComm->osEvent.hEvent, ulTimeout);
	switch(ulResult)
	{
		case WAIT_OBJECT_0:
			//operation completed
			if (!GetOverlappedResult(hComm, &(pComm->osEvent), &ulOvRes, FALSE))
				// An error occurred in the overlapped operation;
				// call GetLastError to find out what it was
				// and abort if it is fatal.
				return COMM_ERR_SYSTEM;
                 
			// Status event is stored in the event flag
			// specified in the original WaitCommEvent call.
			pComm->fWaitOnEvent = FALSE;
			*pulEventMask = pComm->ulEventMask;
			return COMM_SUCCESS;
			break;

		case WAIT_TIMEOUT:
			//operation timed out
			//Do not set pComm->fWaitOnEvent to false, so we continue to
			//wait for the event on the next call of this function.
			*pulEventMask = 0;
			return COMM_ERR_TIMEOUT;
			break;

		case WAIT_FAILED:
		default:
			//error in the WaitForSingleObject. This indicates 
			//a problem with the OVERLAPPED structure's event handle.
			//Oh yes, we do not have a special error code for this
			//situation (may be changed in future releases) and we
			//allways return COMM_ERR_SYSTEM.
			*pulEventMask = 0;
			CloseHandle(pComm->osEvent.hEvent);
			pComm->osEvent.hEvent = 0;
			if (ulResult = GetLastError() != 0)
				return COMM_ERR_SYSTEM;
			else
				return COMM_ERR_SYSTEM;
			break;
	}

	return COMM_SUCCESS;
}

/*  CommDisableEvent
 *
 *	This function disables the monitoring of communication events.
 *
 *  parameter
 *		hComm				serial port handle
 *		
 *  return
 *		error code
 */
long LVSERIAL_API CommDisableEvent(HANDLE hComm)
{
	PCOMM pComm;

	_ASSERT(gpCommList);

	//test the given serial port handle and get the pointer to the
	//serial port data structure.
	__CommLock(ghLock);
	if (ListCheckHandle(gpCommList, hComm) != 0)
	{
		__CommUnLock(ghLock);
		return COMM_ERR_HANDLE;
	}

	pComm = (PCOMM)ListGetElement(gpCommList, hComm);
	__CommUnLock(ghLock);

	_ASSERT(pComm);

	//close the event handle
	if (pComm->osEvent.hEvent != 0)
		CloseHandle(pComm->osEvent.hEvent);

	//return settings to their defaults
	pComm->fWaitOnEvent = FALSE;
	pComm->osEvent.hEvent = 0;

	//set the event mask
	if (!SetCommMask(pComm->hC, COMM_EV_NONE))
		return COMM_ERR_SYSTEM;

	return COMM_SUCCESS;
}

/*  CommSetEventChar
 *
 *	This function sets the event character.
 *
 *  parameter
 *		hComm				serial port handle
 *		ucEventChar			event character
 *		
 *  return
 *		error code
 */
long LVSERIAL_API CommSetEventChar(HANDLE hComm, unsigned char ucEventChar)
{
	PCOMM pComm;						//pointer to the serial port COMM structure
	DCB dcb;							//DCB data structure

	_ASSERT(gpCommList);

	//test the given serial port handle and get the pointer to the
	//serial port data structure.
	__CommLock(ghLock);
	if (ListCheckHandle(gpCommList, hComm) != 0)
	{
		__CommUnLock(ghLock);
		return COMM_ERR_HANDLE;
	}

	pComm = (PCOMM)ListGetElement(gpCommList, hComm);
	__CommUnLock(ghLock);

	_ASSERT(pComm);

	//modify the dcb structure. At first, we lock the device, because no
	//other function should modify the DCB during this operation.
	__CommLock(pComm->hInternalLock);
	if (!GetCommState(pComm->hC, &dcb))
	{
		__CommUnLock(pComm->hInternalLock);
		return __CommClearError(pComm,COMM_ERR_SYSTEM);
	}

	//store the event character in the DCB
	dcb.EvtChar = (char)ucEventChar;

	//and set the DCB
	if (!SetCommState(pComm->hC, &dcb))
	{
		__CommUnLock(pComm->hInternalLock);
		return __CommClearError(pComm,COMM_ERR_SYSTEM);
	}
	__CommUnLock(pComm->hInternalLock);

	return COMM_SUCCESS;
}

/*  CommTransmitChar
 *
 *	The CommTransmitChar function transmits a specified character ahead of 
 *	any pending data in the output buffer of the specified communications 
 *	device.
 *
 *  parameter
 *		hComm				serial port handle
 *		ucChar				character to be transmitted
 *		
 *  return
 *		error code
 */
long LVSERIAL_API CommTransmitChar(HANDLE hComm, unsigned char ucChar)
{
	PCOMM pComm;						//pointer to the serial port COMM structure

	_ASSERT(gpCommList);

	//test the given serial port handle and get the pointer to the
	//serial port data structure.
	__CommLock(ghLock);
	if (ListCheckHandle(gpCommList, hComm) != 0)
	{
		__CommUnLock(ghLock);
		return COMM_ERR_HANDLE;
	}

	pComm = (PCOMM)ListGetElement(gpCommList, hComm);
	__CommUnLock(ghLock);

	_ASSERT(pComm);

	if (!TransmitCommChar(pComm->hC, (char)ucChar))
		return COMM_ERR_SYSTEM;

	return COMM_SUCCESS;
}

/*  CommSetDefaultConfig
 *
 *	The CommSetDefaultConfig function sets the default configuration for 
 *  a communications device.
 *
 *  parameter
 *		pszPort				serial port name
 *		pszOptions			Pointer to a null-terminated string that 
 *							specifies device-control information. The 
 *							string must have the same form as the mode 
 *							command's command-line arguments. For 
 *							example, the following string specifies a 
 *							baud rate of 1200, no parity, 8 data bits, 
 *							and 1 stop bit: 
 *
 *							baud=1200 parity=N data=8 stop=1 
 *
 *							For further information on mode command 
 *							syntax, refer to the end-user documentation 
 *							for your operating system. 
 *		
 *  return
 *		error code
 */
long LVSERIAL_API CommSetDefaultConfig(char *pszPort, char *pszOptions)
{
	unsigned long ulSize;
	LPCOMMCONFIG lpCC;

	//allocate the buffer for the COMMCONFIG structure
	//if the default size is not enough, GetDefaultCommConfig will
	//return the correct size.
	ulSize = 256;
	if ((lpCC = malloc(ulSize)) == NULL)
		return COMM_ERR_OUTOFMEMORY;

	//get the size of the COMMCONFIG structure
	if (!GetDefaultCommConfig(pszPort, lpCC, &ulSize))
	{
		free(lpCC);
		if ( GetLastError() != ERROR_INSUFFICIENT_BUFFER)
			return COMM_ERR_SYSTEM;

		if ((lpCC = malloc(ulSize)) == NULL)
			return COMM_ERR_OUTOFMEMORY;

		//get the actual default configuration
		if (!GetDefaultCommConfig(pszPort, lpCC, &ulSize))
		{
			free(lpCC);
			return COMM_ERR_SYSTEM;
		}
	}

	//modify the default configuration according to the options string
	if (!BuildCommDCB( pszOptions, &(lpCC->dcb)))
	{
		free(lpCC);
		return COMM_ERR_SYSTEM;
	}

	//set the new default configuration
	if (!SetDefaultCommConfig(pszPort, lpCC, ulSize))
	{
		free(lpCC);
		return COMM_ERR_SYSTEM;
	}

	free(lpCC);

	return COMM_SUCCESS;
}

/*----------------------------------------------------------------------------------*/

/*  lvCommOpenPort
 *
 *  Opens the serial port
 *
 *  parameter
 *		phComm				pointer where the function stores the serial port handle
 *		pszPort				name of the serial port to open
 *		plfDuplicate		if this value is true, the function returns a duplicate 
 *							handle if the port is already open
 *		pError				Error cluster
 */
void LVSERIAL_API lvCommOpenPort(
	PHANDLE phComm, 
	LStrHandle shPort, 
	LVBoolean *plfDuplicate, 
	pErrC pError)
{
	long lCode;							//function return code

	//error pointer test
	_ASSERT(!DSCheckPtr(pError));

	//nothing to do in case of an incoming error
	if (pError->lfStatus == LVTRUE)
		return;

	_ASSERT(!DSCheckPtr(shPort));
	_ASSERT(!DSCheckPtr(plfDuplicate));

	//convert the buffer of shPort into a null terminated string
	if ((lCode = NumericArrayResize(uB, 1L, (UHandle *)&shPort, LHStrLen(shPort)+1)) != noErr)
	{
		lv_EasyErrBld(lCode, __FUNCTION__, pError);
		return;
	}
	*(LHStrBuf(shPort)+LHStrLen(shPort)) = '\0';

	//open the serial port
	lCode = CommOpenPort(phComm, (char *)LHStrBuf(shPort), *plfDuplicate == LVTRUE ? TRUE : FALSE);

	//update the error cluster
	if (lCode == COMM_ERR_SYSTEM)
		lv_WinErrBld(GetLastError(), __FUNCTION__, pError);
	else
		lv_WarnOrErrBld(lCode, __FUNCTION__, pError);
}

/*  lvCommOpenPort2
 *
 *  Opens the serial port
 *
 *  parameter
 *		phComm				pointer where the function stores the serial port handle
 *		pszPort				name of the serial port to open
 *		plfDuplicate			if this value is true, the function returns a duplicate 
 *						handle if the port is already open
 *		plfCreatedNew			will be set to true if this function has opened a new handle
 *		pError				Error cluster
 */
void LVSERIAL_API lvCommOpenPort2(
	PHANDLE phComm, 
	LStrHandle shPort, 
	LVBoolean *plfDuplicate, 
	LVBoolean *plfCreatedNew,
	pErrC pError)
{
	long lCode;							//function return code
	int fCreatedNew;

	//error pointer test
	_ASSERT(!DSCheckPtr(pError));

	//nothing to do in case of an incoming error
	if (pError->lfStatus == LVTRUE)
		return;

	_ASSERT(!DSCheckPtr(shPort));
	_ASSERT(!DSCheckPtr(plfDuplicate));
	_ASSERT(!DSCheckPtr(plfCreatedNew));

	//convert the buffer of shPort into a null terminated string
	if ((lCode = NumericArrayResize(uB, 1L, (UHandle *)&shPort, LHStrLen(shPort)+1)) != noErr)
	{
		lv_EasyErrBld(lCode, __FUNCTION__, pError);
		return;
	}
	*(LHStrBuf(shPort)+LHStrLen(shPort)) = '\0';

	//open the serial port
	lCode = CommOpenPort2(
			phComm,
			(char *)LHStrBuf(shPort), 
			*plfDuplicate == LVTRUE ? TRUE : FALSE,
			&fCreatedNew);

	*plfCreatedNew = fCreatedNew ? LVTRUE : LVFALSE;

	//update the error cluster
	if (lCode == COMM_ERR_SYSTEM)
		lv_WinErrBld(GetLastError(), __FUNCTION__, pError);
	else
		lv_WarnOrErrBld(lCode, __FUNCTION__, pError);
}

/*  lvCommClosePort
 *
 *  Closes the serial port. This function is implementd for backward
 *	compatibility and it only calls CommClosePort2 with fCloseAll set
 *	to false.
 *
 *  parameter
 *		hComm				serial port handle
 *		pError				Error cluster
 */
void LVSERIAL_API lvCommClosePort(HANDLE hComm, pErrC pError)
{
	long lCode;							//function return code

	//error pointer test
	if (DSCheckPtr(pError) != noErr)
		return;

	//close the serial port. Note, that we close the serial port 
	//even if there is an incoming error.
	lCode = CommClosePort2(hComm, FALSE);

	//update the error cluster
	if (lCode == COMM_ERR_SYSTEM)
		lv_WinErrBld(GetLastError(), __FUNCTION__, pError);
	else
		lv_WarnOrErrBld(lCode, __FUNCTION__, pError);
}

/*  lvCommClosePort2
 *
 *  Closes the serial port
 *
 *  parameter
 *		hComm				serial port handle
 *		plfCloseAll			if this value is TRUE, the function closes
 *							all opened instances of this port
 *		pError				Error cluster
 */
void LVSERIAL_API lvCommClosePort2(HANDLE hComm, LVBoolean *plfCloseAll, pErrC pError)
{
	long lCode;							//function return code

	//error pointer test
	if (DSCheckPtr(pError) != noErr)
		return;

	//close the serial port. Note, that we close the serial port 
	//even if there is an incoming error.
	lCode = CommClosePort2(hComm, *plfCloseAll);

	//update the error cluster
	if (lCode == COMM_ERR_SYSTEM)
		lv_WinErrBld(GetLastError(), __FUNCTION__, pError);
	else
		lv_WarnOrErrBld(lCode, __FUNCTION__, pError);
}

/*  lvCommClosePort3
 *
 *  Closes the serial port
 *
 *  parameter
 *		hComm				serial port handle
 *		plfCloseAll			if this value is TRUE, the function closes
 *							all opened instances of this port
 *		ulTimeout
 *		pError				Error cluster
 */
void LVSERIAL_API lvCommClosePort3(HANDLE hComm, LVBoolean *plfCloseAll, unsigned long ulTimeout, pErrC pError)
{
	long lCode;							//function return code

	//error pointer test
	if (DSCheckPtr(pError) != noErr)
		return;

	//close the serial port. Note, that we close the serial port 
	//even if there is an incoming error.
	lCode = CommClosePort3(hComm, *plfCloseAll, ulTimeout);

	//update the error cluster
	if (lCode == COMM_ERR_SYSTEM)
		lv_WinErrBld(GetLastError(), __FUNCTION__, pError);
	else
		lv_WarnOrErrBld(lCode, __FUNCTION__, pError);
}

/*  lvCommCloseAll
 *
 *  Closes all serial ports
 *
 *  parameter
 *		pError				Error cluster
 */
void LVSERIAL_API lvCommCloseAll(pErrC pError)
{
	long lCode;							//function return code

	//error pointer test
	if (DSCheckPtr(pError) != noErr)
		return;

	//close the serial ports. Note, that we close the serial port 
	//even if there is an incoming error.
	lCode = CommCloseAll();

	//update the error cluster
	if (lCode == COMM_ERR_SYSTEM)
		lv_WinErrBld(GetLastError(), __FUNCTION__, pError);
	else
		lv_WarnOrErrBld(lCode, __FUNCTION__, pError);
}


/*  lvCommWrite
 *
 *  Writes data to the serial port
 *
 *	The function waits until any pending writes are finished using the
 *	specified timeout value. The write operation itself can also last
 *	up to the specified timeout value. This means, that the complete
 *	execution of this function can last up to twice the specified timeout.
 *
 *  parameter
 *		hComm				serial port handle
 *		shBuffer			buffer containing the bytes to write
 *		ulBytesToWrite		number of bytes to write
 *		pulBytesWritten		pointer where this function stores the number of 
 *							bytes written to the serial port
 *		pError				Error cluster
 */
void LVSERIAL_API lvCommWrite(
	HANDLE hComm, 
	LStrHandle shBuffer, 
	unsigned long * pulBytesWritten, 
	pErrC pError)
{
	long lCode;							//function return code

	//error pointer test
	_ASSERT(!DSCheckPtr(pError));

	//nothing to do in case of an incoming error
	if (pError->lfStatus == LVTRUE)
		return;

	_ASSERT(!DSCheckPtr(shBuffer));
	_ASSERT(!DSCheckPtr(pulBytesWritten));

	//write the data to the serial port
	lCode = CommWrite(hComm, (char *)LHStrBuf(shBuffer), LHStrLen(shBuffer), pulBytesWritten);

	//update the error cluster
	if (lCode == COMM_ERR_SYSTEM)
		lv_WinErrBld(GetLastError(), __FUNCTION__, pError);
	else
		lv_WarnOrErrBld(lCode, __FUNCTION__, pError);
}

/*  lvCommWriteAndReceiveEcho
 *
 *  Writes data to the serial port and received the echo. Note, that this
 *	function does not handle the specified termination condition.
 *
 *  parameter
 *		hComm				serial port handle
 *		shWriteBuffer		buffer containing the bytes to write
 *		pulBytesWritten		pointer where this function stores the number of 
 *							bytes written to the serial port
 *		shReadBuffer		This is the location where this function stores
 *							the received characters
 *		plfStopIfNotEqual	If this value is true, the transmission is stopped,
 *							if the received character is not equal to the
 *							transmitted character.
 *	pError					error cluster
 */
void LVSERIAL_API lvCommWriteAndReceiveEcho(
	HANDLE hComm, 
	LStrHandle shWriteBuffer,
	unsigned long * pulBytesWritten,
	LStrHandle shReadBuffer,
	unsigned long * pulBytesRead,
	LVBoolean *plfStopIfNotEqual,
	pErrC pError)
{
	long lCode;							//function return code

	//error pointer test
	_ASSERT(!DSCheckPtr(pError));

	//in case of an incoming error, we set shReadBuffer to an empty string
	if (pError->lfStatus == LVTRUE)
	{
		_ASSERT(!DSCheckPtr(shReadBuffer));

		if ((lCode = NumericArrayResize(uB, 1L, (UHandle *)&shReadBuffer, 1L)) != noErr)
		{
			lv_EasyErrBld(lCode, __FUNCTION__, pError);
			return;
		}
		LStrLen(*shReadBuffer) = 0;

		return;
	}

	_ASSERT(!DSCheckPtr(shWriteBuffer));
	_ASSERT(!DSCheckPtr(shReadBuffer));
	_ASSERT(!DSCheckPtr(pulBytesWritten));
	_ASSERT(!DSCheckPtr(pulBytesRead));

	//if the shWriteBuffer is empty, we have nothing to transmit. We 
	//change the shReadBuffer to an empty string.
	if (LHStrLen(shWriteBuffer) == 0)
	{
		if ((lCode = NumericArrayResize(uB, 1L, (UHandle *)&shReadBuffer, 1L)) != noErr)
		{
			lv_EasyErrBld(lCode, __FUNCTION__, pError);
			return;
		}
	}
	else
	{
		//there is something to transmit. Because we normally receive each character
		//which we will transmit, so the size of shReadBuffer should be as large as 
		//the shWriteBuffer.
		if ((lCode = NumericArrayResize(uB, 1L, (UHandle *)&shReadBuffer, LHStrLen(shWriteBuffer)+1)) != noErr)
		{
			lv_EasyErrBld(lCode, __FUNCTION__, pError);
			return;
		}
	}

	//transmit and receive data
	lCode = CommWriteAndReceiveEcho(
		hComm, 
		(char *)LHStrBuf(shWriteBuffer), 
		LHStrLen(shWriteBuffer),
		pulBytesWritten,
		(char *)LHStrBuf(shReadBuffer), 
		pulBytesRead,
		(int)*plfStopIfNotEqual);

	//adjust the size of shReadBuffer
	LStrLen(*shReadBuffer) = *pulBytesRead;

	//update the error cluster
	if (lCode == COMM_ERR_SYSTEM)
		lv_WinErrBld(GetLastError(), __FUNCTION__, pError);
	else
		lv_WarnOrErrBld(lCode, __FUNCTION__, pError);
}

/*  lvCommRead
 *
 *  Reads bytes from the serial port and stores the data into the specified
 *  buffer.
 *
 *	The function waits until any pending writes are finished using the
 *	specified timeout value. The write operation itself can also last
 *	up to the specified timeout value. This means, that the complete
 *	execution of this function can last up to twice the specified timeout.
 *
 *  parameter
 *		hComm				serial port handle
 *		shBuffer			pointer to a buffer where this function stores the 
 *							read bytes
 *		ulBytesToRead		number of bytes to read from the serial port
 *		pulBytesRead		pointer where this function stores the read number 
 *							of bytes.
 *		pError				Error cluster
 */
void LVSERIAL_API lvCommRead(
	HANDLE hComm, 
	LStrHandle shBuffer, 
	unsigned long ulBytesToRead, 
	unsigned long * pulBytesRead,
	pErrC pError)
{
	long lCode;							//function return code

	//error pointer test
	_ASSERT(!DSCheckPtr(pError));

	//in case of incoming error, we change the shReadBuffer to an empty string
	if (pError->lfStatus == LVTRUE)
	{
		_ASSERT(!DSCheckPtr(shBuffer));

		if ((lCode = NumericArrayResize(uB, 1L, (UHandle *)&shBuffer, 1L)) != noErr)
		{
			lv_EasyErrBld(lCode, __FUNCTION__, pError);
			return;
		}
		LStrLen(*shBuffer) = 0;

		return;
	}

	_ASSERT(!DSCheckPtr(shBuffer));
	_ASSERT(!DSCheckPtr(pulBytesRead));

	//id the bytes to write is zero, we change the shReadBuffer to an empty string.
	//otherwise we allocate the requested number of bytes for shReadBuffer.
	if (ulBytesToRead == 0)
	{
		if ((lCode = NumericArrayResize(uB, 1L, (UHandle *)&shBuffer, 1L)) != noErr)
		{
			lv_EasyErrBld(lCode, __FUNCTION__, pError);
			return;
		}
	}
	else
	{
		if ((lCode = NumericArrayResize(uB, 1L, (UHandle *)&shBuffer, ulBytesToRead+1)) != noErr)
		{
			lv_EasyErrBld(lCode, __FUNCTION__, pError);
			return;
		}
	}

	//read the data from the serial port
	lCode = CommRead(
		hComm, 
		(char *)LHStrBuf(shBuffer), 
		ulBytesToRead, 
		pulBytesRead);

	//adjuste the size of shReadBuffer
	LStrLen(*shBuffer) = *pulBytesRead;

	//update the error cluster
	if (lCode == COMM_ERR_SYSTEM)
		lv_WinErrBld(GetLastError(), __FUNCTION__, pError);
	else
		lv_WarnOrErrBld(lCode, __FUNCTION__, pError);
}

/*  lvCommBreak
 *
 *  Sets the break state for the specified number of milliseconds
 *
 *  parameter
 *		hComm				serial port handle
 *		ulMilliseconds		break state duration
 *		pError				Error cluster
 */
void LVSERIAL_API lvCommBreak(HANDLE hComm, unsigned long ulMilliseconds, pErrC pError)
{
	long lCode;							//function return code

	//error pointer test
	_ASSERT(!DSCheckPtr(pError));

	//nothing to do in case of an incoming error
	if (pError->lfStatus == LVTRUE)
		return;

	//execute serial port break
	lCode = CommBreak(hComm, ulMilliseconds);

	//update the error cluster
	if (lCode == COMM_ERR_SYSTEM)
		lv_WinErrBld(GetLastError(), __FUNCTION__, pError);
	else
		lv_WarnOrErrBld(lCode, __FUNCTION__, pError);
}

/*  lvCommBuffer
 *
 *  Sets the input and output queue size
 *
 *  parameter
 *		hComm				serial port handle
 *		ulInQueue			input queue size
 *		ulOutQueue			output queue size
 *		pError				Error cluster
 */
void LVSERIAL_API lvCommBuffer(HANDLE hComm, unsigned long ulInQueueSize, unsigned long ulOutQueueSize, pErrC pError)
{
	long lCode;							//function return  code

	//error pointer test
	_ASSERT(!DSCheckPtr(pError));

	//nothing to do in case of an incoming error
	if (pError->lfStatus == LVTRUE)
		return;

	//set the serial port buffer size
	lCode = CommBuffer(hComm, ulInQueueSize, ulOutQueueSize);

	//update the error cluster
	if (lCode == COMM_ERR_SYSTEM)
		lv_WinErrBld(GetLastError(), __FUNCTION__, pError);
	else
		lv_WarnOrErrBld(lCode, __FUNCTION__, pError);
}

/*  lvCommFlush
 *
 *  The CommFlush function discards all characters from the output or input 
 *  buffer of a specified serial port. It can also terminate pending read 
 *  or write operations.
 *
 *  parameter
 *		hComm				serial port handle
 *		ulFlags				Specifies the action to take.
 *			COMM_FLUSH_TXABORT		Kill the pending/current writes to the comm port.
 *			COMM_FLUSH_RXABORT		Kill the pending/current reads to the comm port.
 *			COMM_FLUSH_TXCLEAR		Kill the transmit queue if there.
 *			COMM_FLUSH_RXCLEAR		Kill the typeahead buffer if there.
 *		pError				Error cluster
 */
void LVSERIAL_API lvCommFlush(HANDLE hComm, unsigned long ulFlags, pErrC pError)
{
	long lCode;							//function return code

	//error pointer test
	_ASSERT(!DSCheckPtr(pError));

	//nothing to do in case of an incoming error
	if (pError->lfStatus == LVTRUE)
		return;

	//flush the serial port
	lCode = CommFlush(hComm, ulFlags);

	//update the error code
	if (lCode == COMM_ERR_SYSTEM)
		lv_WinErrBld(GetLastError(), __FUNCTION__, pError);
	else
		lv_WarnOrErrBld(lCode, __FUNCTION__, pError);
}

/*  lvCommStatus
 *
 *  The CommStatus function retrieves information about a communications pError 
 *  and reports the current status of a communications device. The function is 
 *  called when a communications pError occurs, and it clears the device's pError 
 *  flag to enable additional input and output (I/O) operations.
 *
 *  parameter
 *		hComm				serial port handle
 *		pulErrorFlags		pError flags (see COMM_ERRF_ )
 *		pulFlags			status flags (see COMM_STAT_ )
 *		pulInQueue			Number of bytes received by the serial provider but not 
 *							yet read by a CommRead operation.
 *		pulOutQueue			Number of bytes of user data remaining to be transmitted 
 *							for all write operations. 
 *		pError				Error cluster
 */
void LVSERIAL_API lvCommStatus(
	HANDLE hComm, 
	unsigned long * pulErrorFlags, 
	unsigned long * pulStatusFlags,
	unsigned long * pulInQueue,
	unsigned long * pulOutQueue, 
	pErrC pError)
{
	long lCode;							//function return code

	//error pointer test
	_ASSERT(!DSCheckPtr(pError));

	_ASSERT(!DSCheckPtr(pulErrorFlags));
	_ASSERT(!DSCheckPtr(pulStatusFlags));
	_ASSERT(!DSCheckPtr(pulInQueue));
	_ASSERT(!DSCheckPtr(pulOutQueue));

	//set the flags to zero in case of an incoming error
	if (pError->lfStatus == LVTRUE)
	{
		*pulErrorFlags = 0;
		*pulStatusFlags = 0;
		*pulInQueue = 0;
		*pulOutQueue = 0;
		return;
	}

	//get the serial port status
	lCode = CommStatus(hComm, pulErrorFlags, pulStatusFlags, pulInQueue, pulOutQueue);

	//update the error cluster
	if (lCode == COMM_ERR_SYSTEM)
		lv_WinErrBld(GetLastError(), __FUNCTION__, pError);
	else
		lv_WarnOrErrBld(lCode, __FUNCTION__, pError);
}

/*  lvCommBytesAtPort
 *
 *  The function retrieves the number of bytes at the serial ports input queue.
 *
 *  parameter
 *		hComm				serial port handle
 *		pulBytesAtPort		Number of bytes received by the serial provider but not 
 *							yet read by a CommRead operation.
 *		pError				Error cluster
 */
void LVSERIAL_API lvCommBytesAtPort( HANDLE hComm, unsigned long * pulBytesAtPort, pErrC pError)
{
	long lCode;							//function return code

	//error pointer test
	_ASSERT(!DSCheckPtr(pError));
	_ASSERT(!DSCheckPtr(pulBytesAtPort));

	//in case of an incoming error, we set the number of bytes at port to zero
	if (pError->lfStatus == LVTRUE)
	{
		*pulBytesAtPort = 0;
		return;
	}

	//get the number of bytes at the serial port
	lCode = CommBytesAtPort(hComm, pulBytesAtPort);

	//update the error cluster
	if (lCode == COMM_ERR_SYSTEM)
		lv_WinErrBld(GetLastError(), __FUNCTION__, pError);
	else
		lv_WarnOrErrBld(lCode, __FUNCTION__, pError);
}


/*  lvCommModemStatus
 *
 *  The CommModemStatus function retrieves modem control-register values.
 *
 *  parameter
 *		hComm				serial port handle
 *		pulModemStatus		Pointer to a variable that specifies the current state 
 *							of the modem control-register values. (see COMM_MSTAT_ )
 *		pError				Error cluster
 */
void LVSERIAL_API lvCommModemStatus(HANDLE hComm, unsigned long * pulModemStatus, pErrC pError)
{
	long lCode;							//function return code

	//error pointer test
	_ASSERT(!DSCheckPtr(pError));
	_ASSERT(!DSCheckPtr(pulModemStatus));

	//in case of an incoming error we set the modem status to zero
	if (pError->lfStatus == LVTRUE)
	{
		*pulModemStatus = 0;
		return;
	}

	//get the modem status
	lCode = CommModemStatus(hComm, pulModemStatus);

	//update the error cluster
	if (lCode == COMM_ERR_SYSTEM)
		lv_WinErrBld(GetLastError(), __FUNCTION__, pError);
	else
		lv_WarnOrErrBld(lCode, __FUNCTION__, pError);
}

/*  lvCommGetCTS
 *
 *  The CommGetCTS function retrieves modem control-register value.
 *
 *  parameter
 *		hComm				serial port handle
 *		plfCTS				Pointer to a variable that specifies the current state 
 *							of the CTS signal.
 *		pError				Error cluster
 */
void LVSERIAL_API lvCommGetCTS(HANDLE hComm, LVBoolean *plfCTS, pErrC pError)
{
	long lCode;							//function return code
	int iCTS;							//CTS line state

	//error pointer test
	_ASSERT(!DSCheckPtr(pError));
	_ASSERT(!DSCheckPtr(plfCTS));

	//in case of an incoming error, we set the CTS flag to false
	if (pError->lfStatus == LVTRUE)
	{
		*plfCTS = LVFALSE;
		return;
	}

	//get the CTS line state
	lCode = CommGetCTS(hComm, &iCTS);

	//copy the CTS line state to the target pointer
	*plfCTS = iCTS != 0 ? LVTRUE : LVFALSE;

	//update the error cluster
	if (lCode == COMM_ERR_SYSTEM)
		lv_WinErrBld(GetLastError(), __FUNCTION__, pError);
	else
		lv_WarnOrErrBld(lCode, __FUNCTION__, pError);
}

/*  lvCommGetDSR
 *
 *  The CommGetDSR function retrieves modem control-register value.
 *
 *  parameter
 *		hComm				serial port handle
 *		plfDSR				Pointer to a variable that specifies the current state 
 *							of the DSR signal.
 *		pError				Error cluster
 */
void LVSERIAL_API lvCommGetDSR(HANDLE hComm, LVBoolean *plfDSR, pErrC pError)
{
	long lCode;							//function return code
	int iDSR;							//DSR line state

	//error pointer test
	_ASSERT(!DSCheckPtr(pError));
	_ASSERT(!DSCheckPtr(plfDSR));

	//in case of an incoming error we set the DSR flag to false
	if (pError->lfStatus == LVTRUE)
	{
		*plfDSR = LVFALSE;
		return;
	}

	//get the DSR line state
	lCode = CommGetDSR(hComm, &iDSR);

	//copy the DSR line state to the target pointer
	*plfDSR = iDSR != 0 ? LVTRUE : LVFALSE;

	//update the error cluster
	if (lCode == COMM_ERR_SYSTEM)
		lv_WinErrBld(GetLastError(), __FUNCTION__, pError);
	else
		lv_WarnOrErrBld(lCode, __FUNCTION__, pError);
}

/*  lvCommGetRING
 *
 *  The CommGetRING function retrieves modem control-register value.
 *
 *  parameter
 *		hComm				serial port handle
 *		plfRING				Pointer to a variable that specifies the current state 
 *							of the RING signal.
 *		pError				Error cluster
 */
void LVSERIAL_API lvCommGetRING(HANDLE hComm, LVBoolean *plfRING, pErrC pError)
{
	long lCode;							//function return code
	int iRING;							//RING line state

	//error pointer test
	_ASSERT(!DSCheckPtr(pError));
	_ASSERT(!DSCheckPtr(plfRING));

	//in case of an incoming error, we set the RING state to false
	if (pError->lfStatus == LVTRUE)
	{
		*plfRING = LVFALSE;
		return;
	}

	//get the RING state
	lCode = CommGetRING(hComm, &iRING);

	//copy the RING line state to the target pointer
	*plfRING = iRING != 0 ? LVTRUE : LVFALSE;

	//update the error cluster
	if (lCode == COMM_ERR_SYSTEM)
		lv_WinErrBld(GetLastError(), __FUNCTION__, pError);
	else
		lv_WarnOrErrBld(lCode, __FUNCTION__, pError);
}

/*  lvCommGetRLSD
 *
 *  The CommGetRLSD function retrieves modem control-register value.
 *
 *  parameter
 *		hComm				serial port handle
 *		plfRLSD				Pointer to a variable that specifies the current state 
 *							of the RLSD signal (receive-line-signal-detect).
 *		pError				Error cluster
 */
void LVSERIAL_API lvCommGetRLSD(HANDLE hComm, LVBoolean *plfRLSD, pErrC pError)
{
	long lCode;							//function return code
	int iRLSD;							//RLSD line state

	//error pointer test
	_ASSERT(!DSCheckPtr(pError));
	_ASSERT(!DSCheckPtr(plfRLSD));

	//in case of an incoming error, we set the RLSD state to false
	if (pError->lfStatus == LVTRUE)
	{
		*plfRLSD = LVFALSE;
		return;
	}

	//get the RLSD line state
	lCode = CommGetRLSD(hComm, &iRLSD);

	//copy the RLSD line state to the target pointer
	*plfRLSD = iRLSD != 0 ? LVTRUE : LVFALSE;

	//update the error cluster
	if (lCode == COMM_ERR_SYSTEM)
		lv_WinErrBld(GetLastError(), __FUNCTION__, pError);
	else
		lv_WarnOrErrBld(lCode, __FUNCTION__, pError);
}

/*  lvCommSetXOFF
 *
 *  Causes transmission to act as if an XOFF character has been received.
 *
 *  parameter
 *		hComm				serial port handle
 *		pError				Error cluster
 */
void LVSERIAL_API lvCommSetXOFF(HANDLE hComm, pErrC pError)
{
	long lCode;							//function return code

	//error pointer test
	_ASSERT(!DSCheckPtr(pError));

	//nothing to do in case of an incoming error
	if (pError->lfStatus == LVTRUE)
		return;

	//set the COFF state
	lCode = CommSetXOFF(hComm);

	//update the error cluster
	if (lCode == COMM_ERR_SYSTEM)
		lv_WinErrBld(GetLastError(), __FUNCTION__, pError);
	else
		lv_WarnOrErrBld(lCode, __FUNCTION__, pError);
}

/*  lvCommSetXON
 *
 *  Causes transmission to act as if an XON character has been received.
 *
 *  parameter
 *		hComm				serial port handle
 *		pError				Error cluster
 */
void LVSERIAL_API lvCommSetXON(HANDLE hComm, pErrC pError)
{
	long lCode;							//function return code

	//error pointer test
	_ASSERT(!DSCheckPtr(pError));

	//nothing to do in case of an incoming error
	if (pError->lfStatus == LVTRUE)
		return;

	//set the XON state
	lCode = CommSetXON(hComm);

	//update the error cluster
	if (lCode == COMM_ERR_SYSTEM)
		lv_WinErrBld(GetLastError(), __FUNCTION__, pError);
	else
		lv_WarnOrErrBld(lCode, __FUNCTION__, pError);
}

/*  lvCommSetRTS
 *
 *  Sends the RTS (request-to-send) signal.
 *
 *  parameter
 *		hComm				serial port handle
 *		pError				Error cluster
 */
void LVSERIAL_API lvCommSetRTS(HANDLE hComm, pErrC pError)
{
	long lCode;							//function return code

	//error pointer test
	_ASSERT(!DSCheckPtr(pError));

	//nothing to do in case of an incoming error
	if (pError->lfStatus == LVTRUE)
		return;

	//set the RTS line
	lCode = CommSetRTS(hComm);

	//update the error cluster
	if (lCode == COMM_ERR_SYSTEM)
		lv_WinErrBld(GetLastError(), __FUNCTION__, pError);
	else
		lv_WarnOrErrBld(lCode, __FUNCTION__, pError);
}

/*  lvCommClrRTS
 *
 *  Clears the RTS (request-to-send) signal.
 *
 *  parameter
 *		hComm				serial port handle
 *		pError				Error cluster
 */
void LVSERIAL_API lvCommClrRTS(HANDLE hComm, pErrC pError)
{
	long lCode;							//function return code

	//error pointer test
	_ASSERT(!DSCheckPtr(pError));

	if (pError->lfStatus == LVTRUE)
		return;

	//clear the RTS line
	lCode = CommClrRTS(hComm);

	//update the error cluster
	if (lCode == COMM_ERR_SYSTEM)
		lv_WinErrBld(GetLastError(), __FUNCTION__, pError);
	else
		lv_WarnOrErrBld(lCode, __FUNCTION__, pError);
}

/*  lvCommSetDTR
 *
 *  Sends the DTR (data-terminal-ready) signal.
 *
 *  parameter
 *		hComm				serial port handle
 *		pError				Error cluster
 */
void LVSERIAL_API lvCommSetDTR(HANDLE hComm, pErrC pError)
{
	long lCode;							//function return code

	//error pointer test
	_ASSERT(!DSCheckPtr(pError));

	//nothing to do in case of an incoming error
	if (pError->lfStatus == LVTRUE)
		return;

	//set the DTR line
	lCode = CommSetDTR(hComm);

	//update the error cluster
	if (lCode == COMM_ERR_SYSTEM)
		lv_WinErrBld(GetLastError(), __FUNCTION__, pError);
	else
		lv_WarnOrErrBld(lCode, __FUNCTION__, pError);
}

/*  lvCommClrDTR
 *
 *  Clears the DTR (data-terminal-ready) signal.
 *
 *  parameter
 *		hComm				serial port handle
 *		pError				Error cluster
 */
void LVSERIAL_API lvCommClrDTR(HANDLE hComm, pErrC pError)
{
	long lCode;							//function return code

	//error pointer test
	_ASSERT(!DSCheckPtr(pError));

	//nothing to do in case of an incoming error
	if (pError->lfStatus == LVTRUE)
		return;

	//clear the DTR line
	lCode = CommClrDTR(hComm);

	//update the error cluster
	if (lCode == COMM_ERR_SYSTEM)
		lv_WinErrBld(GetLastError(), __FUNCTION__, pError);
	else
		lv_WarnOrErrBld(lCode, __FUNCTION__, pError);
}

/*  lvCommSetBreak
 *
 *  Suspends character transmission and places the transmission line in a break state 
 *  until the CommClearBreak function is called.
 *
 *  parameter
 *		hComm				serial port handle
 *		pError				Error cluster
 */
void LVSERIAL_API lvCommSetBreak(HANDLE hComm, pErrC pError)
{
	long lCode;							//function return code

	//error pointer test
	_ASSERT(!DSCheckPtr(pError));

	//nothing to do in case of an incoming error
	if (pError->lfStatus == LVTRUE)
		return;

	//set the break state
	lCode = CommSetBreak(hComm);

	//update the error cluster
	if (lCode == COMM_ERR_SYSTEM)
		lv_WinErrBld(GetLastError(), __FUNCTION__, pError);
	else
		lv_WarnOrErrBld(lCode, __FUNCTION__, pError);
}

/*  lvCommClearBreak
 *
 *  Restores character transmission and places the transmission line in a nonbreak 
 *  state.
 *
 *  parameter
 *		hComm				serial port handle
 *		pError				Error cluster
 */
void LVSERIAL_API lvCommClearBreak(HANDLE hComm, pErrC pError)
{
	long lCode;							//function return code

	//error pointer test
	_ASSERT(!DSCheckPtr(pError));
		return;

	//nothing to do in case of an incoming error
	if (pError->lfStatus == LVTRUE)
		return;

	//clear the break state
	lCode = CommClearBreak(hComm);

	//update the error cluster
	if (lCode == COMM_ERR_SYSTEM)
		lv_WinErrBld(GetLastError(), __FUNCTION__, pError);
	else
		lv_WarnOrErrBld(lCode, __FUNCTION__, pError);
}

/*  lvCommConfig
 *
 *  Sets the basic serial port communication parameters.
 *
 *  parameter
 *		hComm				serial port handle
 *		ulBaudRate			baud rate
 *		usDataBits			number of data bits
 *		usParity			parity (see COMM_PARITY_ )
 *		usStopBits			number of stop bits (COMM_ONESTOPBIT, COMM_ONE5STOPBITS,
 *							COMM_TWOSTOPBITS or one od thze values 10, 15 or 20).
 *		pError				Error cluster
 */
void LVSERIAL_API lvCommConfig(
	HANDLE hComm, 
	unsigned long ulBaudRate,
	unsigned short usDataBits,
	unsigned short usParity,
	unsigned short usStopBits, 
	pErrC pError)
{
	long lCode;							//function return code

	//error pointer test
	_ASSERT(!DSCheckPtr(pError));

	//nothing to do in case of an incoming error
	if (pError->lfStatus == LVTRUE)
		return;

	//confugure the serial port
	lCode = CommConfig(hComm, ulBaudRate, usDataBits, usParity, usStopBits);

	//update the error cluster
	if (lCode == COMM_ERR_SYSTEM)
		lv_WinErrBld(GetLastError(), __FUNCTION__, pError);
	else
		lv_WarnOrErrBld(lCode, __FUNCTION__, pError);
}

/*  lvCommFlowControl
 *
 *  Configures the basic flow control settings for the serial port.
 *
 *  parameter
 *		hComm				serial port handle
 *		ulFlowControl		flow control mode (see COMM_FLOW_)
 *		ucXOFFCharacter		XOFF character
 *		ucXONCharacter		XON character
 *		pError				Error cluster
 */
void LVSERIAL_API lvCommFlowControl(
	HANDLE hComm,
	unsigned short usFlowControl,
	unsigned char ucXOFFCharacter,
	unsigned char ucXONCharacter, 
	pErrC pError)
{
	long lCode;							//function return code

	//error pointer test
	_ASSERT(!DSCheckPtr(pError));

	//nothing to do in case of an incoming error
	if (pError->lfStatus == LVTRUE)
		return;

	//set the flow control mode
	lCode = CommFlowControl(hComm, usFlowControl, ucXOFFCharacter, ucXONCharacter);

	//update the error cluster
	if (lCode == COMM_ERR_SYSTEM)
		lv_WinErrBld(GetLastError(), __FUNCTION__, pError);
	else
		lv_WarnOrErrBld(lCode, __FUNCTION__, pError);
}

/*  lvCommControl
 *
 *  Extended flow and error handling control configuration
 *
 *  parameter
 *		hComm				serial port handle
 *		lFlags				specifies the handshake and error settings
 *							use the COMM_CTRL_ flags
 *		XonLim				Minimum number of bytes allowed in the input 
 *							buffer before flow control is activated to 
 *							inhibit the sender. Note that the sender may
 *							transmit characters after the flow control 
 *							signal has been activated, so this value should 
 *							never be zero.
 *		XoffLim				Maximum number of bytes allowed in the input 
 *							buffer before flow control is activated to 
 *							allow transmission by the sender.
 *		XonChar				Value of the XON character for both 
 *							transmission and reception.
 *		XoffChar			Value of the XOFF character for both 
 *							transmission and reception.
 *		ErrorChar			Value of the character used to replace bytes 
 *							received with a parity error. 
 *		pError				error cluster		
 */
void LVSERIAL_API lvCommControl(
	HANDLE hComm,
	unsigned long ulFlags,
	unsigned short usXonLim,
	unsigned short usXoffLim,
	unsigned char ucXonChar,
	unsigned char ucXoffChar,
	unsigned char ucErrorChar,
	pErrC pError)
{
	long lCode;							//function return code

	//error pointer test
	_ASSERT(!DSCheckPtr(pError));

	//nothing to do in case of an incoming error
	if (pError->lfStatus == LVTRUE)
		return;

	//set the flow control and erro handling options
	lCode = CommControl(
		hComm,
		ulFlags,
		usXonLim,
		usXoffLim,
		ucXonChar,
		ucXoffChar,
		ucErrorChar);

	//update the error cluster. In case of COMM_ERR_SYSTEM we test if there
	//was a windows error and if yes, we return this error.
	if (lCode == COMM_ERR_SYSTEM)
	{
		lCode = GetLastError();
		if (lCode != 0)
			lv_WinErrBld(GetLastError(), __FUNCTION__, pError);
		else
			lv_EasyErrBld(COMM_ERR_SYSTEM, __FUNCTION__, pError);
	}
	else
		lv_WarnOrErrBld(lCode, __FUNCTION__, pError);
}

/*  lvCommTermination
 *
 *  Configures the termiation characters for the serial port.
 *
 *  parameter
 *		hComm				serial port handle
 *		fTermCharEnable		specifies weather the termination characters are used
 *							to terminate any read operation.
 *		pcTermCharacter		buffer containing the termination characters
 *		lNumberOfTermChar	number of termination characters in pcTermCharacter
 *		pError				Error cluster
 */
void LVSERIAL_API lvCommTermination(
	HANDLE hComm, 
	LVBoolean *plfTermCharEnable,
	LStrHandle shTermCharacter,
	pErrC pError)
{
	long lCode;							//function return code

	//error pointer test
	_ASSERT(!DSCheckPtr(pError));

	//nothing to do in case of an incoming error
	if (pError->lfStatus == LVTRUE)
		return;

	_ASSERT(!DSCheckPtr(plfTermCharEnable));
	_ASSERT(!DSCheckPtr(shTermCharacter));

	//set the termination condition
	lCode = CommTermination(
		hComm,
		(int)*plfTermCharEnable, 
		(char *)LHStrBuf(shTermCharacter), 
		LHStrLen(shTermCharacter));

	//update the error cluster
	if (lCode == COMM_ERR_SYSTEM)
		lv_WinErrBld(GetLastError(), __FUNCTION__, pError);
	else
		lv_WarnOrErrBld(lCode, __FUNCTION__, pError);
}

/*  lvCommTermination2
 *
 *  Configures the termiation characters for the serial port.
 *
 *  parameter
 *		hComm				serial port handle
 *		lTerminationMode	specifies the termination mode, this can be one of the
 *							following value:
 *								COMM_TERM_NONE		no termination
 *								COMM_TERM_CHAR		one or more termination characters
 *								COMM_TERM_STRING	a single termination string
 *								COMM_TERM_REGEX		a regular expression
 *		pcTermination		buffer containing the termination characters or string
 *		lNumberOfTermChar	number of characters in pcTermination
 *		pError				Error cluster
 */
void LVSERIAL_API lvCommTermination2(
	HANDLE hComm, 
	long lTerminationMode,
	LStrHandle shTermination,
	pErrC pError)
{
	long lCode;							//function return code

	//error pointer test
	_ASSERT(!DSCheckPtr(pError));

	//nothing to do in case of an incoming error
	if (pError->lfStatus == LVTRUE)
		return;

	_ASSERT(!DSCheckPtr(shTermination));

	//set the termination mode
	lCode = CommTermination2(
		hComm,
		lTerminationMode, 
		(char *)LHStrBuf(shTermination), 
		LHStrLen(shTermination));

	//update the error cluster
	if (lCode == COMM_ERR_SYSTEM)
		lv_WinErrBld(GetLastError(), __FUNCTION__, pError);
	else
		lv_WarnOrErrBld(lCode, __FUNCTION__, pError);
}

/*  lvCommWriteMode
 *
 *  Configures the termiation characters for the serial port.
 *
 *  parameter
 *		hComm					serial port handle
 *		lWriteMode				sprecifies the write mode
 *									COMM_WRITE_NORMAL
 *									COMM_WRITE_STX
 *									COMM_WRITE_ETX
 *									COMM_WRITE_STXETX
 *		pcStartOfTransmission	buffer containing the start of transmission string
 *		lNumberOfStartChars		number of characters in pcStartOfTransmission
 *		pcEndOfTransmission		buffer containing the end of transmission string
 *		lNumberOfEndChars		number of characters in pcStartOfTransmission
 *		pError					error cluster
 */
void LVSERIAL_API lvCommWriteMode(
	HANDLE hComm, 
	long lWriteMode,
	LStrHandle shStartOfTransmission,
	LStrHandle shEndOfTransmission,
	pErrC pError)
{
	long lCode;							//function return code

	//error pointer test
	_ASSERT(!DSCheckPtr(pError));

	//nothing to do in case of an incoming error
	if (pError->lfStatus == LVTRUE)
		return;

	_ASSERT(!DSCheckPtr(shStartOfTransmission));
	_ASSERT(!DSCheckPtr(shEndOfTransmission));

	//set the write mode
	lCode = CommWriteMode(
		hComm,
		lWriteMode, 
		(char *)LHStrBuf(shStartOfTransmission), 
		LHStrLen(shStartOfTransmission),
		(char *)LHStrBuf(shEndOfTransmission), 
		LHStrLen(shEndOfTransmission));

	//update the error cluster
	if (lCode == COMM_ERR_SYSTEM)
		lv_WinErrBld(GetLastError(), __FUNCTION__, pError);
	else
		lv_WarnOrErrBld(lCode, __FUNCTION__, pError);
}

/*  lvCommTimeout
 *
 *  Configures the timeout value for read and write operations
 *
 *  parameter
 *		hComm				serial port handle
 *		ulTimeout			timeout in milliseconds
 *		pError				error cluster
 */
void LVSERIAL_API lvCommTimeout(
	HANDLE hComm, 
	unsigned long ulTimeout, 
	pErrC pError)
{
	long lCode;							//function return code

	//error pointer test
	_ASSERT(!DSCheckPtr(pError));

	//nothing to do in case of an incoming error
	if (pError->lfStatus == LVTRUE)
		return;

	//set the timeout value
	lCode = CommTimeout(hComm, ulTimeout);

	//update the error cluster
	if (lCode == COMM_ERR_SYSTEM)
		lv_WinErrBld(GetLastError(), __FUNCTION__, pError);
	else
		lv_WarnOrErrBld(lCode, __FUNCTION__, pError);
}

/*  lvCommTimeout2
 *
 *  Configures the timeout value for read and write operations
 *
 *  parameter
 *		hComm							serial port handle
 *		ulReadIntervalTimeout			timeout in milliseconds between two characters
 *		ulReadTotalTimeoutMultiplier	timeout value for each requested character
 *		ulReadTotalTimeoutConstant		timeout constant
 *		ulWriteTotalTimeoutMultiplier	timeout value for each character to write
 *		ulWriteTotalTimeoutConstant		timeout constant
 *		pError							error cluster
 */
void LVSERIAL_API lvCommTimeout2(
	HANDLE hComm, 
	unsigned long ulReadIntervalTimeout,
	unsigned long ulReadTotalTimeoutMultiplier,
	unsigned long ulReadTotalTimeoutConstant,
	unsigned long ulWriteTotalTimeoutMultiplier,
	unsigned long ulWriteTotalTimeoutConstant,
	pErrC pError)
{
	long lCode;							//function return code

	//error pointer test
	_ASSERT(!DSCheckPtr(pError));

	//nothing to do in case of an incoming error
	if (pError->lfStatus == LVTRUE)
		return;

	//set the extended timeout value
	lCode = CommTimeout2(
		hComm, 
		ulReadIntervalTimeout,
		ulReadTotalTimeoutMultiplier,
		ulReadTotalTimeoutConstant,
		ulWriteTotalTimeoutMultiplier,
		ulWriteTotalTimeoutConstant);

	//update the error cluster
	if (lCode == COMM_ERR_SYSTEM)
		lv_WinErrBld(GetLastError(), __FUNCTION__, pError);
	else
		lv_WarnOrErrBld(lCode, __FUNCTION__, pError);
}

/*  lvCommOptions
 *
 *  Sets the serial port options. This function is also called inside
 *  the CommOpen function and allows the user to specify the major 
 *	settings together with the serial port name in one string.
 *
 *  parameter
 *		hComm				serial port handle
 *		shOptions			serial port options string
 *		pError				error cluster
 *		
 *	Options specification
 *	There are two generic different formats of this string:
 *
 *	<baud>,<parity>,<data>,<stop>[,<handshake>]
 *
 *	where
 *		baud		any valid baudrate
 *		partity		N=none, O=odd, E=even, M=mark, S=space
 *		data		is the number of data bits
 *		stop		is the number of stop bits (1, 1.5 or 2)
 *					valid values are also 10, 15 and 20)
 *		handshake	either x (XON/XOFF) or p (DTR/DSR and RTS/CTS handshake)
 *
 *
 *	baudrate:<baud>; parity:<parity>; handshake:<hand>; 
 *  databits=<data>; stopbits:<stop>; timeout:<timeout>
 *
 *	where
 *		baud		any valid baudrate
 *		partity		N=none, O=odd, E=even, M=mark, S=space
 *					(only the first character is tested)
 *		handshake	N=none, X=xonxoff, R=rstcts, D=dsrdtr
 *					(only the first character is tested)
 *		data		is the number of data bits
 *		stop		is the number of stop bits (1, 1.5 or 2)
 *					valid values are also 10, 15 and 20)
 *		timeout		is the read and write timeout in milliseconds
 *		xonchar		specifies the XON character
 *		xoffchar	specifies the XOFF character
 *		xon			enables (on) or disables (off) the XON/XOFF handshake
 *		odsr		Indicates whether the DSR (data-set-ready) signal is 
 *					monitored for output flow control. Possible values 
 *					are on or off.
 *		octs		Indicates whether the CTS (clear-to-send) signal is 
 *					monitored for output flow control. Possible values
 *					are on or off.
 *		dtr			sets the DTR flow control mode to on, off or hs (=handshake)
 *		rts			sets the RTS flow control mode to on, off, hs (=handshake) 
 *					or ts (=toggle)
 *		idsr		this value can be on or off. If it is on, the driver
 *					is sensitive to the DSR signal. The driver only reveives
 *					bytes, if the DSR input line is high.
 *		
 *
 *	String and character compirison is not case sensitive. The second
 *	format is less strict, so you can place whitespaces between the 
 *	keywords and values. You can use "=" instead of ":" and you must't
 *	specify ";" between the values (a single whitespace character is 
 *	enough).
 */
void LVSERIAL_API lvCommOptions(HANDLE hComm, LStrHandle shOptions, pErrC pError)
{
	long lResult;						//fucntion return code

	//error pointer test
	_ASSERT(!DSCheckPtr(pError));

	//nothing to do in case of an incoming error
	if (pError->lfStatus == LVTRUE)
		return;

	_ASSERT(!DSCheckPtr(shOptions));

	//convert the shOptions buffer into a null terminated string
	if ((lResult = NumericArrayResize(uB, 1L, (UHandle *)&shOptions, LHStrLen(shOptions)+1)) != noErr)
	{
		lv_EasyErrBld(lResult, __FUNCTION__, pError);
		return;
	}
	*(LHStrBuf(shOptions)+LHStrLen(shOptions)) = '\0';

	//set the options specified in shOptions
	if ((lResult = CommOptions(hComm, (char *)LHStrBuf(shOptions))) != COMM_SUCCESS)
	{
		//update the error cluster
		if (lResult != COMM_ERR_ARGUMENT)
			lv_WinErrBld(GetLastError(), __FUNCTION__, pError);
		else
			lv_EasyErrBld(lResult, __FUNCTION__, pError);
	}
}

/*  lvCommLock
 *
 *  Locks the serial port
 *
 *  parameter
 *		hComm				serial port handle
 *		ulTimeout			timeout in milliseconds
 *		pError				Error cluster
 */
void LVSERIAL_API lvCommLock(HANDLE hComm, unsigned long ulTimeout, pErrC pError)
{
	long lCode;							//function return code

	//error pointer test
	_ASSERT(!DSCheckPtr(pError));

	//nothing to do in case of an incoming error
	if (pError->lfStatus == LVTRUE)
		return;

	//lock the serial port
	lCode = CommLock(hComm, ulTimeout);

	//update the error cluster
	if (lCode == COMM_ERR_SYSTEM)
		lv_WinErrBld(GetLastError(), __FUNCTION__, pError);
	else
		lv_WarnOrErrBld(lCode, __FUNCTION__, pError);
}

/*  lvCommLock2
 *
 *  Locks the serial port
 *
 *  parameter
 *		hComm				serial port handle
 *		ulTimeout			timeout in milliseconds
 *		plfLocked			set to 1 if the serial port is locked
 *		pError				Error cluster
 */
void LVSERIAL_API lvCommLock2(HANDLE hComm, unsigned long ulTimeout, LVBoolean *pfLocked, pErrC pError)
{
	long lCode;							//function return code

	//error pointer test
	_ASSERT(!DSCheckPtr(pError));

	//nothing to do in case of an incoming error
	if (pError->lfStatus == LVTRUE)
		return;

	//lock the serial port
	lCode = CommLock(hComm, ulTimeout);

	if (pfLocked != NULL)
		*pfLocked = lCode == COMM_SUCCESS ? TRUE : FALSE;

	//update the error cluster
	if (lCode == COMM_ERR_SYSTEM)
		lv_WinErrBld(GetLastError(), __FUNCTION__, pError);
	else
		lv_WarnOrErrBld(lCode, __FUNCTION__, pError);
}

/*  lvCommUnLock
 *
 *  Unlocks the serial port
 *
 *  parameter
 *		hComm				serial port handle
 *		pError				Error cluster
 */
void LVSERIAL_API lvCommUnLock(HANDLE hComm, pErrC pError)
{
	long lCode;							//function return code

	//error pointer test
	_ASSERT(!DSCheckPtr(pError));

	//unlock the serial port
	lCode = CommUnLock(hComm);

	//nothing to do in case of an incoming error
	if (pError->lfStatus == LVTRUE)
		return;

	//update the error cluster
	if (lCode == COMM_ERR_SYSTEM)
		lv_WinErrBld(GetLastError(), __FUNCTION__, pError);
	else
		lv_WarnOrErrBld(lCode, __FUNCTION__, pError);
}

/*  lvCommUnLock2
 *
 *  Unlocks the serial port
 *
 *  parameter
 *		hComm				serial port handle
 *		plfLocked			port will be unlocked if this value is TRUE
 *		pError				Error cluster
 */
void LVSERIAL_API lvCommUnLock2(HANDLE hComm, LVBoolean *pfLocked, pErrC pError)
{
	long lCode;							//function return code

	if (pfLocked != NULL)
	{
		if (!(*pfLocked))
			return;
	}

	//error pointer test
	_ASSERT(!DSCheckPtr(pError));

	//unlock the serial port
	lCode = CommUnLock(hComm);

	//nothing to do in case of an incoming error
	if (pError->lfStatus == LVTRUE)
		return;

	//update the error cluster
	if (lCode == COMM_ERR_SYSTEM)
		lv_WinErrBld(GetLastError(), __FUNCTION__, pError);
	else
		lv_WarnOrErrBld(lCode, __FUNCTION__, pError);
}

/*  lvCommSetUserData
 *
 *  Sets the user specific data for the serial port
 *
 *  parameter
 *		hComm				serial port handle
 *		ulUserData			user data
 *		pError				Error cluster
 */
void LVSERIAL_API lvCommSetUserData(HANDLE hComm, unsigned long ulUserData, pErrC pError)
{
	long lCode;							//function return code

	//error pointer test
	_ASSERT(!DSCheckPtr(pError));

	//nothing to do in case of an incoming error
	if (pError->lfStatus == LVTRUE)
		return;

	//set the user data
	lCode = CommSetUserData(hComm, ulUserData);

	//update the error cluster
	if (lCode == COMM_ERR_SYSTEM)
		lv_WinErrBld(GetLastError(), __FUNCTION__, pError);
	else
		lv_WarnOrErrBld(lCode, __FUNCTION__, pError);
}

/*  lvCommGetUserData
 *
 *  Returns the user specific data
 *
 *  parameter
 *		hComm				serial port handle
 *		pulUserData			pointer to the location where thsi function stores
 *							the user data.
 *		pError				Error cluster
 */
void LVSERIAL_API lvCommGetUserData(HANDLE hComm, unsigned long *pulUserData, pErrC pError)
{
	long lCode;							//function return code

	//error pointer test
	_ASSERT(!DSCheckPtr(pError));
	_ASSERT(!DSCheckPtr(pulUserData));

	//in case of an incoming error, we return a user data value of zero
	if (pError->lfStatus == LVTRUE)
	{
		*pulUserData = 0;
		return;
	}

	//get the user data
	lCode = CommGetUserData(hComm, pulUserData);

	//update the error cluster
	if (lCode == COMM_ERR_SYSTEM)
		lv_WinErrBld(GetLastError(), __FUNCTION__, pError);
	else
		lv_WarnOrErrBld(lCode, __FUNCTION__, pError);
}

/*  lvCommSetUserDataBuffer
 *
 *  Sets the user specific data for the serial port
 *
 *  parameter
 *		hComm				serial port handle
 *		shUserData			user data
 *		pError				Error cluster
 */
void LVSERIAL_API lvCommSetUserDataBuffer(HANDLE hComm, LStrHandle shUserData, pErrC pError)
{
	PCOMM pComm;						//pointer to the serial port COMM structure

	//pointer test
	_ASSERT(gpCommList);
	_ASSERT(!DSCheckPtr(pError));
	_ASSERT(!DSCheckHandle(shUserData));

	//nothing to do in case of an incoming error
	if (pError->lfStatus == LVTRUE)
		return;

	//test the given serial port handle and get the pointer to the
	//serial port data structure.
	__CommLock(ghLock);
	if (ListCheckHandle(gpCommList, hComm) != 0)
	{
		__CommUnLock(ghLock);
		lv_EasyErrBld(COMM_ERR_HANDLE, __FUNCTION__, pError);
		return;
	}

	pComm = (PCOMM)ListGetElement(gpCommList, hComm);
	__CommUnLock(ghLock);

	_ASSERT(pComm);

	//copy the user data

	if (pComm->pcUserData != NULL)
		free(pComm->pcUserData);

	if ((pComm->pcUserData = malloc(LHStrLen(shUserData))) == NULL)
	{
		lv_EasyErrBld(COMM_ERR_OUTOFMEMORY, __FUNCTION__, pError);
		return;
	}

	memcpy(pComm->pcUserData, LHStrBuf(shUserData), LHStrLen(shUserData));
	pComm->lUserDataSize = LHStrLen(shUserData);
}

/*  lvCommGetUserDataBuffer
 *
 *  Returns the user specific data
 *
 *  parameter
 *		hComm				serial port handle
 *		shUserData			user data
 *		pError				Error cluster
 */
void LVSERIAL_API lvCommGetUserDataBuffer(HANDLE hComm, LStrHandle shUserData, pErrC pError)
{
	PCOMM pComm;						//pointer to the serial port COMM structure
	long lResult;						//function return code

	//pointer test
	_ASSERT(gpCommList);
	_ASSERT(!DSCheckPtr(pError));
	_ASSERT(!DSCheckHandle(shUserData));

	//nothing to do in case of an incoming error
	if (pError->lfStatus == LVTRUE)
	{
		LStrLen(*shUserData) = 0;
		return;
	}

	//test the given serial port handle and get the pointer to the
	//serial port data structure.
	__CommLock(ghLock);
	if (ListCheckHandle(gpCommList, hComm) != 0)
	{
		__CommUnLock(ghLock);
		LStrLen(*shUserData) = 0;
		lv_EasyErrBld(COMM_ERR_HANDLE, __FUNCTION__, pError);
		return;
	}

	pComm = (PCOMM)ListGetElement(gpCommList, hComm);
	__CommUnLock(ghLock);

	_ASSERT(pComm);

	//copy the user data
	if ((lResult = NumericArrayResize(uB, 1L, (UHandle *)&shUserData, pComm->lUserDataSize)) != noErr)
	{
		lv_EasyErrBld(lResult, __FUNCTION__, pError);
		LStrLen(*shUserData) = 0;
		return;
	}

	LStrLen(*shUserData) = pComm->lUserDataSize;
	if (pComm->pcUserData != NULL)
		memcpy(LStrBuf(*shUserData), pComm->pcUserData, pComm->lUserDataSize);
}

/*  __lvDisposeStringArrayElements
 *
 *  Disposes all string handles of the array and sets the array size to zero.
 *	This VI ignores any incoming error.
 *
 *  parameter
 *		phalsArray			handle to the array of strings
 *		pError				error cluster
 */
void __lvDisposeStringArrayElements(LStrArrayHandle *phalsArray, pErrC pError)
{
	long lIndex;
	MgErr lErr;

	//clean up the array
	if (*phalsArray != NULL)
	{
		for (lIndex = 0; lIndex < (**phalsArray)->dimSize; lIndex++)
		{
			DSDisposeHandle((**phalsArray)->elt[lIndex]);	
		}
	}

	//adjust array size or allocate new array
	if ((lErr = NumericArrayResize( uL, 1L, (UHandle *)phalsArray, 1)) != noErr)
	{
		lv_EasyErrBld(lErr, __FUNCTION__, pError);
		return;
	}
	(**phalsArray)->dimSize = 0;
	(**phalsArray)->elt[0] = NULL;
}
    
/*  lvCommListPorts
 *
 *  Lists all serial ports and places them into a array of pointers to zero
 *  terminated strings. The array and the strings are allocated by this
 *  function and should. Call CommFree to release the memory.
 *
 *  parameter
 *		halsPorts			handle to the array of strings (port names)
 *		pError				error cluster
 */
void LVSERIAL_API lvCommListPorts(LStrArrayHandle *phalsPorts, pErrC pError)
{
	MgErr err;							//labview function return code
	long lArraySize;					//array size
	long lErr;							//function error code
	HKEY hKey = NULL;					//registry key handle
	unsigned long ulIndex;				//index of registry value
	int fContinue;						//continue enumeration
	unsigned long ulValueBufSize;		//size of value buffer
	unsigned long ulPortBufSize;		//size of port name buffer
	unsigned long ulDfltValueBufSize;	//default value buffer size
	unsigned long ulDfltPortBufSize;	//default port name buffer size
	LPTSTR lpValue;						//registry value name
	char *pszPort;						//serial port name from registry
	LStrHandle shPort;					//serial port name handle to append

	_ASSERT(!DSCheckPtr(pError));
	_ASSERT(!DSCheckPtr(phalsPorts));

	//clean up the incoming array
	__lvDisposeStringArrayElements(phalsPorts, pError);

	//do nothing else if there is an incoming pError
	if (pError->lfStatus == LVTRUE)
		return;

	//open the registry key "HKLM\HARDWARE\DEVICEMAP\SERIALCOMM"
	if ((lErr=RegOpenKeyEx(
			HKEY_LOCAL_MACHINE,
			"HARDWARE\\DEVICEMAP\\SERIALCOMM",
			0,KEY_READ,
			&hKey)) != ERROR_SUCCESS)
	{
		lv_WinErrBld(lErr, __FUNCTION__, pError);
		return;
	}

	//allocate the buffers for the value name and port name. If the size for
	//the value name is not large enough, we allocate a larger buffer if needed.
	//The ulDflt..BufSize variables are the allocated buffer size and ul..BufSize
	//is the variable used to call RegEnumValue().

	//allocate the value name buffer
	ulDfltValueBufSize = 32 * sizeof(TCHAR);
	if ((lpValue = malloc(ulDfltValueBufSize)) == NULL)
	{
		RegCloseKey(hKey);
		lv_EasyErrBld(COMM_ERR_OUTOFMEMORY, __FUNCTION__, pError);
		return;
	}

	//allocate the port name buffer
	ulDfltPortBufSize = 32 * sizeof(TCHAR);
	if ((pszPort = malloc(ulDfltPortBufSize)) == NULL)
	{
		RegCloseKey(hKey);
		free(lpValue);
		lv_EasyErrBld(COMM_ERR_OUTOFMEMORY, __FUNCTION__, pError);
		return;
	}

	//now we are ready to start to retrieve all serial ports.
	for (ulIndex = 0, fContinue = TRUE; fContinue; ulIndex++)
	{
		ulValueBufSize = ulDfltValueBufSize;
		ulPortBufSize = ulDfltPortBufSize;

		//retrieve the serial port name
		lErr = RegEnumValue(
				hKey,
				ulIndex,
				lpValue,
				&ulValueBufSize,
				NULL,
				NULL,
				pszPort,
				&ulPortBufSize);

		if (lErr == ERROR_SUCCESS)
		{
			//operation sucessful: append the port name to array

			//create new handle for the next port name
			if ( (shPort = (LStrHandle)DSNewHandle(sizeof(LStr))) == NULL)
			{
				__lvDisposeStringArrayElements(phalsPorts, pError);
				RegCloseKey(hKey);
				free(lpValue);
				free(pszPort);
				lv_EasyErrBld(COMM_ERR_OUTOFMEMORY, __FUNCTION__, pError);
				return;
			}
			LStrLen(*shPort) = 0;

			//copy the port name to the new string
			LStrPrintf(shPort, (CStr)"%s", pszPort);

			//insert the new port name (shPort) into the string array
			lArraySize = (**(phalsPorts))->dimSize +1;
			if ((err = NumericArrayResize( uL, 1L, (UHandle *)phalsPorts, lArraySize)) != noErr)
			{
				__lvDisposeStringArrayElements(phalsPorts, pError);
				RegCloseKey(hKey);
				DSDisposeHandle(shPort);
				free(lpValue);
				free(pszPort);
				lv_EasyErrBld(COMM_ERR_OUTOFMEMORY, __FUNCTION__, pError);
				return;
			}
			(**(phalsPorts))->dimSize = lArraySize;
			(**(phalsPorts))->elt[lArraySize-1] = shPort;
		}
		else if (lErr == ERROR_MORE_DATA)
		{ 
			//there is not enough space for the value name. If the buffer size 
			//is lower than 1024 byte, we try doubling. If the actual buffer size
			//is larger than 1024 byte, we abort the operation.
			if (ulDfltValueBufSize < 1024)
			{
				ulIndex--;
				free(lpValue);
				ulDfltValueBufSize *= 2;
				if ((lpValue = malloc(ulDfltValueBufSize)) == NULL)
				{
					__lvDisposeStringArrayElements(phalsPorts, pError);
					RegCloseKey(hKey);
					free(pszPort);
					lv_WinErrBld(lErr, __FUNCTION__, pError);
					return;
				}
			}
			else
			{
				//buffer is larger than 1024 byte: Abort with error
				__lvDisposeStringArrayElements(phalsPorts, pError);
				RegCloseKey(hKey);
				free(lpValue);
				free(pszPort);
				lv_WinErrBld(lErr, __FUNCTION__, pError);
				return;
			}
		}
		else if (lErr == ERROR_NO_MORE_ITEMS)
			//there are no more items - finish
			fContinue = FALSE;
		else
		{
			//any other error
			__lvDisposeStringArrayElements(phalsPorts, pError);
			RegCloseKey(hKey);
			free(lpValue);
			free(pszPort);
			lv_WinErrBld(lErr, __FUNCTION__, pError);
			return;
		}
	}

	//close the key and free the buffers.
	RegCloseKey(hKey);
	free(lpValue);
	free(pszPort);
}

/*  lvCommGetHandles
 *
 *	This function returns a list of all existing serial port handles.
 *
 *  parameter
 *		haulHandles			array handle where this function stores the
 *							handles
 *		pErrC				error cluster
 */
void LVSERIAL_API lvCommGetHandles(ULongArrayHandle *phaulHandles, pErrC pError)
{
	MgErr err;							//labview function return code
	long lSize;
	long i;								//index number
	HANDLE hH;

	_ASSERT(!DSCheckPtr(pError));

	//do nothing if there is an incoming pError
	if (pError->lfStatus == LVTRUE)
		return;

	_ASSERT(!DSCheckPtr(phaulHandles));

	__CommLock(ghLock);
	lSize = ListGetLength(gpCommList);

	//adjust array size or allocate new array
	if ((err = NumericArrayResize( uL, 1L, (UHandle *)phaulHandles, lSize)) != noErr)
	{
		__CommUnLock(ghLock);
		(**phaulHandles)->dimSize = 0;
		lv_EasyErrBld(err, __FUNCTION__, pError);
		return;
	}
	(**phaulHandles)->dimSize = lSize;
	(void *)(**phaulHandles)->elt[0] = NULL;

	hH = ListGetFirstHandle(gpCommList);
	(**phaulHandles)->elt[0] = (uInt32)hH;
	
	for (i = 1; i < lSize; i++)
	{
		hH = ListGetNextHandle(gpCommList, hH);
		(**phaulHandles)->elt[i] = (uInt32)hH;
	}

	__CommUnLock(ghLock);
}

/*  lvCommGetPortName
 *
 *	This function returns the real port name and or the original port name, 
 *	which was used when CommOpenPort was called.
 *
 *  parameter
 *		hComm				serial port handle
 *		shName				string where this function stores the
 *							real port name.
 *		shOriginal			string where this functions stores 
 *							the original port name.
 *		pError				error cluster
 */
void LVSERIAL_API lvCommGetPortName(HANDLE hComm, LStrHandle shName, LStrHandle shOriginal, pErrC pError)
{
	PCOMM pComm;						//pointer to the serial port COMM structure
	MgErr err;

	//pointer test
	_ASSERT(!DSCheckPtr(shName));
	_ASSERT(!DSCheckPtr(shOriginal));
	_ASSERT(!DSCheckPtr(pError));

	//nothing to do in case of an incoming error
	if (pError->lfStatus == LVTRUE)
	{
		LStrLen(*shName) = 0;
		LStrLen(*shOriginal) = 0;
		return;
	}

	_ASSERT(gpCommList);

	__CommLock(ghLock);
	if (ListCheckHandle(gpCommList, hComm) != 0)
	{
		__CommUnLock(ghLock);
		LStrLen(*shName) = 0;
		LStrLen(*shOriginal) = 0;
		lv_EasyErrBld(COMM_ERR_HANDLE, __FUNCTION__, pError);
		return;
	}

	pComm = (PCOMM)ListGetElement(gpCommList, hComm);
	__CommUnLock(ghLock);

	_ASSERT(pComm);

	//copy the port name string to the shName string handle
	if ((err = LStrPrintf(shName, (CStr)"%s", pComm->pszPort)) != noErr)
	{
		LStrLen(*shName) = 0;
		LStrLen(*shOriginal) = 0;
		lv_EasyErrBld(COMM_ERR_HANDLE, __FUNCTION__, pError);
	}

	//copy the original port name string to the shOriginal string handle
	if ((err = LStrPrintf(shOriginal, (CStr)"%s", pComm->pszOriginalName)) != noErr)
	{
		LStrLen(*shName) = 0;
		LStrLen(*shOriginal) = 0;
		lv_EasyErrBld(COMM_ERR_HANDLE, __FUNCTION__, pError);
	}
}

/*  lvCommEnableEvent
 *
 *	This function specifies a set of events to be monitored for a 
 *	communications device. 
 *
 *  parameter
 *		hComm				serial port handle
 *		ulEventMask			event mask
 *								COMM_EV_BREAK	A break was detected on input. 
 *								COMM_EV_CTS		The CTS (clear-to-send) signal 
 *												changed state. 
 *								COMM_EV_DSR		The DSR (data-set-ready) signal 
 *												changed state. 
 *								COMM_EV_ERR		A line-status error occurred. Line-
 *												status errors are CE_FRAME, 
 *												CE_OVERRUN and CE_RXPARITY. 
 *								COMM_EV_RING	A ring indicator was detected. 
 *								COMM_EV_RLSD	The RLSD (receive-line-signal-detect)
 *												signal changed state. 
 *								COMM_EV_RXCHAR	A character was received and placed 
 *												in the input buffer. 
 *								COMM_EV_RXFLAG	The event character was received and 
 *												placed in the input buffer. The event 
 *												character is specified in the device's DCB structure, which is applied to a serial port by using the SetCommState function. 
 *								COMM_EV_TXEMPTY	The last character in the output 
 *												buffer was sent. 
 *		pError				error cluster
 */
void LVSERIAL_API lvCommEnableEvent(HANDLE hComm, unsigned long ulEventMask, pErrC pError)
{
	long lResult;

	//pointer test
	_ASSERT(!DSCheckPtr(pError));

	//nothing to do in case of an incoming error
	if (pError->lfStatus == LVTRUE)
		return;

	//set the event mask
	lResult = CommEnableEvent(hComm, ulEventMask);

	//update the error cluster
	if (lResult == COMM_ERR_SYSTEM)
		lv_WinErrBld(GetLastError(), __FUNCTION__, pError);
	else
		lv_WarnOrErrBld(lResult, __FUNCTION__, pError);
}

/*  lvCommWaitForEvent
 *
 *	This function waits for an event to occur for a specified communications device. 
 *	The set of events that are monitored by this function is contained in the event 
 *	mask associated with the device handle. 
 *
 *  parameter
 *		hComm				serial port handle
 *		ulTimeout			Specifies the time-out interval, in milliseconds.
 *		pulEventMask		Pointer to a variable that receives a mask indicating 
 *							the type of event that occurred. If an error occurs, 
 *							the value is zero; otherwise, it is one of the COMM_EV_
 *							values (see CommSetEventMask).
 *		pError				error cluster
 */
void LVSERIAL_API lvCommWaitForEvent(HANDLE hComm, unsigned long ulTimeout, unsigned long *pulEventMask, pErrC pError)
{
	long lResult;

	//pointer test
	_ASSERT(!DSCheckPtr(pulEventMask));

	//nothing to do in case of an incoming error
	if (pError->lfStatus == LVTRUE)
	{
		*pulEventMask = 0;
		return;
	}

	//wait for the communication event
	lResult = CommWaitForEvent(hComm, ulTimeout, pulEventMask);

	//update the error cluster
	if (lResult == COMM_ERR_SYSTEM)
		lv_WinErrBld(GetLastError(), __FUNCTION__, pError);
	else
		lv_WarnOrErrBld(lResult, __FUNCTION__, pError);
}

/*  lvCommWaitForEvent
 *
 *	This function disables the monitoring of communication events.
 *
 *  parameter
 *		hComm				serial port handle
 *		pError				error cluster
 */
void LVSERIAL_API lvCommDisableEvent(HANDLE hComm, pErrC pError)
{
	long lResult;

	//wait for the communication event
	lResult = CommDisableEvent(hComm);

	//update the error cluster
	if (lResult == COMM_ERR_SYSTEM)
		lv_WinErrBld(GetLastError(), __FUNCTION__, pError);
	else
		lv_WarnOrErrBld(lResult, __FUNCTION__, pError);
}

/*  lvCommSetEventChar
 *
 *	This function sets the event character.
 *
 *  parameter
 *		hComm				serial port handle
 *		ucEventChar			event character
 *		
 *  return
 *		error code
 */
void LVSERIAL_API lvCommSetEventChar(HANDLE hComm, unsigned char ucEventChar, pErrC pError)
{
	long lResult;

	//fo nothing in case of an incoming error
	if (pError->lfStatus == LVTRUE)
		return;

	//set the event character
	lResult = CommSetEventChar(hComm, ucEventChar);

	//update the error cluster
	if (lResult == COMM_ERR_SYSTEM)
		lv_WinErrBld(GetLastError(), __FUNCTION__, pError);
	else
		lv_WarnOrErrBld(lResult, __FUNCTION__, pError);
}

/*  lvCommTransmitChar
 *
 *	The lvCommTransmitChar function transmits a specified character ahead of 
 *	any pending data in the output buffer of the specified communications 
 *	device.
 *
 *  parameter
 *		hComm				serial port handle
 *		ucChar				character to be transmitted
 *		pError				error cluster
 */
void LVSERIAL_API lvCommTransmitChar(HANDLE hComm, unsigned char ucChar, pErrC pError)
{
	PCOMM pComm;						//pointer to the serial port COMM structure

	_ASSERT(gpCommList);

	if (pError->lfStatus == LVTRUE)
		return;

	//test the given serial port handle and get the pointer to the
	//serial port data structure.
	if (ListCheckHandle(gpCommList, hComm) != 0)
	{
		lv_EasyErrBld(COMM_ERR_HANDLE, __FUNCTION__, pError);
		return;
	}

	pComm = (PCOMM)ListGetElement(gpCommList, hComm);

	_ASSERT(pComm);


	if (!TransmitCommChar(pComm->hC, (char)ucChar))
		lv_WinErrBld(GetLastError(), __FUNCTION__, pError);
}

/*  lvCommSetDefaultConfig
 *
 *	The lvCommSetDefaultConfig function sets the default configuration for 
 *  a communications device.
 *
 *  parameter
 *		shPort				serial port name
 *		shOptions			options string (see mode command)
 *		
 *  return
 *		error code
 */
void LVSERIAL_API lvCommSetDefaultConfig(LStrHandle shPort, LStrHandle shOptions, pErrC pError)
{
	long lResult;

	_ASSERT(!DSCheckHandle(shPort));
	_ASSERT(!DSCheckHandle(shOptions));

	//convert the buffer of shPort into a null terminated string
	if ((lResult = NumericArrayResize(uB, 1L, (UHandle *)&shPort, LHStrLen(shPort)+1)) != noErr)
	{
		lv_EasyErrBld(lResult, __FUNCTION__, pError);
		return;
	}
	*(LHStrBuf(shPort)+LHStrLen(shPort)) = '\0';

	//convert the buffer of shOptions into a null terminated string
	if ((lResult = NumericArrayResize(uB, 1L, (UHandle *)&shOptions, LHStrLen(shOptions)+1)) != noErr)
	{
		lv_EasyErrBld(lResult, __FUNCTION__, pError);
		return;
	}
	*(LHStrBuf(shOptions)+LHStrLen(shOptions)) = '\0';

	lResult = CommSetDefaultConfig(LHStrBuf(shPort), LHStrBuf(shOptions));

	if (lResult == COMM_ERR_SYSTEM)
		lv_WinErrBld(GetLastError(), __FUNCTION__, pError);
	else
		lv_WarnOrErrBld(lResult, __FUNCTION__, pError);
}


/*----------------------------------------------------------------------------------*/

/*  ogCommOpenPort
 *
 *  Opens the serial port
 *
 *  parameter
 *		phComm				pointer where the function stores the serial port handle
 *		pszPort				name of the serial port to open
 *		plfDuplicate		if this value is true, the function returns a duplicate 
 *							handle if the port is already open
 *		pError				Error cluster
 */
void LVSERIAL_API ogCommOpenPort(PHANDLE phComm, LStrHandle shPort, LVBoolean *plfDuplicate, pErrC pError)
{
	lvCommOpenPort(phComm, shPort, plfDuplicate, pError);
}

/*  ogCommClosePort
 *
 *  Closes the serial port. This function is implementd for backward
 *	compatibility and it only calls lvCommClosePort.
 *
 *  parameter
 *		phComm				serial port handle
 *		pError				Error cluster
 */
void LVSERIAL_API ogCommClosePort(PHANDLE phComm, pErrC pError)
{
	lvCommClosePort(*phComm, pError);
}

/*  ogCommClosePort2
 *
 *  Closes the serial port
 *
 *  parameter
 *		phComm				serial port handle
 *		plfCloseAll			if this value is TRUE, the function closes
 *							all opened instances of this port
 *		pError				Error cluster
 */
void LVSERIAL_API ogCommClosePort2(PHANDLE phComm, LVBoolean *plfCloseAll, pErrC pError)
{
	lvCommClosePort2(*phComm, plfCloseAll, pError);
}

/*  ogCommClosePort3
 *
 *  Closes the serial port
 *
 *  parameter
 *		phComm				serial port handle
 *		plfCloseAll			if this value is TRUE, the function closes
 *							all opened instances of this port
 *		ulTimeout			Timeout waiting for pending read and write operations
 *		pError				Error cluster
 */
void LVSERIAL_API ogCommClosePort3(PHANDLE phComm, LVBoolean *plfCloseAll, unsigned long ulTimeout, pErrC pError)
{
	lvCommClosePort3(*phComm, plfCloseAll, ulTimeout, pError);
}

/*  ogCommCloseAll
 *
 *  Closes all serial ports
 *
 *  parameter
 *		pError				Error cluster
 */
void LVSERIAL_API ogCommCloseAll(pErrC pError)
{
	lvCommCloseAll(pError);
}

/*  ogCommWrite
 *
 *  Writes data to the serial port
 *
 *	The function waits until any pending writes are finished using the
 *	specified timeout value. The write operation itself can also last
 *	up to the specified timeout value. This means, that the complete
 *	execution of this function can last up to twice the specified timeout.
 *
 *  parameter
 *		phComm				serial port handle
 *		shBuffer			buffer containing the bytes to write
 *		ulBytesToWrite		number of bytes to write
 *		pulBytesWritten		pointer where this function stores the number of 
 *							bytes written to the serial port
 *		pError				Error cluster
 */
void LVSERIAL_API ogCommWrite(
	PHANDLE phComm, 
	LStrHandle shBuffer, 
	unsigned long * pulBytesWritten, 
	pErrC pError)
{
	lvCommWrite(*phComm, shBuffer, pulBytesWritten, pError);
}

/*  ogCommWriteAndReceiveEcho
 *
 *  Writes data to the serial port and received the echo. Note, that this
 *	function does not handle the specified termination condition.
 *
 *  parameter
 *		phComm				serial port handle
 *		shWriteBuffer		buffer containing the bytes to write
 *		pulBytesWritten		pointer where this function stores the number of 
 *							bytes written to the serial port
 *		shReadBuffer		This is the location where this function stores
 *							the received characters
 *		plfStopIfNotEqual	If this value is true, the transmission is stopped,
 *							if the received character is not equal to the
 *							transmitted character.
 *	pError					error cluster
 */
void LVSERIAL_API ogCommWriteAndReceiveEcho(
	PHANDLE phComm, 
	LStrHandle shWriteBuffer,
	unsigned long * pulBytesWritten,
	LStrHandle shReadBuffer,
	unsigned long * pulBytesRead,
	LVBoolean *plfStopIfNotEqual,
	pErrC pError)
{
	lvCommWriteAndReceiveEcho(
		*phComm, 
		shWriteBuffer,
		pulBytesWritten,
		shReadBuffer,
		pulBytesRead,
		plfStopIfNotEqual,
		pError);
}

/*  ogCommRead
 *
 *  Reads bytes from the serial port and stores the data into the specified
 *  buffer.
 *
 *	The function waits until any pending writes are finished using the
 *	specified timeout value. The write operation itself can also last
 *	up to the specified timeout value. This means, that the complete
 *	execution of this function can last up to twice the specified timeout.
 *
 *  parameter
 *		phComm				serial port handle
 *		shBuffer			pointer to a buffer where this function stores the 
 *							read bytes
 *		ulBytesToRead		number of bytes to read from the serial port
 *		pulBytesRead		pointer where this function stores the read number 
 *							of bytes.
 *		pError				Error cluster
 */
void LVSERIAL_API ogCommRead(
	PHANDLE phComm, 
	LStrHandle shBuffer, 
	unsigned long ulBytesToRead, 
	unsigned long * pulBytesRead,
	pErrC pError)
{
	lvCommRead(*phComm, shBuffer, ulBytesToRead, pulBytesRead, pError);
}

/*  ogCommBreak
 *
 *  Sets the break state for the specified number of milliseconds
 *
 *  parameter
 *		phComm				serial port handle
 *		ulMilliseconds		break state duration
 *		pError				Error cluster
 */
void LVSERIAL_API ogCommBreak(PHANDLE phComm, unsigned long ulMilliseconds, pErrC pError)
{
	lvCommBreak(*phComm, ulMilliseconds, pError);
}

/*  ogCommBuffer
 *
 *  Sets the input and output queue size
 *
 *  parameter
 *		phComm				serial port handle
 *		ulInQueue			input queue size
 *		ulOutQueue			output queue size
 *		pError				Error cluster
 */
void LVSERIAL_API ogCommBuffer(PHANDLE phComm, unsigned long ulInQueueSize, unsigned long ulOutQueueSize, pErrC pError)
{
	lvCommBuffer(*phComm, ulInQueueSize, ulOutQueueSize, pError);
}

/*  ogCommFlush
 *
 *  The CommFlush function discards all characters from the output or input 
 *  buffer of a specified serial port. It can also terminate pending read 
 *  or write operations.
 *
 *  parameter
 *		phComm				serial port handle
 *		ulFlags				Specifies the action to take.
 *			COMM_FLUSH_TXABORT		Kill the pending/current writes to the comm port.
 *			COMM_FLUSH_RXABORT		Kill the pending/current reads to the comm port.
 *			COMM_FLUSH_TXCLEAR		Kill the transmit queue if there.
 *			COMM_FLUSH_RXCLEAR		Kill the typeahead buffer if there.
 *		pError				Error cluster
 */
void LVSERIAL_API ogCommFlush(PHANDLE phComm, unsigned long ulFlags, pErrC pError)
{
	lvCommFlush(*phComm, ulFlags, pError);
}

/*  ogCommStatus
 *
 *  The CommStatus function retrieves information about a communications pError 
 *  and reports the current status of a communications device. The function is 
 *  called when a communications pError occurs, and it clears the device's pError 
 *  flag to enable additional input and output (I/O) operations.
 *
 *  parameter
 *		phComm				serial port handle
 *		pulErrorFlags		pError flags (see COMM_ERRF_ )
 *		pulFlags			status flags (see COMM_STAT_ )
 *		pulInQueue			Number of bytes received by the serial provider but not 
 *							yet read by a CommRead operation.
 *		pulOutQueue			Number of bytes of user data remaining to be transmitted 
 *							for all write operations. 
 *		pError				Error cluster
 */
void LVSERIAL_API ogCommStatus(
	PHANDLE phComm, 
	unsigned long * pulErrorFlags, 
	unsigned long * pulStatusFlags,
	unsigned long * pulInQueue,
	unsigned long * pulOutQueue, 
	pErrC pError)
{
	lvCommStatus(
		*phComm,
		pulErrorFlags,
		pulStatusFlags,
		pulInQueue,
		pulOutQueue,
		pError);
}

/*  ogCommBytesAtPort
 *
 *  The function retrieves the number of bytes at the serial ports input queue.
 *
 *  parameter
 *		phComm				serial port handle
 *		pulBytesAtPort		Number of bytes received by the serial provider but not 
 *							yet read by a CommRead operation.
 *		pError				Error cluster
 */
void LVSERIAL_API ogCommBytesAtPort(PHANDLE phComm, unsigned long * pulBytesAtPort, pErrC pError)
{
	lvCommBytesAtPort(*phComm, pulBytesAtPort, pError);
}

/*  ogCommModemStatus
 *
 *  The CommModemStatus function retrieves modem control-register values.
 *
 *  parameter
 *		phComm				serial port handle
 *		pulModemStatus		Pointer to a variable that specifies the current state 
 *							of the modem control-register values. (see COMM_MSTAT_ )
 *		pError				Error cluster
 */
void LVSERIAL_API ogCommModemStatus(PHANDLE phComm, unsigned long * pulModemStatus, pErrC pError)
{
	lvCommModemStatus(*phComm, pulModemStatus, pError);
}

/*  ogCommGetCTS
 *
 *  The CommGetCTS function retrieves modem control-register value.
 *
 *  parameter
 *		phComm				serial port handle
 *		plfCTS				Pointer to a variable that specifies the current state 
 *							of the CTS signal.
 *		pError				Error cluster
 */
void LVSERIAL_API ogCommGetCTS(PHANDLE phComm, LVBoolean *plfCTS, pErrC pError)
{
	lvCommGetCTS(*phComm, plfCTS, pError);
}

/*  ogCommGetDSR
 *
 *  The CommGetDSR function retrieves modem control-register value.
 *
 *  parameter
 *		phComm				serial port handle
 *		plfDSR				Pointer to a variable that specifies the current state 
 *							of the DSR signal.
 *		pError				Error cluster
 */
void LVSERIAL_API ogCommGetDSR(PHANDLE phComm, LVBoolean *plfDSR, pErrC pError)
{
	lvCommGetDSR(*phComm, plfDSR, pError);
}

/*  ogCommGetRING
 *
 *  The CommGetRING function retrieves modem control-register value.
 *
 *  parameter
 *		phComm				serial port handle
 *		plfRING				Pointer to a variable that specifies the current state 
 *							of the RING signal.
 *		pError				Error cluster
 */
void LVSERIAL_API ogCommGetRING(PHANDLE phComm, LVBoolean *plfRING, pErrC pError)
{
	lvCommGetRING(*phComm, plfRING, pError);
}

/*  ogCommGetRLSD
 *
 *  The CommGetRLSD function retrieves modem control-register value.
 *
 *  parameter
 *		phComm				serial port handle
 *		plfRLSD				Pointer to a variable that specifies the current state 
 *							of the RLSD signal (receive-line-signal-detect).
 *		pError				Error cluster
 */
void LVSERIAL_API ogCommGetRLSD(PHANDLE phComm, LVBoolean *plfRLSD, pErrC pError)
{
	lvCommGetRLSD(*phComm, plfRLSD, pError);
}

/*  ogCommSetXOFF
 *
 *  Causes transmission to act as if an XOFF character has been received.
 *
 *  parameter
 *		phComm				serial port handle
 *		pError				Error cluster
 */
void LVSERIAL_API ogCommSetXOFF(PHANDLE phComm, pErrC pError)
{
	lvCommSetXOFF(*phComm, pError);
}

/*  ogCommSetXON
 *
 *  Causes transmission to act as if an XON character has been received.
 *
 *  parameter
 *		phComm				serial port handle
 *		pError				Error cluster
 */
void LVSERIAL_API ogCommSetXON(PHANDLE phComm, pErrC pError)
{
	lvCommSetXON(*phComm, pError);
}

/*  ogCommSetRTS
 *
 *  Sends the RTS (request-to-send) signal.
 *
 *  parameter
 *		phComm				serial port handle
 *		pError				Error cluster
 */
void LVSERIAL_API ogCommSetRTS(PHANDLE phComm, pErrC pError)
{
	lvCommSetRTS(*phComm, pError);
}

/*  ogCommClrRTS
 *
 *  Clears the RTS (request-to-send) signal.
 *
 *  parameter
 *		phComm				serial port handle
 *		pError				Error cluster
 */
void LVSERIAL_API ogCommClrRTS(PHANDLE phComm, pErrC pError)
{
	lvCommClrRTS(*phComm, pError);
}

/*  ogCommSetDTR
 *
 *  Sends the DTR (data-terminal-ready) signal.
 *
 *  parameter
 *		phComm				serial port handle
 *		pError				Error cluster
 */
void LVSERIAL_API ogCommSetDTR(PHANDLE phComm, pErrC pError)
{
	lvCommSetDTR(*phComm, pError);
}

/*  ogCommClrDTR
 *
 *  Clears the DTR (data-terminal-ready) signal.
 *
 *  parameter
 *		phComm				serial port handle
 *		pError				Error cluster
 */
void LVSERIAL_API ogCommClrDTR(PHANDLE phComm, pErrC pError)
{
	lvCommClrDTR(*phComm, pError);
}

/*  ogCommSetBreak
 *
 *  Suspends character transmission and places the transmission line in a break state 
 *  until the CommClearBreak function is called.
 *
 *  parameter
 *		phComm				serial port handle
 *		pError				Error cluster
 */
void LVSERIAL_API ogCommSetBreak(PHANDLE phComm, pErrC pError)
{
	lvCommSetBreak(*phComm, pError);
}

/*  ogCommClearBreak
 *
 *  Restores character transmission and places the transmission line in a nonbreak 
 *  state.
 *
 *  parameter
 *		phComm				serial port handle
 *		pError				Error cluster
 */
void LVSERIAL_API ogCommClearBreak(PHANDLE phComm, pErrC pError)
{
	lvCommClearBreak(*phComm, pError);
}

/*  ogCommConfig
 *
 *  Sets the basic serial port communication parameters.
 *
 *  parameter
 *		phComm				serial port handle
 *		ulBaudRate			baud rate
 *		usDataBits			number of data bits
 *		usParity			parity (see COMM_PARITY_ )
 *		usStopBits			number of stop bits (COMM_ONESTOPBIT, COMM_ONE5STOPBITS,
 *							COMM_TWOSTOPBITS or one od thze values 10, 15 or 20).
 *		pError				Error cluster
 */
void LVSERIAL_API ogCommConfig(
	PHANDLE phComm, 
	unsigned long ulBaudRate,
	unsigned short usDataBits,
	unsigned short usParity,
	unsigned short usStopBits, 
	pErrC pError)
{
	lvCommConfig(
		*phComm, 
		ulBaudRate,
		usDataBits,
		usParity,
		usStopBits,
		pError);
}

/*  ogCommFlowControl
 *
 *  Configures the basic flow control settings for the serial port.
 *
 *  parameter
 *		phComm				serial port handle
 *		ulFlowControl		flow control mode (see COMM_FLOW_)
 *		ucXOFFCharacter		XOFF character
 *		ucXONCharacter		XON character
 *		pError				Error cluster
 */
void LVSERIAL_API ogCommFlowControl(
	PHANDLE phComm,
	unsigned short usFlowControl,
	unsigned char ucXOFFCharacter,
	unsigned char ucXONCharacter, 
	pErrC pError)
{
	lvCommFlowControl(
		*phComm, 
		usFlowControl, 
		ucXOFFCharacter,
		ucXONCharacter,
		pError);
}

/*  ogCommControl
 *
 *  Extended flow and error handling control configuration
 *
 *  parameter
 *		phComm				serial port handle
 *		lFlags				specifies the handshake and error settings
 *							use the COMM_CTRL_ flags
 *		XonLim				Minimum number of bytes allowed in the input 
 *							buffer before flow control is activated to 
 *							inhibit the sender. Note that the sender may
 *							transmit characters after the flow control 
 *							signal has been activated, so this value should 
 *							never be zero.
 *		XoffLim				Maximum number of bytes allowed in the input 
 *							buffer before flow control is activated to 
 *							allow transmission by the sender.
 *		XonChar				Value of the XON character for both 
 *							transmission and reception.
 *		XoffChar			Value of the XOFF character for both 
 *							transmission and reception.
 *		ErrorChar			Value of the character used to replace bytes 
 *							received with a parity error. 
 *		pError				error cluster		
 */
void LVSERIAL_API ogCommControl(
	PHANDLE phComm,
	unsigned long ulFlags,
	unsigned short usXonLim,
	unsigned short usXoffLim,
	unsigned char ucXonChar,
	unsigned char ucXoffChar,
	unsigned char ucErrorChar,
	pErrC pError)
{
	lvCommControl(
		*phComm,
		ulFlags,
		usXonLim,
		usXoffLim,
		ucXonChar,
		ucXoffChar,
		ucErrorChar,
		pError);
}

/*  ogCommTermination
 *
 *  Configures the termiation characters for the serial port.
 *
 *  parameter
 *		phComm				serial port handle
 *		fTermCharEnable		specifies weather the termination characters are used
 *							to terminate any read operation.
 *		pcTermCharacter		buffer containing the termination characters
 *		lNumberOfTermChar	number of termination characters in pcTermCharacter
 *		pError				Error cluster
 */
void LVSERIAL_API ogCommTermination(
	PHANDLE phComm, 
	LVBoolean *plfTermCharEnable,
	LStrHandle shTermCharacter,
	pErrC pError)
{
	lvCommTermination( *phComm, plfTermCharEnable, shTermCharacter, pError);
}

/*  ogCommTermination2
 *
 *  Configures the termiation characters for the serial port.
 *
 *  parameter
 *		phComm				serial port handle
 *		lTerminationMode	specifies the termination mode, this can be one of the
 *							following value:
 *								COMM_TERM_NONE		no termination
 *								COMM_TERM_CHAR		one or more termination characters
 *								COMM_TERM_STRING	a single termination string
 *								COMM_TERM_REGEX		a regular expression
 *		pcTermination		buffer containing the termination characters or string
 *		lNumberOfTermChar	number of characters in pcTermination
 *		pError				Error cluster
 */
void LVSERIAL_API ogCommTermination2(
	PHANDLE phComm, 
	long lTerminationMode,
	LStrHandle shTermination,
	pErrC pError)
{
	lvCommTermination2(*phComm, lTerminationMode, shTermination, pError);
}

/*  ogCommWriteMode
 *
 *  Configures the termiation characters for the serial port.
 *
 *  parameter
 *		phComm					serial port handle
 *		lWriteMode				sprecifies the write mode
 *									COMM_WRITE_NORMAL
 *									COMM_WRITE_STX
 *									COMM_WRITE_ETX
 *									COMM_WRITE_STXETX
 *		pcStartOfTransmission	buffer containing the start of transmission string
 *		lNumberOfStartChars		number of characters in pcStartOfTransmission
 *		pcEndOfTransmission		buffer containing the end of transmission string
 *		lNumberOfEndChars		number of characters in pcStartOfTransmission
 *		pError					error cluster
 */
void LVSERIAL_API ogCommWriteMode(
	PHANDLE phComm, 
	long lWriteMode,
	LStrHandle shStartOfTransmission,
	LStrHandle shEndOfTransmission,
	pErrC pError)
{
	lvCommWriteMode(
		*phComm,
		lWriteMode,
		shStartOfTransmission,
		shEndOfTransmission,
		pError);
}

/*  ogCommTimeout
 *
 *  Configures the timeout value for read and write operations
 *
 *  parameter
 *		hComm				serial port handle
 *		ulTimeout			timeout in milliseconds
 *		pError				error cluster
 */
void LVSERIAL_API ogCommTimeout(
	PHANDLE phComm, 
	unsigned long ulTimeout, 
	pErrC pError)
{
	lvCommTimeout(*phComm, ulTimeout, pError);
}

/*  ogCommTimeout2
 *
 *  Configures the timeout value for read and write operations
 *
 *  parameter
 *		phComm							serial port handle
 *		ulReadIntervalTimeout			timeout in milliseconds between two characters
 *		ulReadTotalTimeoutMultiplier	timeout value for each requested character
 *		ulReadTotalTimeoutConstant		timeout constant
 *		ulWriteTotalTimeoutMultiplier	timeout value for each character to write
 *		ulWriteTotalTimeoutConstant		timeout constant
 *		pError							error cluster
 */
void LVSERIAL_API ogCommTimeout2(
	PHANDLE phComm, 
	unsigned long ulReadIntervalTimeout,
	unsigned long ulReadTotalTimeoutMultiplier,
	unsigned long ulReadTotalTimeoutConstant,
	unsigned long ulWriteTotalTimeoutMultiplier,
	unsigned long ulWriteTotalTimeoutConstant,
	pErrC pError)
{
	lvCommTimeout2(
		*phComm, 
		ulReadIntervalTimeout,
		ulReadTotalTimeoutMultiplier,
		ulReadTotalTimeoutConstant,
		ulWriteTotalTimeoutMultiplier,
		ulWriteTotalTimeoutConstant,
		pError);
}

/*  ogCommOptions
 *
 *  Sets the serial port options. This function is also called inside
 *  the CommOpen function and allows the user to specify the major 
 *	settings together with the serial port name in one string.
 *
 *  parameter
 *		phComm				serial port handle
 *		shOptions			serial port options string
 *		pError				error cluster
 *		
 *	Options specification
 *	There are two generic different formats of this string:
 *
 *	<baud>,<parity>,<data>,<stop>[,<handshake>]
 *
 *	where
 *		baud		any valid baudrate
 *		partity		N=none, O=odd, E=even, M=mark, S=space
 *		data		is the number of data bits
 *		stop		is the number of stop bits (1, 1.5 or 2)
 *					valid values are also 10, 15 and 20)
 *		handshake	either x (XON/XOFF) or p (DTR/DSR and RTS/CTS handshake)
 *
 *
 *	baudrate:<baud>; parity:<parity>; handshake:<hand>; 
 *  databits=<data>; stopbits:<stop>; timeout:<timeout>
 *
 *	where
 *		baud		any valid baudrate
 *		partity		N=none, O=odd, E=even, M=mark, S=space
 *					(only the first character is tested)
 *		handshake	N=none, X=xonxoff, R=rstcts, D=dsrdtr
 *					(only the first character is tested)
 *		data		is the number of data bits
 *		stop		is the number of stop bits (1, 1.5 or 2)
 *					valid values are also 10, 15 and 20)
 *		timeout		is the read and write timeout in milliseconds
 *		xonchar		specifies the XON character
 *		xoffchar	specifies the XOFF character
 *		xon			enables (on) or disables (off) the XON/XOFF handshake
 *		odsr		Indicates whether the DSR (data-set-ready) signal is 
 *					monitored for output flow control. Possible values 
 *					are on or off.
 *		octs		Indicates whether the CTS (clear-to-send) signal is 
 *					monitored for output flow control. Possible values
 *					are on or off.
 *		dtr			sets the DTR flow control mode to on, off or hs (=handshake)
 *		rts			sets the RTS flow control mode to on, off, hs (=handshake) 
 *					or ts (=toggle)
 *		idsr		this value can be on or off. If it is on, the driver
 *					is sensitive to the DSR signal. The driver only reveives
 *					bytes, if the DSR input line is high.
 *		
 *
 *	String and character compirison is not case sensitive. The second
 *	format is less strict, so you can place whitespaces between the 
 *	keywords and values. You can use "=" instead of ":" and you must't
 *	specify ";" between the values (a single whitespace character is 
 *	enough).
 */
void LVSERIAL_API ogCommOptions(PHANDLE phComm, LStrHandle shOptions, pErrC pError)
{
	lvCommOptions(*phComm, shOptions, pError);
}

/*  ogCommLock
 *
 *  Locks the serial port
 *
 *  parameter
 *		phComm				serial port handle
 *		ulTimeout			timeout in milliseconds
 *		pError				Error cluster
 */
void LVSERIAL_API ogCommLock(PHANDLE phComm, unsigned long ulTimeout, pErrC pError)
{
	lvCommLock(*phComm, ulTimeout, pError);
}

/*  ogCommUnLock
 *
 *  Unlocks the serial port
 *
 *  parameter
 *		phComm				serial port handle
 *		pError				Error cluster
 */
void LVSERIAL_API ogCommUnLock(PHANDLE phComm, pErrC pError)
{
	lvCommUnLock(*phComm, pError);
}

/*  ogCommSetUserData
 *
 *  Sets the user specific data for the serial port
 *
 *  parameter
 *		phComm				serial port handle
 *		ulUserData			user data
 *		pError				Error cluster
 */
void LVSERIAL_API ogCommSetUserData(PHANDLE phComm, unsigned long ulUserData, pErrC pError)
{
	lvCommSetUserData(*phComm, ulUserData, pError);
}

/*  ogCommGetUserData
 *
 *  Returns the user specific data
 *
 *  parameter
 *		phComm				serial port handle
 *		pulUserData			pointer to the location where thsi function stores
 *							the user data.
 *		pError				Error cluster
 */
void LVSERIAL_API ogCommGetUserData(PHANDLE phComm, unsigned long *pulUserData, pErrC pError)
{
	lvCommGetUserData(*phComm, pulUserData, pError);
}

/*  ogCommSetUserDataBuffer
 *
 *  Sets the user specific data for the serial port
 *
 *  parameter
 *		phComm				serial port handle
 *		shUserData			user data
 *		pError				Error cluster
 */
void LVSERIAL_API ogCommSetUserDataBuffer(PHANDLE phComm, LStrHandle shUserData, pErrC pError)
{
	lvCommSetUserDataBuffer(*phComm, shUserData, pError);
}

/*  ogCommGetUserDataBuffer
 *
 *  Returns the user specific data
 *
 *  parameter
 *		phComm				serial port handle
 *		shUserData			user data
 *		pError				Error cluster
 */
void LVSERIAL_API ogCommGetUserDataBuffer(PHANDLE phComm, LStrHandle shUserData, pErrC pError)
{
	lvCommGetUserDataBuffer(*phComm, shUserData, pError);
}
   
/*  ogCommListPorts
 *
 *  Lists all serial ports and places them into a array of pointers to zero
 *  terminated strings. The array and the strings are allocated by this
 *  function and should. Call CommFree to release the memory.
 *
 *  parameter
 *		halsPorts			handle to the array of strings (port names)
 *		pError				error cluster
 *		
 *  return
 *		error code
 */
void LVSERIAL_API ogCommListPorts(LStrArrayHandle *phalsPorts, pErrC pError)
{
	lvCommListPorts(phalsPorts, pError);
}

/*  ogCommGetHandles
 *
 *	This function returns a list of all existing serial port handles.
 *
 *  parameter
 *		haulHandles			array handle where this function stores the
 *							handles
 *		pErrC				error cluster
 */
void LVSERIAL_API ogCommGetHandles(ULongArrayHandle hapulHandles, pErrC pError)
{
	MgErr err;							//labview function return code
	long lSize;
	long i,j;								//index number
	HANDLE hH;
	PHANDLE phH;

	//lvCommGetHandles(phaulHandles, pError);

	_ASSERT(!DSCheckPtr(pError));

	//do nothing if there is an incoming pError
	if (pError->lfStatus == LVTRUE)
		return;

	_ASSERT(!DSCheckPtr(hapulHandles));

	lSize = ListGetLength(gpCommList);

	//adjust array size or allocate new array
	if ((err = NumericArrayResize( uL, 1L, (UHandle *)&hapulHandles, lSize)) != noErr)
	{
		(*hapulHandles)->dimSize = 0;
		lv_EasyErrBld(err, __FUNCTION__, pError);
		return;
	}

	(void *)(*hapulHandles)->elt[0] = NULL;

	if ((phH = (PHANDLE)DSNewPtr(sizeof(PHANDLE))) == NULL)
	{
		lv_EasyErrBld(mFullErr, __FUNCTION__, pError);
		(*hapulHandles)->dimSize = 0;
		return;
	}

	*phH = hH = ListGetFirstHandle(gpCommList);
	(*hapulHandles)->elt[0] = (uInt32)*phH;
	
	for (i = 1; i < lSize; i++)
	{
		//allocate memory for the next handle
		if ((phH = (PHANDLE)DSNewPtr(sizeof(PHANDLE))) == NULL)
		{
			//memory allocation error
			//dispose all previously allocated pointers
			for (j = i-1; j >= 0; j--)
				DSDisposePtr((UPtr)((*hapulHandles)->elt[j]));
			
			//set the dimension size to zero
			(*hapulHandles)->dimSize = 0;
			
			//update the error cluster
			lv_EasyErrBld(mFullErr, __FUNCTION__, pError);
			return;
		}

		//get the next handle and copy it into the arry
		*phH = hH = ListGetNextHandle(gpCommList, hH);
		(*hapulHandles)->elt[i] = (uInt32)*phH;
	}

	//update the array size
	(*hapulHandles)->dimSize = lSize;
}

/*  ogCommGetPortName
 *
 *	This function returns the real port name and or the original port name, 
 *	which was used when CommOpenPort was called.
 *
 *  parameter
 *		phComm				serial port handle
 *		shName				string where thsi function stores
 *							the real port name
 *		shOriginal			string where this function stores
 *							the original port name
 *		pError				error cluster
 */
void LVSERIAL_API ogCommGetPortName(PHANDLE phComm, LStrHandle shName, LStrHandle shOriginal, pErrC pError)
{
	lvCommGetPortName(*phComm, shName, shOriginal, pError);
}

/*  ogCommEnableEvent
 *
 *	This function specifies a set of events to be monitored for a 
 *	communications device. 
 *
 *  parameter
 *		phComm				serial port handle
 *		ulEventMask			event mask
 *								COMM_EV_BREAK	A break was detected on input. 
 *								COMM_EV_CTS		The CTS (clear-to-send) signal 
 *												changed state. 
 *								COMM_EV_DSR		The DSR (data-set-ready) signal 
 *												changed state. 
 *								COMM_EV_ERR		A line-status error occurred. Line-
 *												status errors are CE_FRAME, 
 *												CE_OVERRUN and CE_RXPARITY. 
 *								COMM_EV_RING	A ring indicator was detected. 
 *								COMM_EV_RLSD	The RLSD (receive-line-signal-detect)
 *												signal changed state. 
 *								COMM_EV_RXCHAR	A character was received and placed 
 *												in the input buffer. 
 *								COMM_EV_RXFLAG	The event character was received and 
 *												placed in the input buffer. The event 
 *												character is specified in the device's DCB structure, which is applied to a serial port by using the SetCommState function. 
 *								COMM_EV_TXEMPTY	The last character in the output 
 *												buffer was sent. 
 *		pError				error cluster
 */
void LVSERIAL_API ogCommEnableEvent(PHANDLE phComm, unsigned long ulEventMask, pErrC pError)
{
	lvCommEnableEvent(*phComm, ulEventMask, pError);
}

/*  ogCommWaitForEvent
 *
 *	This function waits for an event to occur for a specified communications device. 
 *	The set of events that are monitored by this function is contained in the event 
 *	mask associated with the device handle. 
 *
 *  parameter
 *		phComm				serial port handle
 *		ulTimeout			Specifies the time-out interval, in milliseconds.
 *		pulEventMask		Pointer to a variable that receives a mask indicating 
 *							the type of event that occurred. If an error occurs, 
 *							the value is zero; otherwise, it is one of the COMM_EV_
 *							values (see CommSetEventMask).
 *		pError				error cluster
 */
void LVSERIAL_API ogCommWaitForEvent(PHANDLE phComm, unsigned long ulTimeout, unsigned long *pulEventMask, pErrC pError)
{
	lvCommWaitForEvent(*phComm, ulTimeout, pulEventMask, pError);
}

/*  ogCommWaitForEvent
 *
 *	This function disables the monitoring of communication events.
 *
 *  parameter
 *		phComm				serial port handle
 *		pError				error cluster
 */
void LVSERIAL_API ogCommDisableEvent(PHANDLE phComm, pErrC pError)
{
	lvCommDisableEvent(*phComm, pError);
}

/*  ogCommSetEventChar
 *
 *	This function sets the event character.
 *
 *  parameter
 *		phComm				serial port handle
 *		shEventChar			event character (only the first character is used)
 *		
 *  return
 *		error code
 */
void LVSERIAL_API ogCommSetEventChar(PHANDLE phComm, unsigned char ucEventChar, pErrC pError)
{
	lvCommSetEventChar(*phComm, ucEventChar, pError);
}

/*  ogCommTransmitChar
 *
 *	The lvCommTransmitChar function transmits a specified character ahead of 
 *	any pending data in the output buffer of the specified communications 
 *	device.
 *
 *  parameter
 *		phComm				serial port handle
 *		cChar				character to be transmitted
 *		pError				error cluster
 */
void LVSERIAL_API ogCommTransmitChar(PHANDLE phComm, unsigned char ucChar, pErrC pError)
{
	lvCommTransmitChar(*phComm, ucChar, pError);
}

/*  ogCommSetDefaultConfig
 *
 *	The ogCommSetDefaultConfig function sets the default configuration for 
 *  a communications device.
 *
 *  parameter
 *		shPort				serial port name
 *		shOptions			options string (see mode command)
 *		
 *  return
 *		error code
 */
void LVSERIAL_API ogCommSetDefaultConfig(LStrHandle shPort, LStrHandle shOptions, pErrC pError)
{
	lvCommSetDefaultConfig(shPort, shOptions, pError);
}
